// ALL3500 Library
// 1.00 - 2017-10-20
// 1.01 - 2017-10-29
// 1.02 - 2017-11-16
// (c) Copyright 2017
//
// Zur Verwendung mit allen Allnet Libraries


// Beispiel für den ersten Sensor, der übergeben wird:
//
// L...!....1....!....2....!....3....!....4....!....5....!....6....!....7....!....8....!....9....!....0....!....1....!
// [{"id":"1","name":"Interner Sensor","description":"Temperatursensor","fe_view":"1","sort":"2:2",
// "fe_cvs_show_typ":"1","actor_analogValue":null,"digitalToText":"0::","tileColors":"1e7eac:900000:900000",
// "tileFormats":"55:","lang_port_identifier":null,"fading":null,"device_type":"4","value":"36.00","error":0,
// "config":{"icon":"","display":{"min":"0","max":"60"},"limit":{"min":"5","max":"50"}},"info":{"activ":"1",
// "enabled":"1","unit":"\u00b0C","type":"1","view":"1","chipid":"2","chipnumber":"2","chipaddress":"3",
// "helperchipnumber":"0","helperchipaddress":"0","bitaddress":"0"},"minmax":{"today":{"min":"35.50","max":"37.12"},
// "absolute":{"min":"19.62","max":"255.93"}},"connection":{"port":"4","bus":"67","group":"0"}},


// *** Globale Variablen ***

byte          all3500_SensorCount = 0;                          // Aktuelle Anzahl an Sensoren im ALL3500
String        all3500_SensorName  [32];                         // Maximal 32 Sensoren werden ausgelesen (max. ca. 2 KB)
String        all3500_SensorInfo  [32];                         // Maximal 32 Sensoren werden ausgelesen
double        all3500_SensorValue [32];                         // Maximal 32 Sensoren werden ausgelesen
String        all3500_SensorUnit  [32];                         // Maximal 32 Sensoren werden ausgelesen
int           all3500_SensorChipID[32];                         // Maximal 32 Sensoren werden ausgelesen


// *** ALL3500 Funktionen ***

String all3500_SensorStatus (byte index)
{
  String s = strform(index, 48, 2, false);
  s += " # ";
  s += hexbyte(all3500_SensorChipID[index]);
  s += " : ";
  s += left(all3500_SensorName[index] + spc(25), 25);
  s += left(all3500_SensorInfo[index] + spc(25), 25);
  s += strrealform (all3500_SensorValue[index], 32, 3, 2, false, false);
  s += " ";
  s += all3500_SensorUnit[index];
  return s;
}

void all3500_ReadSensorData (String all_url, bool message)
{
  all3500_SensorCount = 0;
  for (int i = 0; i < 32; i++)
  {
    all3500_SensorName  [i] = "";
    all3500_SensorInfo  [i] = "";
    all3500_SensorValue [i] = 0.0;
    all3500_SensorUnit  [i] = "";
    all3500_SensorChipID[i] = 0;
  }
  HTTPClient http;
  http.begin(all_url); //HTTP
  int httpCode = http.GET();  // start connection and send HTTP header
  if (message)
  {
    Serial.print("all3500_ReadSensors: HTTP GET Result: " + str(httpCode) + ", Length = ");
  }
  if (httpCode > 0) // httpCode will be negative on error
  {
    String result = http.getString();
    int L = len(result);
    int p = 1;
    int n = 1;
    if (message)
    {
      Serial.println(str(L));
      Serial.println("");
      Serial.println(fillcenter(" " + all_url + " ", "-", 80) + chr(8));
    }
    int count = 0;
    while ((p > 0) && (n > 0))
    {
      p = position(result, quotate("name"), p);
      if (p > 0)
      {
        n = position(result, chr(34), p + 8);
        if (n > 0)
        {
          L = n - p - 8;
          String SensorName = convertescape(mid(result, p + 8, L));
          p = position(result, quotate("description"), n);
          if (p > 0)
          {
            n = position(result, chr(34), p + 15);
            if (n > 0)
            {
              L = n - p - 15;
              String SensorInfo = convertescape(mid(result, p + 15, L));
              p = position(result, quotate("value"), n);
              if (p > 0)
              {
                n = position(result, chr(34), p + 9);
                if (n > 0)
                {
                  L = n - p - 9;
                  double SensorValue = realval(convertescape(mid(result, p + 9, L)));
                  p = position(result, quotate("unit"), n);
                  if (p > 0)
                  {
                    n = position(result, chr(34), p + 8);
                    if (n > 0)
                    {
                      L = n - p - 8;
                      String SensorUnit = convertescape(mid(result, p + 8, L));
                      p = position(result, quotate("chipid"), n);
                      if (p > 0)
                      {
                        n = position(result, chr(34), p + 10);
                        if (n > 0)
                        {
                          L = n - p - 10;
                          int ChipID = val(convertescape(mid(result, p + 10, L)));
                          all3500_SensorName  [count] = SensorName;
                          all3500_SensorInfo  [count] = SensorInfo;
                          all3500_SensorValue [count] = SensorValue;
                          all3500_SensorUnit  [count] = SensorUnit;
                          all3500_SensorChipID[count] = ChipID;
                          if (message)
                          {
                            Serial.println(all3500_SensorStatus(count));
                          }
                          count += 1;
                          all3500_SensorCount = count;
                          p = position(result, "," + quotate("group") + ":", p);
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
    Serial.println(repeatstr("-", 80) + chr(8));
  }
  else
  {
    Serial.println("0, Error Message: " + http.errorToString(httpCode));
  }
  http.end();
}