// BRK Clock Library
// 1.00 - 2017-06-04
// 1.01 - 2017-06-16
// 1.02 - 2017-06-23
// 1.03 - 2017-06-30 - English Language
// 1.04 - 2017-11-15
// (c) Copyright 2017
//
// Zur Verwendung mit dem Allnet IoT-Brick

#include <Adafruit_GFX.h>
#include <Adafruit_NeoMatrix.h>
#include <Adafruit_NeoPixel.h>

#include <all_Sound.h>

#define LED 64  // Anzahl LEDS  

//  Aufbau der 8 x 8 LED-Brick-Matrix:

//  F(63) Ü(62) N(61) F(60) Z(59) E(58) H(57) N(56)

//  V(48) O(49) R(50) p(51) N(52) A(53) C(54) H(55)

//  H(47) A(46) L(45) B(44) V(43) I(42) E(41) R(40)

//  E(32) I(33) N(34) S(35) E(36) C(37) H(38) S(39)
    
//  S(31) I(30) E(29) Z(28) W(27) Ö(26) L(25) F(24)
    
//  B(16) E(17) N(18) D(19) R(20) E(21) I(22) Ü(23)
    
//  R(15) Z(14) E(13) H(12) N(11) E(10) U(09) N(08)
    
//  K(00) A(01) C(02) H(03) T(04) E(05) L(06) F(07)


// *** Globale Typen ***

struct BRKclock_Parameter
{
//    32 Bytes Parameter-Header:
    int64 parameterID       = 0;                                  // Kennung des Parameter-Blocks "BRKClock" im Klartext
    int64 parameterHash     = 0;                                  // Kennung des Parameter-Blocks "BRKClock" als verschlüsselter Hash
    int32 parameterSize     = 0;                                  // Anzahl Bytes im Parameter Block
    int32 parameterReserve  = 0;                                  // Platzhalter für weiteren Header-Eintrag
    int32 parameterVersion  = 0;                                  // Version des Parameter-Blocks
    int32 parameterCount    = 0;                                  // Anzahl folgenden Byte-Parameter im Block
    
//    Start des Parameter-Blocks:
    byte  clockColorRed     = 0;                                  // RGBW-Farbe der Uhrzeitanzeige
    byte  clockColorGreen   = 0;
    byte  clockColorBlue    = 0;
    byte  clockColorWhite   = 0;
    byte  tempColorRed      = 0;                                  // RGBW-Farbe der Temperaturanzeige
    byte  tempColorGreen    = 0;
    byte  tempColorBlue     = 0;
    byte  tempColorWhite    = 0;
    byte  IP_address0       = 0;                                  // 4 Bytes IP-Adresse
    byte  IP_address1       = 0;
    byte  IP_address2       = 0;
    byte  IP_address3       = 0;
    byte  IP_gateway0       = 0;                                  // 4 Bytes Gateway
    byte  IP_gateway1       = 0;
    byte  IP_gateway2       = 0;
    byte  IP_gateway3       = 0;
    byte  IP_subnet0        = 0;                                  // 4 Bytes Subnetzmaske
    byte  IP_subnet1        = 0;
    byte  IP_subnet2        = 0;
    byte  IP_subnet3        = 0;
    byte  staticIP          = 0;                                  // 0 = Dynamische IP-Adresse, 1 = Statische IP-Adresse
    byte  alarmMode         = 0;                                  // 0 = Wecker aus,    1 = Wecker ein
    byte  temperatureMode   = 0;                                  // 0 = Temperaturanzeige aus,   1 = Temperaturanzeige jede Minute
    byte  colorMode         = 0;                                  // 0 = Random Schrift, 1 = Weiße Schrift, 2 = Rote Schrift, usw.
    byte  buttonMode        = 0;                                  // 0 = IP-Adresse anzeigen, 1 = Temperatur anzeigen, usw.
    byte  alarmHour         = 0;                                  // Weckzeit: Stunden
    byte  alarmMinute       = 0;                                  // Weckzeit: Minuten
    byte  language          = 0;                                  // Sprache: 0 = Deutsch, 1 = Englisch
  double  temperatureOffset = 0.0;                                // Temperatur-Offset +/-
};


// *** Globale Konstanten ***

#if !defined(BRKclock_PIN)                                        // Wenn BRKclock_PIN nicht definiert wurde
  #define BRKclock_PIN 14                                         // ESP8266 = 14, ESP32 = 18
#endif


// *** Globale Variablen ***

BRKclock_Parameter BRKclock_ParameterData;

Adafruit_NeoPixel  BRKclock_Pixel  = Adafruit_NeoPixel(LED, BRKclock_PIN, NEO_RGBW + NEO_KHZ800);

Adafruit_NeoMatrix BRKclock_Matrix = Adafruit_NeoMatrix(8, 8, BRKclock_PIN,
                                                        NEO_MATRIX_BOTTOM + NEO_MATRIX_LEFT +
                                                        NEO_MATRIX_ROWS + NEO_MATRIX_ZIGZAG,
                                                        NEO_GRBW + NEO_KHZ800);
// *** Farbe der Uhrzeit-Anzeige ***

uint32_t BRKclock_clockColor = BRKclock_Pixel.Color(0x00, 0xFF, 0x00, 0); // GRBW (grün, rot, blau, weiß), 32 Bit
byte     BRKclock_clockColorRed       = 0xFF;                             // Farbanteil für Rot-Kanal
byte     BRKclock_clockColorGreen     = 0x00;                             // Farbanteil für Grün-Kanal
byte     BRKclock_clockColorBlue      = 0x00;                             // Farbanteil für Blau-Kanal
byte     BRKclock_clockColorWhite     = 0x00;                             // Farbanteil für Weiß-Kanal


// *** Farbe der Temperatur-Anzeige ***

uint32_t BRKclock_tempColor  = BRKclock_Pixel.Color(0xFF, 0x00, 0x00, 0); // RGBW (rot, grün, blau, weiß), 32 Bit
byte     BRKclock_tempColorRed        = 0xFF;                             // Farbanteil für Rot-Kanal
byte     BRKclock_tempColorGreen      = 0x00;                             // Farbanteil für Grün-Kanal
byte     BRKclock_tempColorBlue       = 0x00;                             // Farbanteil für Blau-Kanal
byte     BRKclock_tempColorWhite      = 0x00;                             // Farbanteil für Weiß-Kanal


// *** Benutzerdefinierte Einstellungen ***

byte     BRKclock_staticIP            = 0;                                // 0 = Dynamische IP-Adresse, 1 = Statische IP-Adresse
int      BRKclock_alarmMode           = 0;                                // 0 = Wecker aus, 1 = Wecker ein
int      BRKclock_temperatureMode     = 1;                                // 0 = Temperaturanzeige aus, 1 = Temperaturanzeige jede Minute
int      BRKclock_colorMode           = 2;                                // 0 = Random Schrift, 1 = Weiße Schrift, 2 = Rote Schrift, usw.
int      BRKclock_buttonMode          = 0;                                // 0 = IP-Adresse anzeigen, 1 = Temperatur anzeigen, usw.
byte     BRKclock_alarmHour           = 0;                                // Weckzeit: Stunden
byte     BRKclock_alarmMinute         = 0;                                // Weckzeit: Minuten
byte     BRKclock_language            = 0;                                // Sprache: 0 = Deutsch, 1 = Englisch
double   BRKclock_temperatureOffset   = 0.0;                              // Temperatur-Offset +/-


// *** Globale Konstanten für spezielle 16 Bit Farbtabellen im RGB-Modus ***

const uint16_t BRKclock_Matrix16Color [ ] = {BRKclock_Matrix.Color(0x00, 0x00, 0x00),  //  0 = Schwarz
                                             BRKclock_Matrix.Color(0xFF, 0x00, 0x00),  //  1 = Rot
                                             BRKclock_Matrix.Color(0x00, 0x00, 0x80),  //  2 = Dunkelblau
                                             BRKclock_Matrix.Color(0xFF, 0x00, 0xFF),  //  3 = Purpur (Magenta)
                                             BRKclock_Matrix.Color(0x00, 0x80, 0x00),  //  4 = Dunkelgrün
                                             BRKclock_Matrix.Color(0x54, 0x54, 0x54),  //  5 = Dunkelgrau
                                             BRKclock_Matrix.Color(0x00, 0x00, 0xFF),  //  6 = Blau
                                             BRKclock_Matrix.Color(0x00, 0x80, 0xFF),  //  7 = Hellblau
                                             BRKclock_Matrix.Color(0x80, 0x40, 0x00),  //  8 = Braun
                                             BRKclock_Matrix.Color(0xFF, 0x80, 0x00),  //  9 = Orange
                                             BRKclock_Matrix.Color(0xA8, 0xA8, 0xA8),  // 10 = Hellgrau
                                             BRKclock_Matrix.Color(0xFF, 0x80, 0x80),  // 11 = Pink
                                             BRKclock_Matrix.Color(0x00, 0xFF, 0x00),  // 12 = Grün
                                             BRKclock_Matrix.Color(0xFF, 0xFF, 0x00),  // 13 = Gelb
                                             BRKclock_Matrix.Color(0x00, 0xFF, 0xFF),  // 14 = Türkis (Cyan)
                                             BRKclock_Matrix.Color(0xFF, 0xFF, 0xFF)}; // 15 = Weiß

const uint16_t BRKclock_Matrix8Color  [ ] = {BRKclock_Matrix.Color(0x00, 0x00, 0x00),  //  0 = Schwarz
                                             BRKclock_Matrix.Color(0xFF, 0x00, 0x00),  //  1 = Rot
                                             BRKclock_Matrix.Color(0xFF, 0x00, 0xFF),  //  2 = Purpur (Magenta)
                                             BRKclock_Matrix.Color(0x00, 0x00, 0xFF),  //  3 = Blau
                                             BRKclock_Matrix.Color(0x00, 0xFF, 0x00),  //  4 = Grün
                                             BRKclock_Matrix.Color(0xFF, 0xFF, 0x00),  //  5 = Gelb
                                             BRKclock_Matrix.Color(0x00, 0xFF, 0xFF),  //  6 = Türkis (Cyan)
                                             BRKclock_Matrix.Color(0xFF, 0xFF, 0xFF)}; //  7 = Weiß


// *** Globale Konstante für Gamma-Wert (Helligkeitskorrektur) ***

const byte BRKclock_Gamma             [ ] = {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                                               0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  1,  1,
                                               1,  1,  1,  1,  1,  1,  1,  1,  1,  2,  2,  2,  2,  2,  2,  2,
                                               2,  3,  3,  3,  3,  3,  3,  3,  4,  4,  4,  4,  4,  5,  5,  5,
                                               5,  6,  6,  6,  6,  7,  7,  7,  7,  8,  8,  8,  9,  9,  9, 10,
                                              10, 10, 11, 11, 11, 12, 12, 13, 13, 13, 14, 14, 15, 15, 16, 16,
                                              17, 17, 18, 18, 19, 19, 20, 20, 21, 21, 22, 22, 23, 24, 24, 25,
                                              25, 26, 27, 27, 28, 29, 29, 30, 31, 32, 32, 33, 34, 35, 35, 36,
                                              37, 38, 39, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 50,
                                              51, 52, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 66, 67, 68,
                                              69, 70, 72, 73, 74, 75, 77, 78, 79, 81, 82, 83, 85, 86, 87, 89,
                                              90, 92, 93, 95, 96, 98, 99,101,102,104,105,107,109,110,112,114,
                                             115,117,119,120,122,124,126,127,129,131,133,135,137,138,140,142,
                                             144,146,148,150,152,154,156,158,160,162,164,167,169,171,173,175,
                                             177,180,182,184,186,189,191,193,196,198,200,203,205,208,210,213,
                                             215,218,220,223,225,228,231,233,236,239,241,244,247,249,252,255  };


// *** BRK-Clock Grundlagen ***

void BRKclock_SetTemperatureRGBWColor (byte red, byte green, byte blue, byte white)
{
  // Serial.println("Set Temp. Color: R = " + hexbyte(red) + ", G = " + hexbyte(green) + ", B = " + hexbyte(blue) + ", W = " + hexbyte(white));
  BRKclock_tempColorRed   = red;
  BRKclock_tempColorGreen = green;
  BRKclock_tempColorBlue  = blue;
  BRKclock_tempColorWhite = white;
  if (red != 0)
  {
    const byte min = 24;
    red   = byte(((double)red   / 255.0) * (255.0 - (double)min)) + min;    // Rote LED zündet erst bei 24
  }
  if (green != 0)
  {
    const byte min = 12;
    green = byte(((double)green / 255.0) * (255.0 - (double)min)) + min;    // Grüne LED zündet erst bei 12
  }
  if (blue != 0)
  {
    const byte min = 24;
    blue  = byte(((double)blue  / 255.0) * (255.0 - (double)min)) + min;    // Blaue LED zündet erst bei 24
  }
  if (white != 0)
  {
    const byte min = 3;
    white = byte(((double)white / 255.0) * (255.0 - (double)min)) + min;    // Weiße LED zündet erst bei 3
  }
  BRKclock_tempColor = BRKclock_Pixel.Color(red, green, blue, white);       // (rot, grün, blau, weiß)
  BRKclock_Matrix.setPassThruColor(BRKclock_Pixel.Color(red, green, blue, white)); // In den RGBW-Modus wechseln
}

void BRKclock_SetTemperatureRGBColor (byte red, byte green, byte blue)
{
  // Serial.println("Set Temp. Color: R = " + hexbyte(red) + ", G = " + hexbyte(green) + ", B = " + hexbyte(blue));
  BRKclock_tempColorRed   = red;
  BRKclock_tempColorGreen = green;
  BRKclock_tempColorBlue  = blue;
  if (red != 0)
  {
    red   = byte(((double)red   / 255.0) * 231.0) + 24;         // Rote LED zündet erst bei 24
  }
  if (green != 0)
  {
    green = byte(((double)green / 255.0) * 243.0) + 12;         // Grüne LED zündet erst bei 12
  }
  if (blue != 0)
  {
    blue  = byte(((double)blue  / 255.0) * 231.0) + 24;         // Blaue LED zündet erst bei 24
  }
  BRKclock_tempColor = BRKclock_Matrix.Color(red, green, blue); // (rot, grün, blau)
  BRKclock_Matrix.setPassThruColor();                           // In den RGB-Modus wechseln
}

void BRKclock_SetClockRGBWColor (byte red, byte green, byte blue, byte white)
{
  // Serial.println("Set Clock Color: R = " + hexbyte(red) + ", G = " + hexbyte(green) + ", B = " + hexbyte(blue) + ", W = " + hexbyte(white));
  BRKclock_clockColorRed   = red;
  BRKclock_clockColorGreen = green;
  BRKclock_clockColorBlue  = blue;
  BRKclock_clockColorWhite = white;
  BRKclock_clockColor = BRKclock_Pixel.Color(green, red, blue, white); // (grün, rot, blau, weiß)
}

void BRKclock_SetClock8Color (byte c)
{
  switch (c)
  {
    case 0: // Schwarz
      BRKclock_SetClockRGBWColor(0x00, 0x00, 0x00, 0);
      break;
    case 1: // Rot
      BRKclock_SetClockRGBWColor(0xFF, 0x00, 0x00, 0);
      break;
    case 2: // Purpur (Magenta)
      BRKclock_SetClockRGBWColor(0xFF, 0x00, 0xFF, 0);
      break;
    case 3: // Blau
      BRKclock_SetClockRGBWColor(0x00, 0x00, 0xFF, 0);
     break;
    case 4: // Grün
      BRKclock_SetClockRGBWColor(0x00, 0xFF, 0x00, 0);
      break;
    case 5: // Gelb
      BRKclock_SetClockRGBWColor(0xFF, 0xFF, 0x00, 0);
      break;
    case 6: // Türkis (Cyan)
      BRKclock_SetClockRGBWColor(0x00, 0xFF, 0xFF, 0);
      break;
    case 7: // Weiß
      BRKclock_SetClockRGBWColor(0xFF, 0xFF, 0xFF, 0);
      break;
    default:
      break;
  }
}

void BRKclock_SetTemperature8Color (byte c)
{
  switch (c)
  {
    case 0: // Schwarz
      BRKclock_SetTemperatureRGBWColor(0x00, 0x00, 0x00, 0);
      break;
    case 1: // Rot
      BRKclock_SetTemperatureRGBWColor(0xFF, 0x00, 0x00, 0);
      break;
    case 2: // Purpur (Magenta)
      BRKclock_SetTemperatureRGBWColor(0xFF, 0x00, 0xFF, 0);
      break;
    case 3: // Blau
      BRKclock_SetTemperatureRGBWColor(0x00, 0x00, 0xFF, 0);
     break;
    case 4: // Grün
      BRKclock_SetTemperatureRGBWColor(0x00, 0xFF, 0x00, 0);
      break;
    case 5: // Gelb
      BRKclock_SetTemperatureRGBWColor(0xFF, 0xFF, 0x00, 0);
      break;
    case 6: // Türkis (Cyan)
      BRKclock_SetTemperatureRGBWColor(0x00, 0xFF, 0xFF, 0);
      break;
    case 7: // Weiß
      BRKclock_SetTemperatureRGBWColor(0xFF, 0xFF, 0xFF, 0);
      break;
    default:
      break;
  }
}

void BRKclock_SetClock16Color (byte c)
{
  switch (c)
  {
    case 0: // Schwarz
      BRKclock_SetClockRGBWColor(0x00, 0x00, 0x00, 0);
      break;
    case 1: // Rot
      BRKclock_SetClockRGBWColor(0xFF, 0x00, 0x00, 0);
      break;
    case 2: // Dunkelblau
      BRKclock_SetClockRGBWColor(0x00, 0x00, 0x80, 0);
      break;
    case 3: // Purpur (Magenta)
      BRKclock_SetClockRGBWColor(0xFF, 0x00, 0xFF, 0);
      break;
    case 4: // Dunkelgrün
      BRKclock_SetClockRGBWColor(0x00, 0x80, 0x00, 0);
      break;
    case 5: // Dunkelgrau
      BRKclock_SetClockRGBWColor(0x54, 0x54, 0x54, 0);
      break;
    case 6: // Blau
      BRKclock_SetClockRGBWColor(0x00, 0x00, 0xFF, 0);
     break;
    case 7: // Hellblau
      BRKclock_SetClockRGBWColor(0x00, 0x80, 0xFF, 0);
     break;
    case 8: // Braun
      BRKclock_SetClockRGBWColor(0x80, 0x40, 0x00, 0);
     break;
    case 9: // Orange
      BRKclock_SetClockRGBWColor(0xFF, 0x80, 0x00, 0);
     break;
    case 10: // Hellgrau
      BRKclock_SetClockRGBWColor(0xA8, 0xA8, 0xA8, 0);
     break;
    case 11: // Pink
      BRKclock_SetClockRGBWColor(0xFF, 0x80, 0x80, 0);
     break;
    case 12: // Grün
      BRKclock_SetClockRGBWColor(0x00, 0xFF, 0x00, 0);
      break;
    case 13: // Gelb
      BRKclock_SetClockRGBWColor(0xFF, 0xFF, 0x00, 0);
      break;
    case 14: // Türkis (Cyan)
      BRKclock_SetClockRGBWColor(0x00, 0xFF, 0xFF, 0);
      break;
    case 15: // Weiß
      BRKclock_SetClockRGBWColor(0xFF, 0xFF, 0xFF, 0);
      break;
    default:
      break;
  }
}

void BRKclock_SetTemperature16Color (byte c)
{
  switch (c)
  {
    case 0: // Schwarz
      BRKclock_SetTemperatureRGBWColor(0x00, 0x00, 0x00, 0);
      break;
    case 1: // Rot
      BRKclock_SetTemperatureRGBWColor(0xFF, 0x00, 0x00, 0);
      break;
    case 2: // Dunkelblau
      BRKclock_SetTemperatureRGBWColor(0x00, 0x00, 0x80, 0);
      break;
    case 3: // Purpur (Magenta)
      BRKclock_SetTemperatureRGBWColor(0xFF, 0x00, 0xFF, 0);
      break;
    case 4: // Dunkelgrün
      BRKclock_SetTemperatureRGBWColor(0x00, 0x80, 0x00, 0);
      break;
    case 5: // Dunkelgrau
      BRKclock_SetTemperatureRGBWColor(0x54, 0x54, 0x54, 0);
      break;
    case 6: // Blau
      BRKclock_SetTemperatureRGBWColor(0x00, 0x00, 0xFF, 0);
     break;
    case 7: // Hellblau
      BRKclock_SetTemperatureRGBWColor(0x00, 0x80, 0xFF, 0);
     break;
    case 8: // Braun
      BRKclock_SetTemperatureRGBWColor(0x80, 0x40, 0x00, 0);
     break;
    case 9: // Orange
      BRKclock_SetTemperatureRGBWColor(0xFF, 0x80, 0x00, 0);
     break;
    case 10: // Hellgrau
      BRKclock_SetTemperatureRGBWColor(0xA8, 0xA8, 0xA8, 0);
     break;
    case 11: // Pink
      BRKclock_SetTemperatureRGBWColor(0xFF, 0x80, 0x80, 0);
     break;
    case 12: // Grün
      BRKclock_SetTemperatureRGBWColor(0x00, 0xFF, 0x00, 0);
      break;
    case 13: // Gelb
      BRKclock_SetTemperatureRGBWColor(0xFF, 0xFF, 0x00, 0);
      break;
    case 14: // Türkis (Cyan)
      BRKclock_SetTemperatureRGBWColor(0x00, 0xFF, 0xFF, 0);
      break;
    case 15: // Weiß
      BRKclock_SetTemperatureRGBWColor(0xFF, 0xFF, 0xFF, 0);
      break;
    default:
      break;
  }
}

void BRKclock_Show ()
{
  delay(1);
  #if defined(ESP32)                                            // ESP 32
    portDISABLE_INTERRUPTS();                                   // Interrupts ausschalten
  #endif
  BRKclock_Pixel.show();                                        // Anzeige aktualisieren
  #if defined(ESP32)                                            // ESP 32
    portENABLE_INTERRUPTS();                                    // Interrupts einschalten
  #endif
}

void BRKclock_Clear (bool RGBW)                                 // Wahlweise RGB- oder RGBW-Modus
{
  BRKclock_Matrix.setPassThruColor(BRKclock_Pixel.Color(0, 0, 0, 0)); // Schwarzen Hintergrund für fillScreen
  BRKclock_Matrix.fillScreen(0);                                // Alle Pixel ausschalten (schwarz)
  if (RGBW)
  {
    BRKclock_Matrix.setPassThruColor();                         // In den RGB-Modus wechseln
  }
  if (RGBW)
  {
    BRKclock_SetTemperatureRGBWColor(BRKclock_tempColorRed,     // Alte RGBW-Farbe/Modus wiederherstellen
                                     BRKclock_tempColorGreen,
                                     BRKclock_tempColorBlue,
                                     BRKclock_tempColorWhite);
  }
  else
  {
    BRKclock_SetTemperatureRGBColor (BRKclock_tempColorRed,     // Alte RGB-Farbe/Modus wiederherstellen
                                     BRKclock_tempColorGreen,
                                     BRKclock_tempColorBlue);
  }
  BRKclock_Pixel.clear();                                       // Pixel löschen
  BRKclock_Show();                                              // Anzeige aktualisieren
}

void BRKclock_Init ()
{
  BRKclock_Pixel.begin();
  BRKclock_Pixel.setBrightness(255);                            // Helligkeit (0..255)
  BRKclock_Clear(true);                                         // Pixel löschen & Anzeige aktualisieren, RGBW-Modus
  BRKclock_Matrix.begin();                                      // Ausgabe auf der Matrix vorbereiten (für die Demos)
  BRKclock_Matrix.setTextWrap(false);                           // Kein Zeilenumbruch
  BRKclock_Matrix.setTextSize(1);                               // Normale Textgröße, keine Vergrößerung
  BRKclock_Matrix.setBrightness(100);                           // Helligkeit 100%
}

void BRKclock_ShowBRK ()
{
  BRKclock_Pixel.clear();                                       // Pixel löschen und "BRK" anzeigen
  BRKclock_Pixel.setPixelColor(0, 250, 0, 0);                   // "B" in Blau
  BRKclock_Pixel.setPixelColor(15, 0, 250, 0);                  // "R" in Rot
  BRKclock_Pixel.setPixelColor(16, 0, 0, 250);                  // "K" in Grün
  BRKclock_Show();                                              // Anzeige aktualisieren
}

void BRKclock_WLANautoConnect (bool useSavedSettings)
{
  BRKclock_ShowBRK();
  IoT_WLANautoConnect(useSavedSettings);                        // Verbindung, ggf. gespeicherte, über WLAN herstellen
}

void BRKclock_FullColor (uint32_t c)
{
  for (int i = 0; i < BRKclock_Pixel.numPixels(); i++)
  {
    BRKclock_Pixel.setPixelColor(i, c);
  }
  BRKclock_Show();                                              // Anzeige aktualisieren
}

uint32_t BRKclock_clockColorWheel (byte angle)   // Einen Winkel von 0-255 aus dem Farbkreis auswählen
{                                           // Die Farben sind eine Abbildung von r-g-b-r
  angle = 255 - (angle & 255);              // Es wird der zugehörige Farbwert als uint32_t ermittelt
  if (angle < 85)
  {
    return BRKclock_Pixel.Color(255 - angle * 3, 0, angle * 3);
  }
  if(angle < 170)
  {
    angle -= 85;
    return BRKclock_Pixel.Color(0, angle * 3, 255 - angle * 3);
  }
  angle -= 170;
  return BRKclock_Pixel.Color(angle * 3, 255 - angle * 3, 0);
}

void BRKclock_SetClockRandomColor ()
{
  byte c = rnd(14) + 1;                                          // Indizierte Farbe zwischen 1 und 15 (ohne schwarz)
  Serial.print("Random Color #" + str(c) + " - ");
  BRKclock_SetClockRGBWColor(t_RGBColor16ValueRed[c], t_RGBColor16ValueGreen[c], t_RGBColor16ValueBlue[c], 0);
}

void BRKclock_TestMatrix (int waitTime)
{
  float factor = 256.0 / float(BRKclock_Pixel.numPixels());      // Multiplikationsfaktor, um alle Farben zu zeigen
  IoT_DisplayClear(16);                                          // OLED Display löschen (16 Punkt Schrift)
  IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                       // Zentrierte Darstellung des Textes
  IoT_DisplayDrawText(64, 25, "BRK Clock");
  IoT_DisplayUpdate();
  for (int i = 0; i < BRKclock_Pixel.numPixels(); i++)           // LED Test -> Grün - Rot - Blau - Grün
  {
    BRKclock_Pixel.setPixelColor(i, BRKclock_clockColorWheel((int(float(i) * factor)) & 255));
    BRKclock_Show();                                             // Anzeige aktualisieren
    delay(waitTime);
  }
}

void BRKclock_Plot8color (int x, int y, int color)
{
  BRKclock_SetTemperature16Color(color & 15);                    // Indizierte Farbe für Rechteck setzen
  BRKclock_Matrix.drawRect(x, y, 1, 1, 0);
}

void BRKclock_Plot16color (int x, int y, int color)
{
  BRKclock_SetTemperature16Color(color & 15);                    // Indizierte Farbe für Rechteck setzen
  BRKclock_Matrix.drawRect(x, y, 1, 1, 0);
}

void BRKclock_Plot (int x, int y)
{
  BRKclock_Matrix.drawRect(x, y, 1, 1, BRKclock_tempColor);
}

void BRKclock_DrawNumber (int x, int y, byte num)
{
  switch (num)
  {
    case 0:
      BRKclock_Plot(x,     y);
      BRKclock_Plot(x + 1, y);
      BRKclock_Plot(x + 2, y);
      BRKclock_Plot(x,     y + 1);
      BRKclock_Plot(x + 2, y + 1);
      BRKclock_Plot(x,     y + 2);
      BRKclock_Plot(x + 2, y + 2);
      BRKclock_Plot(x,     y + 3);
      BRKclock_Plot(x + 2, y + 3);
      BRKclock_Plot(x,     y + 4);
      BRKclock_Plot(x + 1, y + 4);
      BRKclock_Plot(x + 2, y + 4);
      break;
    case 1:
      BRKclock_Plot(x + 1, y);
      BRKclock_Plot(x + 1, y + 1);
      BRKclock_Plot(x + 1, y + 2);
      BRKclock_Plot(x + 1, y + 3);
      BRKclock_Plot(x + 1, y + 4);
      break;
    case 2:
      BRKclock_Plot(x,     y);
      BRKclock_Plot(x + 1, y);
      BRKclock_Plot(x + 2, y);
      BRKclock_Plot(x + 2, y + 1);
      BRKclock_Plot(x,     y + 2);
      BRKclock_Plot(x + 1, y + 2);
      BRKclock_Plot(x + 2, y + 2);
      BRKclock_Plot(x,     y + 3);
      BRKclock_Plot(x,     y + 4);
      BRKclock_Plot(x + 1, y + 4);
      BRKclock_Plot(x + 2, y + 4);
      break;
    case 3:
      BRKclock_Plot(x,     y);
      BRKclock_Plot(x + 1, y);
      BRKclock_Plot(x + 2, y);
      BRKclock_Plot(x + 2, y + 1);
      BRKclock_Plot(x + 1, y + 2);
      BRKclock_Plot(x + 2, y + 2);
      BRKclock_Plot(x + 2, y + 3);
      BRKclock_Plot(x,     y + 4);
      BRKclock_Plot(x + 1, y + 4);
      BRKclock_Plot(x + 2, y + 4);
      break;
    case 4:
      BRKclock_Plot(x,     y);
      BRKclock_Plot(x + 2, y);
      BRKclock_Plot(x,     y + 1);
      BRKclock_Plot(x + 2, y + 1);
      BRKclock_Plot(x,     y + 2);
      BRKclock_Plot(x + 1, y + 2);
      BRKclock_Plot(x + 2, y + 2);
      BRKclock_Plot(x + 2, y + 3);
      BRKclock_Plot(x + 2, y + 4);
     break;
    case 5:
      BRKclock_Plot(x,     y);
      BRKclock_Plot(x + 1, y);
      BRKclock_Plot(x + 2, y);
      BRKclock_Plot(x,     y + 1);
      BRKclock_Plot(x,     y + 2);
      BRKclock_Plot(x + 1, y + 2);
      BRKclock_Plot(x + 2, y + 2);
      BRKclock_Plot(x + 2, y + 3);
      BRKclock_Plot(x,     y + 4);
      BRKclock_Plot(x + 1, y + 4);
      BRKclock_Plot(x + 2, y + 4);
      break;
    case 6:
      BRKclock_Plot(x,     y);
      BRKclock_Plot(x + 1, y);
      BRKclock_Plot(x + 2, y);
      BRKclock_Plot(x,     y + 1);
      BRKclock_Plot(x,     y + 2);
      BRKclock_Plot(x + 1, y + 2);
      BRKclock_Plot(x + 2, y + 2);
      BRKclock_Plot(x,     y + 3);
      BRKclock_Plot(x + 2, y + 3);
      BRKclock_Plot(x,     y + 4);
      BRKclock_Plot(x + 1, y + 4);
      BRKclock_Plot(x + 2, y + 4);
      break;
    case 7:
      BRKclock_Plot(x,     y);
      BRKclock_Plot(x + 1, y);
      BRKclock_Plot(x + 2, y);
      BRKclock_Plot(x + 2, y + 1);
      BRKclock_Plot(x + 2, y + 2);
      BRKclock_Plot(x + 2, y + 3);
      BRKclock_Plot(x + 2, y + 4);
      break;
    case 8:
      BRKclock_Plot(x,     y);
      BRKclock_Plot(x + 1, y);
      BRKclock_Plot(x + 2, y);
      BRKclock_Plot(x,     y + 1);
      BRKclock_Plot(x + 2, y + 1);
      BRKclock_Plot(x,     y + 2);
      BRKclock_Plot(x + 1, y + 2);
      BRKclock_Plot(x + 2, y + 2);
      BRKclock_Plot(x,     y + 3);
      BRKclock_Plot(x + 2, y + 3);
      BRKclock_Plot(x,     y + 4);
      BRKclock_Plot(x + 1, y + 4);
      BRKclock_Plot(x + 2, y + 4);
      break;
    case 9:
      BRKclock_Plot(x,     y);
      BRKclock_Plot(x + 1, y);
      BRKclock_Plot(x + 2, y);
      BRKclock_Plot(x,     y + 1);
      BRKclock_Plot(x + 2, y + 1);
      BRKclock_Plot(x,     y + 2);
      BRKclock_Plot(x + 1, y + 2);
      BRKclock_Plot(x + 2, y + 2);
      BRKclock_Plot(x + 2, y + 3);
      BRKclock_Plot(x,     y + 4);
      BRKclock_Plot(x + 1, y + 4);
      BRKclock_Plot(x + 2, y + 4);
      break;
    default:
      break;
  }
}

void BRKclock_Print (String s, int pause)                           // Laufschrift
{
  BRKclock_Clear(true);                                             // Pixel löschen & Anzeige aktualisieren
  int x = BRKclock_Matrix.width();                                  // Breite der Matrix (8)
  IoT_WaitMessage();                                                // Mitteilung anzeigen
  while (not(IoT_Keypress()))
  {
    int L = (len(s) - 1) * 8;                                       // Breite der Laufschrift insgesamt in Pixel
    BRKclock_Clear(true);                                           // Pixel löschen & Anzeige aktualisieren

    BRKclock_Matrix.setCursor(x, 0);                                // Von unten rechts reinlaufen, x verringert sich im loop()
    BRKclock_Matrix.print(s);                                       // Laufschrift "Brick'R'knowledge" loslassen...
    if (--x < -L)
    {
      break;
    }
    BRKclock_Matrix.show();
    IoT_WaitKeypress(pause, false);                                 // 100 ms. oder auf Taster warten
  }
}

void BRKclock_PrintTemperature (int temperature, int pause)
{
  if ((temperature < 0) || (temperature > 99))
  {
    temperature = 0;
  }
  BRKclock_Clear(true);                                             // Pixel löschen & Anzeige aktualisieren
  BRKclock_DrawNumber(0, 2, temperature / 10);                      // Zehner
  BRKclock_DrawNumber(4, 2, temperature % 10);                      // Einer
  BRKclock_Plot(7, 1);                                              // Grad
  BRKclock_Matrix.show();                                           // Zahl zeigen
  IoT_WaitKeypress(pause, true);                                    // 10 Sekunden. oder auf Taster warten
}


// *** BRK-Clock Parameter für das EEPROM des ESP8266 ***

bool BRKclock_ParameterValidate ()
{
  if ((BRKclock_ParameterData.parameterID == crc64("BRKClock", 0)) &&
      (BRKclock_ParameterData.parameterHash == xorshift128plus("BRKClock", 0)))
  {
    return (true);                                                            // Parameter sind gültig
  }
  else
  {
    return (false);                                                           // Ungültige Parameter
  }
}

bool BRKclock_ParameterApply ()
{
  if (BRKclock_ParameterValidate())
  {
    BRKclock_clockColorRed = BRKclock_ParameterData.clockColorRed;            // RGBW-Farbe der Uhrzeitanzeige
    BRKclock_clockColorGreen = BRKclock_ParameterData.clockColorGreen;
    BRKclock_clockColorBlue = BRKclock_ParameterData.clockColorBlue;
    BRKclock_clockColorWhite = BRKclock_ParameterData.clockColorWhite;
    BRKclock_SetClockRGBWColor(BRKclock_clockColorRed, BRKclock_clockColorGreen, BRKclock_clockColorBlue, BRKclock_clockColorWhite); // Schriftfarbe
    BRKclock_tempColorRed = BRKclock_ParameterData.tempColorRed;              // RGBW-Farbe der Temperaturanzeige
    BRKclock_tempColorGreen = BRKclock_ParameterData.tempColorGreen;
    BRKclock_tempColorBlue = BRKclock_ParameterData.tempColorBlue;
    BRKclock_tempColorWhite = BRKclock_ParameterData.tempColorWhite;
    BRKclock_SetTemperatureRGBWColor(BRKclock_tempColorRed, BRKclock_tempColorGreen, BRKclock_tempColorBlue, BRKclock_tempColorWhite);
    if (BRKclock_ParameterData.staticIP == 1)
    {
      WiFi.localIP()[0] = BRKclock_ParameterData.IP_address0;                 // 4 Bytes IP-Adresse
      WiFi.localIP()[1] = BRKclock_ParameterData.IP_address1;
      WiFi.localIP()[2] = BRKclock_ParameterData.IP_address2;
      WiFi.localIP()[3] = BRKclock_ParameterData.IP_address3;
      WiFi.gatewayIP()[0] = BRKclock_ParameterData.IP_gateway0;               // 4 Bytes Gateway
      WiFi.gatewayIP()[1] = BRKclock_ParameterData.IP_gateway1;
      WiFi.gatewayIP()[2] = BRKclock_ParameterData.IP_gateway2;
      WiFi.gatewayIP()[3] = BRKclock_ParameterData.IP_gateway3;
      WiFi.subnetMask()[0] = BRKclock_ParameterData.IP_subnet0;               // 4 Bytes Subnetzmaske
      WiFi.subnetMask()[1] = BRKclock_ParameterData.IP_subnet1;
      WiFi.subnetMask()[2] = BRKclock_ParameterData.IP_subnet2;
      WiFi.subnetMask()[3] = BRKclock_ParameterData.IP_subnet3;
      IPAddress ip1 = IPAddress(BRKclock_ParameterData.IP_address0,
                                BRKclock_ParameterData.IP_address1,
                                BRKclock_ParameterData.IP_address2,
                                BRKclock_ParameterData.IP_address3);
      IPAddress ip2 = IPAddress(BRKclock_ParameterData.IP_gateway0,
                                BRKclock_ParameterData.IP_gateway1,
                                BRKclock_ParameterData.IP_gateway2,
                                BRKclock_ParameterData.IP_gateway3);
      IPAddress ip3 = IPAddress(BRKclock_ParameterData.IP_subnet0,
                                BRKclock_ParameterData.IP_subnet1,
                                BRKclock_ParameterData.IP_subnet2,
                                BRKclock_ParameterData.IP_subnet3);
      WiFi.config(ip1, ip2, ip3);                                             // Parameter: IP, Gateway, Subnet, DNS
    }
    BRKclock_staticIP = BRKclock_ParameterData.staticIP;                      // 0 = Dynamische IP-Adresse, 1 = Statische IP-Adresse
    BRKclock_alarmMode = BRKclock_ParameterData.alarmMode;                    // 0 = Wecker aus,    1 = Wecker ein
    BRKclock_temperatureMode = BRKclock_ParameterData.temperatureMode;        // 0 = Temperaturanzeige aus,   1 = Temperaturanzeige jede Minute
    BRKclock_colorMode = BRKclock_ParameterData.colorMode;                    // 0 = Random Schrift, 1 = Weiße Schrift, 2 = Rote Schrift, usw.
    BRKclock_buttonMode = BRKclock_ParameterData.buttonMode;                  // 0 = IP-Adresse anzeigen, 1 = Temperatur anzeigen, usw.
    BRKclock_alarmHour = BRKclock_ParameterData.alarmHour;                    // Weckzeit: Stunden
    BRKclock_alarmMinute = BRKclock_ParameterData.alarmMinute;                // Weckzeit: Minuten
    BRKclock_language = BRKclock_ParameterData.language;                      // Sprache: 0 = Deutsch, 1 = Englisch
    BRKclock_temperatureOffset = BRKclock_ParameterData.temperatureOffset;    // Temperatur-Offset +/-
    return (true);                                                            // Parameter-Block ist gültig und wurde verwendet
  }
  else
  {
    return (false);                                                           // Parameter-Block ist ungültig und wurde nicht verwendet
  }
}

void BRKclock_ParameterCreate ()
{
//   32 Bytes Parameter-Header:
  BRKclock_ParameterData.parameterID       = crc64("BRKClock", 0);            // Kennung des Parameter-Blocks "BRKClock" im Klartext
  BRKclock_ParameterData.parameterHash     = xorshift128plus("BRKClock", 0);  // Kennung des Parameter-Blocks "BRKClock" als verschlüsselter Hash
  BRKclock_ParameterData.parameterSize     = sizeof(BRKclock_Parameter);      // Anzahl Bytes im Parameter Block
  BRKclock_ParameterData.parameterReserve  = 0;                               // Platzhalter für weiteren Header-Eintrag
  BRKclock_ParameterData.parameterVersion  = 1;                               // Version des Parameter-Blocks
  BRKclock_ParameterData.parameterCount    = 26;                              // Anzahl folgenden Byte-Parameter im Block
    
//   Start des Parameter-Blocks:
  BRKclock_ParameterData.clockColorRed     = BRKclock_clockColorRed;          // RGBW-Farbe der Uhrzeitanzeige
  BRKclock_ParameterData.clockColorGreen   = BRKclock_clockColorGreen;
  BRKclock_ParameterData.clockColorBlue    = BRKclock_clockColorBlue;
  BRKclock_ParameterData.clockColorWhite   = BRKclock_clockColorWhite;
  BRKclock_ParameterData.tempColorRed      = BRKclock_tempColorRed;           // RGBW-Farbe der Temperaturanzeige
  BRKclock_ParameterData.tempColorGreen    = BRKclock_tempColorGreen;
  BRKclock_ParameterData.tempColorBlue     = BRKclock_tempColorBlue;
  BRKclock_ParameterData.tempColorWhite    = BRKclock_tempColorWhite;
  BRKclock_ParameterData.IP_address0       = WiFi.localIP()[0];               // 4 Bytes IP-Adresse
  BRKclock_ParameterData.IP_address1       = WiFi.localIP()[1];
  BRKclock_ParameterData.IP_address2       = WiFi.localIP()[2];
  BRKclock_ParameterData.IP_address3       = WiFi.localIP()[3];
  BRKclock_ParameterData.IP_gateway0       = WiFi.gatewayIP()[0];             // 4 Bytes Gateway
  BRKclock_ParameterData.IP_gateway1       = WiFi.gatewayIP()[1];
  BRKclock_ParameterData.IP_gateway2       = WiFi.gatewayIP()[2];
  BRKclock_ParameterData.IP_gateway3       = WiFi.gatewayIP()[3];
  BRKclock_ParameterData.IP_subnet0        = WiFi.subnetMask()[0];            // 4 Bytes Subnetzmaske
  BRKclock_ParameterData.IP_subnet1        = WiFi.subnetMask()[1];
  BRKclock_ParameterData.IP_subnet2        = WiFi.subnetMask()[2];
  BRKclock_ParameterData.IP_subnet3        = WiFi.subnetMask()[3];
  BRKclock_ParameterData.staticIP          = BRKclock_staticIP;               // 0 = Dynamische IP-Adresse, 1 = Statische IP-Adresse
  BRKclock_ParameterData.alarmMode         = BRKclock_alarmMode;              // 0 = Wecker aus,    1 = Wecker ein
  BRKclock_ParameterData.temperatureMode   = BRKclock_temperatureMode;        // 0 = Temperaturanzeige aus,   1 = Temperaturanzeige jede Minute
  BRKclock_ParameterData.colorMode         = BRKclock_colorMode;              // 0 = Random Schrift, 1 = Weiße Schrift, 2 = Rote Schrift, usw.
  BRKclock_ParameterData.buttonMode        = BRKclock_buttonMode;             // 0 = IP-Adresse anzeigen, 1 = Temperatur anzeigen, usw.
  BRKclock_ParameterData.alarmHour         = BRKclock_alarmHour;              // Weckzeit: Stunden
  BRKclock_ParameterData.alarmMinute       = BRKclock_alarmMinute;            // Weckzeit: Minuten
  BRKclock_ParameterData.language          = BRKclock_language;               // Sprache: 0 = Deutsch, 1 = Englisch
  BRKclock_ParameterData.temperatureOffset = BRKclock_temperatureOffset;      // Temperatur-Offset +/-
}

void BRKclock_ParameterNew ()
{
//   32 Bytes Parameter-Header:
  BRKclock_ParameterData.parameterID       = crc64("BRKClock", 0);            // Kennung des Parameter-Blocks "BRKClock" im Klartext
  BRKclock_ParameterData.parameterHash     = xorshift128plus("BRKClock", 0);  // Kennung des Parameter-Blocks "BRKClock" als verschlüsselter Hash
  BRKclock_ParameterData.parameterSize     = sizeof(BRKclock_Parameter);      // Anzahl Bytes im Parameter Block
  BRKclock_ParameterData.parameterReserve  = 0;                               // Platzhalter für weiteren Header-Eintrag
  BRKclock_ParameterData.parameterVersion  = 1;                               // Version des Parameter-Blocks
  BRKclock_ParameterData.parameterCount    = 26;                              // Anzahl folgenden Byte-Parameter im Block
    
//   Start des Parameter-Blocks:
  BRKclock_ParameterData.clockColorRed     = 0xFF;                            // RGBW-Farbe der Uhrzeitanzeige
  BRKclock_ParameterData.clockColorGreen   = 0x00;
  BRKclock_ParameterData.clockColorBlue    = 0x00;
  BRKclock_ParameterData.clockColorWhite   = 0x00;
  BRKclock_ParameterData.tempColorRed      = 0xFF;                            // RGBW-Farbe der Temperaturanzeige
  BRKclock_ParameterData.tempColorGreen    = 0x00;
  BRKclock_ParameterData.tempColorBlue     = 0x00;
  BRKclock_ParameterData.tempColorWhite    = 0x00;
  BRKclock_ParameterData.IP_address0       = 0;                               // 4 Bytes IP-Adresse
  BRKclock_ParameterData.IP_address1       = 0;
  BRKclock_ParameterData.IP_address2       = 0;
  BRKclock_ParameterData.IP_address3       = 0;
  BRKclock_ParameterData.IP_gateway0       = 0;                               // 4 Bytes Gateway
  BRKclock_ParameterData.IP_gateway1       = 0;
  BRKclock_ParameterData.IP_gateway2       = 0;
  BRKclock_ParameterData.IP_gateway3       = 0;
  BRKclock_ParameterData.IP_subnet0        = 0;                               // 4 Bytes Subnetzmaske
  BRKclock_ParameterData.IP_subnet1        = 0;
  BRKclock_ParameterData.IP_subnet2        = 0;
  BRKclock_ParameterData.IP_subnet3        = 0;
  BRKclock_ParameterData.staticIP          = 0;                               // 0 = Dynamische IP-Adresse, 1 = Statische IP-Adresse
  BRKclock_ParameterData.alarmMode         = 0;                               // 0 = Wecker aus,    1 = Wecker ein
  BRKclock_ParameterData.temperatureMode   = 1;                               // 0 = Temperaturanzeige aus,   1 = Temperaturanzeige jede Minute
  BRKclock_ParameterData.colorMode         = 2;                               // 0 = Random Schrift, 1 = Weiße Schrift, 2 = Rote Schrift, usw.
  BRKclock_ParameterData.buttonMode        = 0;                               // 0 = IP-Adresse anzeigen, 1 = Temperatur anzeigen, usw.
  BRKclock_ParameterData.alarmHour         = 0;                               // Weckzeit: Stunden
  BRKclock_ParameterData.alarmMinute       = 0;                               // Weckzeit: Minuten
  BRKclock_ParameterData.language          = 0;                               // Sprache: 0 = Deutsch, 1 = Englisch
  BRKclock_ParameterData.temperatureOffset = 0.0;                             // Temperatur-Offset +/-
}

bool BRKclock_ParameterLoad ()
{
  EEPROM.get(0, BRKclock_ParameterData); // Parameter aus dem EEPROM laden und danach die geladenen Daten validieren
  if (BRKclock_ParameterValidate())
  {
    return (true);                                                            // Parameter sind gültig und wurden erfolgreich geladen
  }
  else
  {
    return (false);                                                           // Ungültige Parameter
  }
}

void BRKclock_ParameterSave ()
{
  if (BRKclock_ParameterValidate())
  {
    EEPROM.put(0, BRKclock_ParameterData); // Parameter in das EEPROM sichern
  }
}


// *** BRK-Clock Uhr-Funktionen in Deutsch ***

void BRKclock_fuenf ()
{
  for (int i = 60; i <= 63; i++)
  {   
    BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
  }
}

void BRKclock_zehn ()
{
  for (int i = 56; i <= 59; i++)
  {   
    BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
  }
}

void BRKclock_vor ()
{
  for (int i = 48; i <= 50; i++)
  {   
    BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
  }
}

void BRKclock_nach ()
{
  for (int i = 52; i <= 55; i++)
  {   
    BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
  }
}

void BRKclock_halb ()
{
  for (int i = 44; i <= 47; i++)
  {   
    BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
  }
}

void BRKclock_Stunde (int hr)
{
  hr = hr % 12;
  if (hr == 0)      // 12 oder 24 Uhr
  {
    hr = 12;
  }
  
  switch (hr)
  {
    case 1:
      for (int i = 32; i <= 35; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 2:
      for (int i = 27; i <= 28; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      for (int i = 21; i <= 22; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 3:
      for (int i = 19; i <= 22; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 4:
      for (int i = 40; i <= 43; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 5:
      for (int i = 7; i <= 8; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      for (int i = 23; i <= 24; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 6:
      for (int i = 35; i <= 39; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 7:
      for (int i = 16; i <= 18; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      for (int i = 29; i <= 31; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 8:
      for (int i = 1; i <= 4; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 9:
      for (int i = 8; i <= 11; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 10:
      for (int i = 11; i <= 14; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 11:
      for (int i = 5; i <= 7; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 12:
      for (int i = 24; i <= 28; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;  
    default: 
      Serial.println("Fehler: Stunden");         // Fehler Stunden
      for (int i = 0; i < BRKclock_Pixel.numPixels(); i++)
      {  
        BRKclock_Pixel.setPixelColor(i, 255, 0, 0); // red
      }
      break;
  }
}

void BRKclock_Zeit (int h, int m)
{
  Serial.print(IoT_NTPtime() + ": ");
  BRKclock_Pixel.clear();
  if ((m <= 2) || (m >= 58))  // h (h:58-h:02)
  {      
    Serial.println("h (h:58-h:02)");
    if (m <= 2)
    {
      BRKclock_Stunde(h);
    }
    else
    {
      BRKclock_Stunde(h + 1);
    }
  }
  else if ((3 <= m) && (m <= 7))
  {    // FÜNF NACH h (h:03-h:07) 
    Serial.println("FUENF NACH h (h:03-h:07)");
    BRKclock_fuenf();
    BRKclock_nach();
    BRKclock_Stunde(h);
  }
  else if ((8 <= m) && (m <= 12))
  {   // ZEHN NACH h (h:08-h:12) 
    Serial.println("ZEHN NACH h (h:08-h:12)");
    BRKclock_zehn();
    BRKclock_nach();
    BRKclock_Stunde(h);
  }
  else if ((13 <= m) && (m <= 17))
  {  // FÜNFZEHN NACH h (h:13-h:17)
    Serial.println("FUENFZEHN NACH h (h:13-h:17)");
    BRKclock_fuenf(); 
    BRKclock_zehn();
    BRKclock_nach();
    BRKclock_Stunde(h);
  }
  else if ((18<= m) && (m <= 22))
  {  // ZEHN VOR HALB h+1 (h:18-h:22)
    Serial.println("ZEHN VOR HALB h+1 (h:18-h:22)");
    BRKclock_zehn();
    BRKclock_vor();
    BRKclock_halb();
    BRKclock_Stunde(h + 1);
  }
  else if ((23<= m) && (m <= 27))
  {  // FÜNF VOR HALB h+1 (h:23-h:27)
    Serial.println("FUENF VOR HALB h+1 (h:23-h:27)");
    BRKclock_fuenf();
    BRKclock_vor();
    BRKclock_halb();
    BRKclock_Stunde(h + 1);
  }
  else if ((28<= m) && (m <= 32))
  {  //  HALB h+1 (h:28-h:32)
    Serial.println("HALB h+1 (h:28-h:32)");
    BRKclock_halb();
    BRKclock_Stunde(h + 1);
  }
  else if ((33<= m) && (m <= 37))
  {  // FÜNF NACH HALB h+1 (h:33-h:37) 
    Serial.println("FUENF NACH HALB h+1 (h:33-h:37)");
    BRKclock_fuenf();
    BRKclock_nach();
    BRKclock_halb();
    BRKclock_Stunde(h + 1);
  }
  else if ((38<= m) && (m <= 42))
  {  // ZEHN NACH HALB h+1 (h:38-h:42) 
    Serial.println("ZEHN NACH HALB h+1 (h:38-h:42)");
    BRKclock_zehn();
    BRKclock_nach();
    BRKclock_halb();
    BRKclock_Stunde(h + 1);
  }
  else if ((43<= m) && (m <= 47))
  {  // FÜNFZEHN VOR h+1 (h:43-h:47) 
    Serial.println("FUENFZEHN VOR h+1 (h:43-h:47)");
    BRKclock_fuenf();
    BRKclock_zehn();
    BRKclock_vor();
    BRKclock_Stunde(h + 1);
  }
  else if ((48<= m) && (m <= 52))
  {  // ZEHN VOR h+1 (h:48-h:52)
    Serial.println("ZEHN VOR h+1 (h:48-h:52)");
    BRKclock_zehn();
    BRKclock_vor();
    BRKclock_Stunde(h + 1);
  }
  else if ((53<= m) && (m <= 57))
  {  // FÜNF VOR h+1 (h:53-h:57)
    Serial.println("FUENF VOR h+1 (h:53-h:57)");
    BRKclock_fuenf();
    BRKclock_vor();
    BRKclock_Stunde(h + 1);
  }
  else
  {                                         // Fehler Minuten
    Serial.println("Fehler: Minuten");
    for (int j = 0; j < BRKclock_Pixel.numPixels(); j++)
    {
      BRKclock_Pixel.setPixelColor(j, 100, 100, 100);    // Weiß
    }
  }
  BRKclock_Show();                         // Anzeige aktualisieren
}


// *** BRK-Clock Uhr-Funktionen in Englisch ***

void BRKclock_five ()
{
  for (int i = 48; i <= 49; i++)
  {   
    BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
  }
  BRKclock_Pixel.setPixelColor(51, BRKclock_clockColor);
  BRKclock_Pixel.setPixelColor(53, BRKclock_clockColor);
}

void BRKclock_ten ()
{
  for (int i = 52; i <= 53; i++)
  {   
    BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
  }
  BRKclock_Pixel.setPixelColor(55, BRKclock_clockColor);
}

void BRKclock_fifteen ()
{
  for (int i = 48; i <= 50; i++)
  {   
    BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
  }
  for (int i=52; i<=55; i++)
  {   
    BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
  }
}

void BRKclock_twenty ()
{
  for (int i=56; i<=61; i++)
  {   
    BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
  }
}

void BRKclock_to ()
{
  for (int i=40; i<=41; i++)
  {   
    BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
  }
}

void BRKclock_past ()
{
  for (int i=41; i<=44; i++)
  {   
    BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
  }
}

void BRKclock_half ()
{
  for (int i = 62; i <= 63; i++)
  {   
    BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
  }
  for (int i = 46; i <= 47; i++)
  {   
    BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
  }
}

void BRKclock_Hour (int hr)
{
  hr = hr % 12;
  if (hr == 0)      // 12 oder 24 Uhr
  {
    hr = 12;
  }
  
  switch (hr)
  {
    case 1:
      for (int i = 29; i <= 31; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 2:
      for (int i = 16; i <= 17; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      } 
      BRKclock_Pixel.setPixelColor(14, BRKclock_clockColor);
      break;
    case 3:
      for (int i = 24; i <= 28; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 4:
      for (int i = 12; i <= 15; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 5:
      for (int i = 8; i <= 11; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 6:
      for (int i = 0; i <= 2; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 7:
      for (int i = 3; i <= 7; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 8:
      for (int i = 35; i <= 39; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 9:
      for (int i = 32; i <= 35; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 10:
        BRKclock_Pixel.setPixelColor(23, BRKclock_clockColor);
        BRKclock_Pixel.setPixelColor(24, BRKclock_clockColor);
        BRKclock_Pixel.setPixelColor(39, BRKclock_clockColor);
      break;
    case 11:
      for (int i = 18; i <= 23; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      break;
    case 12:
      for (int i = 16; i <= 19; i++)
      {   
        BRKclock_Pixel.setPixelColor(i, BRKclock_clockColor);
      }
      BRKclock_Pixel.setPixelColor(21, BRKclock_clockColor);
      BRKclock_Pixel.setPixelColor(22, BRKclock_clockColor);
      break;  
    default: 
      Serial.println("Error: Hours");         // Fehler Stunden
      for (int i = 0; i < BRKclock_Pixel.numPixels(); i++)
      {  
        BRKclock_Pixel.setPixelColor(i, 255, 0, 0); // red
      }
      break;
  }
}

void BRKclock_Time (int h, int m)
{
  Serial.print(IoT_NTPtime() + ": ");
  BRKclock_Pixel.clear();
  if ((m <= 2) || (m >= 58))        // h (h:58-h:02)
  {
    Serial.println("h (h:58-h:02)");
    if (m <= 2)
    {
      BRKclock_Hour(h);
    }
    else
    {
      BRKclock_Hour(h+1);
    }
  }
  else if ((3 <= m) && (m <= 7))    // Five past h (h:03-h:07) 
  {
    Serial.println("Five past h (h:03-h:07)");
    BRKclock_five();
    BRKclock_past();
    BRKclock_Hour(h);
  }
  else if ((8 <= m) && (m <= 12))   // Ten past h (h:08-h:12) 
  {
    Serial.println("Ten past h (h:08-h:12)");
    BRKclock_ten();
    BRKclock_past();
    BRKclock_Hour(h);
  }
  else if ((13 <= m) && (m <= 17))  // Fifteen past h (h:13-h:17)
  {
    Serial.println("Fifteen past h (h:13-h:17)");
    BRKclock_fifteen(); 
    BRKclock_past();
    BRKclock_Hour(h);
  }
  else if ((18<= m) && (m <= 22))  // Twenty past h+1 (h:18-h:22)
  {
    Serial.println("Twenty past h+1 (h:18-h:22)");
    BRKclock_twenty();
    BRKclock_past();
    BRKclock_Hour(h);
  }
  else if ((23<= m) && (m <= 27))  // Twenty Five past h+1 (h:23-h:27)
  {
    Serial.println("Twenty Five past h+1 (h:23-h:27)");
    BRKclock_five();
    BRKclock_twenty();
    BRKclock_past();
    BRKclock_Hour(h);
  }
  else if ((28<= m) && (m <= 32))  //  Half past h+1 (h:28-h:32)
  {
    Serial.println("Half past h+1 (h:28-h:32)");
    BRKclock_half();
    BRKclock_past();
    BRKclock_Hour(h);
  }
  else if ((33<= m) && (m <= 37))  // Twenty Five to h+1 (h:33-h:37) 
  {
    Serial.println("Twenty Five to h+1 (h:33-h:37)");
    BRKclock_five();
    BRKclock_twenty();
    BRKclock_to();
    BRKclock_Hour(h+1);
  }
  else if ((38<= m) && (m <= 42))  // Twenty to h+1 (h:38-h:42) 
  {
    Serial.println("Twenty to h+1 (h:38-h:42)");
    BRKclock_twenty();
    BRKclock_to();
    BRKclock_Hour(h+1);
  }
  else if ((43<= m) && (m <= 47))  // Fifteen to h+1 (h:43-h:47) 
  {
    Serial.println("Fifteen to h+1 (h:43-h:47)");
    BRKclock_fifteen();
    BRKclock_to();
    BRKclock_Hour(h+1);
  }
  else if ((48<= m) && (m <= 52))  // Ten to h+1 (h:48-h:52)
  {
    Serial.println("Ten to h+1 (h:48-h:52)");
    BRKclock_ten();
    BRKclock_to();
    BRKclock_Hour(h+1);
  }
  else if ((53<= m) && (m <= 57))  // Five to h+1 (h:53-h:57)
  {
    Serial.println("Five to h+1 (h:53-h:57)");
    BRKclock_five();
    BRKclock_to();
    BRKclock_Hour(h+1);
  }
  else
  {                                         // Fehler Minuten
    Serial.println("Error: Minutes");
    for (int j = 0; j < BRKclock_Pixel.numPixels(); j++)
    {
      BRKclock_Pixel.setPixelColor(j, 100, 100, 100);    // Weiß
    }
  }
  BRKclock_Show();                                             // Anzeige aktualisieren
}


// *** BRK-Clock Uhr-Funktionen International ***

void BRKclock_International (int h, int m)
{
  switch (BRKclock_language)
  {
    case 0:
      BRKclock_Zeit (h, m);
      break;
    case 1:
      BRKclock_Time (h, m);
      break;
    default:
      break;
  }
}

// *** BRK-Clock Demos ***

void BRKclock_Kaleidoscope (int size, bool color16)
{
  BRKclock_Clear(false);                                            // Pixel löschen & Anzeige aktualisieren
  long t = millis();
  size--;
  for (int w = 0; w <= size * 10; w++)
  {
    for (int i = 0; i <= (size / 2); i++)
    {
      for (int j = 0; j <= (size / 2); j++)
      {
        int k = i + j;
        int color = j * 3 / (i + 3) + (i + 1) * w * 3;
        if (color16)
        {
          BRKclock_Plot16color(i, k, color);
          BRKclock_Plot16color(k, i, color);
          BRKclock_Plot16color(size - i, size - k, color);
          BRKclock_Plot16color(size - k, size - i, color);
          BRKclock_Plot16color(k, size - i, color);
          BRKclock_Plot16color(size - i, k, color);
          BRKclock_Plot16color(i, size - k, color);
          BRKclock_Plot16color(size - k, i, color);
        }
        else
        {
          BRKclock_Plot8color(i, k, color);
          BRKclock_Plot8color(k, i, color);
          BRKclock_Plot8color(size - i, size - k, color);
          BRKclock_Plot8color(size - k, size - i, color);
          BRKclock_Plot8color(k, size - i, color);
          BRKclock_Plot8color(size - i, k, color);
          BRKclock_Plot8color(i, size - k, color);
          BRKclock_Plot8color(size - k, i, color);
        }
        BRKclock_Matrix.show();
        IoT_Idle();
        IoT_WaitKeypress(50, false);                                // 50 ms. oder auf Taster warten
        if (IoT_Keypress())
        {
          return;
        }
      }
    }
  }
}

void BRKclock_Rectangles ()
{
  BRKclock_Clear(false);                                            // Pixel löschen & Anzeige aktualisieren
  int i = 0;
  IoT_WaitMessage();                                                // Mitteilung anzeigen
  while (not(IoT_Keypress()))
  {
    i++;
    BRKclock_SetTemperature16Color(i & 15);                         // Indizierte Farbe für Rechteck setzen
    BRKclock_Matrix.drawRect(0, 0, 8, 8, BRKclock_Matrix16Color[i & 15]);
    BRKclock_Matrix.show();
    IoT_WaitKeypress(500, false);                                   // 500 ms. oder auf Taster warten
    i++;
    BRKclock_SetTemperature16Color(i & 15);                         // Indizierte Farbe für Rechteck setzen
    BRKclock_Matrix.drawRect(1, 1, 6, 6, BRKclock_Matrix16Color[i & 15]);
    BRKclock_Matrix.show();
    IoT_WaitKeypress(500, false);                                   // 500 ms. oder auf Taster warten
    i++;
    BRKclock_SetTemperature16Color(i & 15);                         // Indizierte Farbe für Rechteck setzen
    BRKclock_Matrix.drawRect(2, 2, 4, 4, BRKclock_Matrix16Color[i & 15]);
    BRKclock_Matrix.show();
    IoT_WaitKeypress(500, false);                                   // 500 ms. oder auf Taster warten
    i++;
    BRKclock_SetTemperature16Color(i & 15);                         // Indizierte Farbe für Rechteck setzen
    BRKclock_Matrix.drawRect(3, 3, 2, 2, BRKclock_Matrix16Color[i & 15]);
    BRKclock_Matrix.show();
    IoT_WaitKeypress(500, false);                                   // 500 ms. oder auf Taster warten
  }
}

void BRKclock_clockColorWipe (uint32_t c, byte wait)                // Schlangenlinien
{
  for (int i = 0; i < BRKclock_Pixel.numPixels(); i++)
  {
    BRKclock_Pixel.setPixelColor(i, c);
    BRKclock_Show();                                                // Anzeige aktualisieren
    IoT_WaitKeypress(wait, false);                                  // wait ms. oder auf Taster warten
    if (IoT_Keypress())
    {
      return;
    }
  }
}

void BRKclock_WhiteOverRainbow (byte wait, byte whiteSpeed, byte whiteLength) // Weiße Welle über Regenbogen
{
  if (whiteLength >= BRKclock_Pixel.numPixels())
  {
    whiteLength = BRKclock_Pixel.numPixels() - 1;
  }
  int head = whiteLength - 1;
  int tail = 0;
  int loops = 3;
  int loopNum = 0;
  static unsigned long lastTime = 0;
  while (true)
  {
    for (int j = 0; j < 256; j++)
    {
      for (int i = 0; i < BRKclock_Pixel.numPixels(); i++)
      {
        if ((i >= tail && i <= head) || (tail > head && i >= tail) || (tail > head && i <= head) )
        {
          BRKclock_Pixel.setPixelColor(i, BRKclock_Pixel.Color(0,0,0, 255));
        }
        else
        {
          BRKclock_Pixel.setPixelColor(i, BRKclock_clockColorWheel(((i * 256 / BRKclock_Pixel.numPixels()) + j) & 255));
        }
      }
      if (millis() - lastTime > whiteSpeed)
      {
        head++;
        tail++;
        if(head == BRKclock_Pixel.numPixels())
        {
          loopNum++;
        }
        lastTime = millis();
      }
      if (loopNum == loops)
      {
        return;
      }
      head%=BRKclock_Pixel.numPixels();
      tail%=BRKclock_Pixel.numPixels();
      BRKclock_Show();                                            // Anzeige aktualisieren
      IoT_WaitKeypress(wait, false);                              // wait ms. oder auf Taster warten
      if (IoT_Keypress())
      {
        return;
      }
    }
  }
}

void BRKclock_PulseWhite (byte wait, byte count)                  // Weißes Pulsieren
{
  for (byte loop = 1; loop <= count; loop++)
  {
    for (int j = 0; j < 256 ; j++)
    {
      for (int i = 0; i < BRKclock_Pixel.numPixels(); i++)
      {
        BRKclock_Pixel.setPixelColor(i, BRKclock_Pixel.Color(0,0,0, BRKclock_Gamma[j] ) );
      }
      IoT_WaitKeypress(wait, false);                              // wait ms. oder auf Taster warten
      if (IoT_Keypress())
      {
        return;
      }
      BRKclock_Show();                                            // Anzeige aktualisieren
    }
    for (int j = 255; j >= 0 ; j--)
    {
      for (int i = 0; i < BRKclock_Pixel.numPixels(); i++)
      {
        BRKclock_Pixel.setPixelColor(i, BRKclock_Pixel.Color(0,0,0, BRKclock_Gamma[j] ) );
      }
      IoT_WaitKeypress(wait, false);                              // wait ms. oder auf Taster warten
      if (IoT_Keypress())
      {
        return;
      }
      BRKclock_Show();                                            // Anzeige aktualisieren
    }
  }
}

void BRKclock_RainbowFade2White (byte wait, int rainbowLoops, int whiteLoops) // Regenbogen wird weiß
{
  float fadeMax = 100.0;
  int fadeVal = 0;
  uint32_t wheelVal;
  int redVal, greenVal, blueVal;
  
  for (int k = 0 ; k < rainbowLoops ; k ++)
  {  
    for (int j=0; j<256; j++) // 5 Zyklen für alle Farben des Farbrades
    {
      for (int i=0; i< BRKclock_Pixel.numPixels(); i++)
      {
        wheelVal = BRKclock_clockColorWheel(((i * 256 / BRKclock_Pixel.numPixels()) + j) & 255);
        redVal = ((wheelVal >> 16) & 0xFF) * float(fadeVal/fadeMax);
        greenVal = ((wheelVal >> 8) & 0xFF) * float(fadeVal/fadeMax);
        blueVal = (wheelVal & 0xFF) * float(fadeVal/fadeMax);
        BRKclock_Pixel.setPixelColor(i, BRKclock_Pixel.Color(redVal, greenVal, blueVal));
      }
      if (k == 0 && fadeVal < fadeMax - 1)      // Erster Durchlauf, Fade In
      {
          fadeVal++;
      }
      else if (k == rainbowLoops - 1 && j > 255 - fadeMax)        // Letzter Durchlauf, Fade Out
      {
          fadeVal--;
      }
      BRKclock_Show();                                            // Anzeige aktualisieren
      IoT_WaitKeypress(wait, false);                              // wait ms. oder auf Taster warten
      if (IoT_Keypress())
      {
        return;
      }
    }
  }
  IoT_WaitKeypress(500, false);                                   // 500 ms. oder auf Taster warten
  if (IoT_Keypress())
  {
    return;
  }

  for (int k = 0 ; k < whiteLoops ; k ++)
  {
    for (int j = 0; j < 256 ; j++)
    {
      for (int i = 0; i < BRKclock_Pixel.numPixels(); i++)
      {
        BRKclock_Pixel.setPixelColor(i, BRKclock_Pixel.Color(0,0,0, BRKclock_Gamma[j]));
      }
      BRKclock_Show();                                            // Anzeige aktualisieren
    }
    IoT_WaitKeypress(1000, false);                                // 1 Sekunde oder auf Taster warten
    if (IoT_Keypress())
    {
      return;
    }
    for (int j = 255; j >= 0 ; j--)
    {
      for (int i = 0; i < BRKclock_Pixel.numPixels(); i++)
      {
        BRKclock_Pixel.setPixelColor(i, BRKclock_Pixel.Color(0,0,0, BRKclock_Gamma[j]));
      }
      BRKclock_Show();                                            // Anzeige aktualisieren
    }
  }
  IoT_WaitKeypress(500, false);                                   // 500 ms. oder auf Taster warten
}

void BRKclock_AmbienceColors (int wait)                           // Ambientebeleuchtung
{
  BRKclock_FullColor(BRKclock_Pixel.Color(0,255,0,0));            // Rot
  IoT_WaitKeypress(wait, false);                                  // wait ms. oder auf Taster warten
  if (IoT_Keypress())
  {
    return;
  }
  BRKclock_FullColor(BRKclock_Pixel.Color(255,0,0,0));            // Grün
  IoT_WaitKeypress(wait, false);                                  // wait ms. oder auf Taster warten
  if (IoT_Keypress())
  {
    return;
  }
  BRKclock_FullColor(BRKclock_Pixel.Color(0,0,255,0));            // Blau
  IoT_WaitKeypress(wait, false);                                  // wait ms. oder auf Taster warten
  if (IoT_Keypress())
  {
    return;
  }
  BRKclock_FullColor(BRKclock_Pixel.Color(0,0,0,255));            // Weiß
  IoT_WaitKeypress(wait, false);                                  // wait ms. oder auf Taster warten
  if (IoT_Keypress())
  {
    return;
  }
}

void BRKclock_RainbowCycle (byte wait)                            // Regenbogenzyklus
{
  for (int j = 0; j < 256 * 5; j++) // 5 cycles of all colors on wheel
  {
    for (int i = 0; i < BRKclock_Pixel.numPixels(); i++)
    {
      BRKclock_Pixel.setPixelColor(i, BRKclock_clockColorWheel(((i * 256 / BRKclock_Pixel.numPixels()) + j) & 255));
    }
    BRKclock_Show();                                              // Anzeige aktualisieren
    IoT_WaitKeypress(wait, false);                                // wait ms. oder auf Taster warten
    if (IoT_Keypress())
    {
      return;
    }
  }
}

void BRKclock_Rainbow (byte wait)
{
  for (int j = 0; j < 256; j++)
  {
    for (int i = 0; i < BRKclock_Pixel.numPixels(); i++)
    {
      BRKclock_Pixel.setPixelColor(i, BRKclock_clockColorWheel((i + j) & 255));
    }
    BRKclock_Show();                                              // Anzeige aktualisieren
    IoT_WaitKeypress(wait, false);                                // wait ms. oder auf Taster warten
    if (IoT_Keypress())
    {
      return;
    }
  }
}

void BRKclock_SnakeLines (byte wait)
{
  for (int i = 1; i <= 7; i++)
  {
    if (IoT_Keypress())
    {
      return;
    }
    BRKclock_clockColorWipe(BRKclock_Pixel.Color(t_RGBColor8ValueGreen[i],
                                            t_RGBColor8ValueRed[i],
                                            t_RGBColor8ValueBlue[i]), wait);
  }
}

