// DS18B20 Library (Temperatur-Sensor)
// 1.00 - 2017-06-16
// (c) Copyright 2017
//
// Zur Verwendung mit dem Allnet IoT-Brick


#include <OneWire.h>
#include <DallasTemperature.h>


// *** Globale Variablen ***

bool                 DS18B20_Inited                    = false;     // Variable für die Initialisierung der DS18B20 Benutzung
OneWire              DS18B20_oneWire(DS18B20_PIN);      
DallasTemperature    DS18B20_Sensor(&DS18B20_oneWire);
DeviceAddress        DS18B20_SensorAddr;


// *** Temperatur-Sensor Funktionen ***

bool DS18B20_Init (bool messages)
{
  DS18B20_Inited = false;
  DS18B20_Sensor.begin();
  int numberOfDevices = DS18B20_Sensor.getDeviceCount();
  if (numberOfDevices > 0)
  {
    if (DS18B20_Sensor.getAddress(DS18B20_SensorAddr, 0))
    {
      if (DS18B20_Sensor.validFamily(DS18B20_SensorAddr))
      {
        DS18B20_Sensor.setResolution(DS18B20_SensorAddr, 9);
        if (messages)
        {
          Serial.print("Valid Temperature Sensor found, resolution currently set to: ");
          Serial.println(str(DS18B20_Sensor.getResolution(DS18B20_SensorAddr)));
          Serial.print("Temperature is");
        }
        DS18B20_Sensor.requestTemperatures();                      // Temperatur anfordern. Ist notwendig, sonst erscheint immer 85°
        double t = DS18B20_Sensor.getTempC(DS18B20_SensorAddr);
        if (not(isnan(t)))
        {
          int TemperatureVal = int(t);
          if (TemperatureVal >= -100)
          {
            if (messages)
            { // 2 Nachkommastellen, mit Tausenderpunkt, mit ggf. Nullen nach dem Komma
              String Temperature = strrealform (t, 32, 1, 0, true, false) + "°C";
              Serial.println(": " + Temperature);
            }
            DS18B20_Inited = true;
          }
          else
          {
            if (messages)
            {
              Serial.println(" not valid."); // -127 = Disconnected
            }
          }
        }
        else
        {
          if (messages)
          {
            Serial.println(" invalid.");
          }
        }
      }
      else
      {
        if (messages)
        {
          Serial.println("Invalid Temperature Sensor found.");
        }
      }
    }
    else
    {
      if (messages)
      {
        Serial.println("Temperature Sensor does not respond.");
      }
    }
  }
  else
  {
    if (messages)
    {
      Serial.println("Temperature Sensor not found.");
    }
  }
  return (DS18B20_Inited);
}

double DS18B20_Temperature ()
{
  if (DS18B20_Inited)
  {
    DS18B20_Sensor.setResolution(DS18B20_SensorAddr, 9);
    DS18B20_Sensor.requestTemperatures();                      // Temperatur anfordern. Ist notwendig, sonst ändert sie sich nicht.
    return (DS18B20_Sensor.getTempC(DS18B20_SensorAddr));
  }
  else
  {
    return (-127);
  }
}
