// HTTP Library (Client, Server, Seitenaufbau)
// 1.00 - 2017-12-14
// (c) Copyright 2017
//
// Zur Verwendung mit allen Allnet Libraries

#if defined(ESP8266)    // ESP 8266 (32 Bit)
  ESP8266WebServer IoT_WebServer(80);                  // Webserver starten auf Port 80
#endif
#if defined(ESP32)
  WebServer        IoT_WebServer(80);                  // Webserver starten auf Port 80
#endif


// *** Web-Client (HTTP) des IoT-Bricks ***

String IoT_GetContentHTTP (String url, bool messages)
{
  String result = "";
  bool newLine = true;
  HTTPClient http;
  http.begin(url);
  IoT_WebServer.handleClient();                                     // Bediene die http Anfragen
  int httpErrorCode = http.GET();                                   // Verbindung aufbauen und HTTP-Header laden
  if (httpErrorCode > 0)  // httpCode will be negative on error
  {
    String payload = http.getString();
    int L = len(payload);
    if (messages)
    {
      Serial.println(fillcenter(" " + url + " ", "-", 80) + chr(8));
    }
    for (int i = 0; i < L; i++)
    {
      int c = payload[i];
      if (c == 10)
      {
        c = 13;
      }
      newLine = (c == 13);
      result += chr(c);
      if (messages)
      {
        Serial.print(chr(c));
      }
    }
    if (messages)
    {
      if (not(newLine))
      {
        Serial.println("");
      }
      Serial.println(fillcenter(" HTTP GET Result: " + str(httpErrorCode) + ", Length = " + str(L) + " ", "-", 80) + chr(8));
    }
  }
  else
  {
    if (messages)
    {
      String err = http.errorToString(httpErrorCode);
      Serial.println(fillcenter(" " + url + " ", "-", 80) + chr(8));
      Serial.println(fillcenter(" HTTP GET Result: " + str(httpErrorCode) + " (" + err  + "), Length = 0 ", "-", 80) + chr(8));
    }
  }
  http.end();
  return result;
}

String IoT_GetCurrencyECB (String currency)
{
  String ecburl = "http://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml";
  String s = IoT_GetContentHTTP(ecburl, false);
  String rate = "";
  if (len(s) > 100)                                                 // Daten wurden empfangen
  {
    int c = position(s, "currency='" + currency + "'", 1);
    if (c > 0)
    {
      int r = position(s, "rate='", c);
      if (r > 0)
      {
        int apo = position(s, "'", r + 6);
        if (apo > 0)
        {
          rate = mid(s, r + 6, apo - r - 6);
        }
      }
    }
  }
  return (rate);
}


// *** Web-Server (HTTP) des IoT-Bricks ***

void IoT_WebUpdate (String *html)
{
  #if defined(ESP8266)    // ESP 8266 (32 Bit)
    ESP.wdtFeed();                                                  // Watchdog Timer zurücksetzen
  #endif
  IoT_WebServer.handleClient();                                     // Bediene die http Anfragen
  IoT_WebServer.send(200, "text/html", *html);                      // HTTP-Statuscode 200 = OK (Anfrage erfolgreich)
  IoT_WebServer.handleClient();                                     // Bediene die http Anfragen
  #if defined(ESP8266)    // ESP 8266 (32 Bit)
    ESP.wdtFeed();                                                  // Watchdog Timer zurücksetzen
  #endif
}

void IoT_WebPrintAllFields ()
{
  String s = "";
  byte argumentCount = IoT_WebServer.args();                        // Anzahl der Felder auf der Web-Seite
  if (argumentCount > 0)                                            // Ist überhaupt eine Eingabe vorhanden?
  {
    for (byte i = 0; i < argumentCount; i++)                        // Alle erlaubten Eingabefelder durchsuchen
    {
      Serial.print("Field #" + strform(i, 48, 2, true) + ": ");     // Die Ausgabe von Unicodes stört die Terminal-Ausgabe,
      s = cleanasc(IoT_WebServer.argName(i));                       // daher hier alle Unicode-Zeichen sinnvoll ersetzen
      Serial.print(left(s + spc(16), 16));                          // Feldname ausgeben, max. 16 Zeichen (Asc II)
      Serial.print(" = " + chr(34));                                // Feldinhalt in Anführungszeichen ausgeben,
      Serial.print(cleanasc(IoT_WebServer.arg(i)));                 // Alle Unicode-Zeichen sinnvoll ersetzen                                      
      Serial.println(chr(34));                                      // max. 32 Zeichen (Asc II)
    }
  }
  #if defined(ESP8266)                                              // ESP 8266 (32 Bit)
    ESP.wdtFeed();                                                  // Watchdog Timer zurücksetzen
  #endif
}

bool IoT_WebTestField (String varName)                              // Ergibt true, wenn ein Web-Feld mit dem Namen existiert
{
  byte argumentCount = IoT_WebServer.args();                        // Anzahl der Felder auf der Web-Seite
  if (argumentCount > 0)                                            // Ist überhaupt eine Eingabe vorhanden?
  {
    for (byte i = 0; i < argumentCount; i++)                        // Alle erlaubten Eingabefelder durchsuchen
    {
      if (IoT_WebServer.argName(i) == varName)                      // Das Feld mit dem angegebenen varName suchen
      {
         return (true);                                             // Feld auslesen und als Funktionsergebnis übergeben
      }
     }
  }
  return (false);
}

String IoT_WebGetField (String varName)                             // Ergibt den Inhalt des Web-Feldes
{
  byte argumentCount = IoT_WebServer.args();                        // Anzahl der Felder auf der Web-Seite
  if (argumentCount > 0)                                            // Ist überhaupt eine Eingabe vorhanden?
  {
    for (byte i = 0; i < argumentCount; i++)                        // Alle erlaubten Eingabefelder durchsuchen
    {
      if (IoT_WebServer.argName(i) == varName)                      // Das Feld mit dem angegebenen varName suchen
      {
         return (IoT_WebServer.arg(i));                             // Feld auslesen und als Funktionsergebnis übergeben
      }
    }
  }
  return ("");
}


// *** Web-Seitenaufbau ***

String IoT_WebFormOpen (String formName)
{
  String s = "<form action = 'http://" + strIP(WiFi.localIP(), false) + "/" + formName + "' method='POST'>";
  return (s);
}

String IoT_WebFormClose ()
{
  String s = "</form>";
  return (s);
}

String IoT_WebFormSubmitButton (String formName)
{
  String s = "<input type='submit' value='" + cleanhtml(formName) + "'>";
  return (s);
}

String IoT_WebFormActionButton (String actionName, String title)
{
  String s = IoT_WebFormOpen(actionName) + IoT_WebFormSubmitButton(title) + IoT_WebFormClose();
  return (s);
}

String IoT_WebCheckBox (String title, String varName, String varValue, bool checked)
{
  String s = "<input type='checkbox' name='" + varName + "' value='" + varValue + "'";
  if (checked)
  {
    s += " checked";
  }
  s += ">";
  s += cleanhtml(title);
  return (s);
}

String IoT_WebRadioButton (String title, int varIndex, String varName, String varValue, int selectedIndex)
{
  String s = "<input type='radio' id='" + varName + strform(varIndex, 48, 3, false) + "' name='" + varName + "' value='" + varValue + "'";
  if (varIndex == selectedIndex)
  {
    s += " checked";
  }
  s += ">";
  s += cleanhtml(title);
  return (s);
}

String IoT_WebInput (String title, String varName, String varValue, int width)
{
  String s = cleanhtml(title) + "<input type='text' name='" + varName + "' value='" + varValue + "' size='" + str(width) + "'>";
  return (s);
}

String IoT_WebClientIP ()
{
  String s = "<script type='text/javascript' src='https://l2.io/ip.js'></script>";
  return (s);
}

String IoT_WebHtab (int h)
{
  String s = "<span style='padding-left:" + String(h) + "px;'></span>";
  return (s);
}
