// MQTT Library
// 1.00 - 2017-07-04
// 1.01 - 2017-11-15
// (c) Copyright 2017
//
// Zur Verwendung mit allen Allnet Libraries

#include <PubSubClient.h>
#include <Client.h>

#define MQTTlibrary

// *** Allgemeine Globale Variablen ***

WiFiClient   MQTT_WifiClient;                                       // Der MQTT-Wifi-Client für den Verbindungsaufbau
PubSubClient MQTT_Client (MQTT_WifiClient);                         // Der MQTT-Kommunikations-Client

String MQTT_ServerURL           = "";                               // MQTT Server URL, z.B. "iot.allnet.de"
uint16 MQTT_ServerPort          = 0;                                // MQTT Server Port, normalerweise 1883
String MQTT_ServerClientID      = "";                               // Eine zufällige Unique-ID
String MQTT_ServerUserName      = "";                               // Benutzer-Name
String MQTT_ServerUserPW        = "";                               // Benutzer-Password
String MQTT_ServerTopic         = "";                               // Das Thema (als Pfad), das benutzt werden soll

String MQTT_EventQueue          = "";                               // MQTT Event-Warteschlange
bool   MQTT_EventAvail          = false;                            // Wird true, sobald ein Event verfügbar ist
uint32 MQTT_EventCurrentTime    = 0;                                // Millisekunden-Zeit des letzten Events oder 0 = Kein Event
String MQTT_EventCurrentTopic   = "";                               // Das Thema (als Pfad) des letzten Events
String MQTT_EventCurrentMessage = "";                               // Die Mitteilung zum Thema des letzten Events
bool   MQTT_EventBusy           = false;                            // True = Der Event-Handler ist grade aktiv, MQTT_EventQueue nicht verfügbar
bool   MQTT_EventTrace          = false;                            // True = Jeden Event sobald er ankommt auf dem Terminal ausgeben
bool   MQTT_ConnectTrace        = true;                             // True = Jeden Connect-Versuch auf dem Terminal ausgeben


// *** Globale Variablen für Werte vom Allnet-MQTT-Server als Fixkomma in 1/100 °C ***

int16  MQTT_AllnetTemperaturHamburg      = 0;
int16  MQTT_AllnetTemperaturBerlin       = 0;
int16  MQTT_AllnetTemperaturFrankfurt    = 0;
int16  MQTT_AllnetTemperaturStuttgart    = 0;
int16  MQTT_AllnetTemperaturHannover     = 0;
int16  MQTT_AllnetTemperaturMuenchen     = 0;
int16  MQTT_AllnetTemperaturKoeln        = 0;
int16  MQTT_AllnetTemperaturDuesseldorf  = 0;
int16  MQTT_AllnetTemperaturNuernberg    = 0;
int16  MQTT_AllnetTemperaturDresden      = 0;
int16  MQTT_AllnetTemperaturNaumburg     = 0;
int16  MQTT_AllnetTemperaturHeidelberg   = 0;
int16  MQTT_AllnetTemperaturCottbus      = 0;
int16  MQTT_AllnetTemperaturSchwerin     = 0;
int16  MQTT_AllnetTemperaturMainz        = 0;
int16  MQTT_AllnetTemperaturWiesbaden    = 0;
int16  MQTT_AllnetTemperaturBremen       = 0;
int16  MQTT_AllnetTemperaturDortmund     = 0;
int16  MQTT_AllnetTemperaturKiel         = 0;
int16  MQTT_AllnetTemperaturLeipzig      = 0;
int16  MQTT_AllnetTemperaturRegensburg   = 0;
int16  MQTT_AllnetTemperaturRostock      = 0;
int16  MQTT_AllnetTemperaturSaarbruecken = 0;
int16  MQTT_AllnetTemperaturTrier        = 0;
int16  MQTT_AllnetTemperaturUlm          = 0;
int16  MQTT_AllnetTemperaturWuerzburg    = 0;
int16  MQTT_AllnetTemperaturOldenburg    = 0;
int16  MQTT_AllnetTemperaturPotsdam      = 0;
int16  MQTT_AllnetTemperaturMagdeburg    = 0;


// *** Globale Variablen für Werte vom Allnet-MQTT-Server ***

double MQTT_AllnetFinanceDaxVal          = 0.0;
double MQTT_AllnetFinanceMDaxVal         = 0.0;
double MQTT_AllnetFinanceEStx50Val       = 0.0;
double MQTT_AllnetFinanceGoldVal         = 0.0;
double MQTT_AllnetFinanceOelVal          = 0.0;

double MQTT_AllnetFinanceUSDVal          = 0.0;  // US Dollar in EUR
double MQTT_AllnetFinanceCHFVal          = 0.0;  // Schweizer Franken in EUR
double MQTT_AllnetFinanceGBPVal          = 0.0;  // Britisches Pfund in EUR
double MQTT_AllnetFinanceJPYVal          = 0.0;  // Japanischer Yen in EUR
double MQTT_AllnetFinanceCNYVal          = 0.0;  // Chinesischer Renminbi Yuan in EUR

double MQTT_AllnetFinanceBTCVal          = 0.0;  // Bitcoin
double MQTT_AllnetFinanceETHVal          = 0.0;  // Ethereum
double MQTT_AllnetFinanceBCHVal          = 0.0;  // Bitcoin Cash
double MQTT_AllnetFinanceLTCVal          = 0.0;  // Litecoin


// *** MQTT-Funktionen ***

void MQTT_ClearEvents ()
{
  while (MQTT_EventBusy)    // Neuer Event wird gerade empfangen
  {
    delay(1);               // 1 Millisekunde warten
  }
  MQTT_EventQueue = "";
  MQTT_EventAvail = false;
  MQTT_EventCurrentTime = 0;
  MQTT_EventCurrentTopic = "";
  MQTT_EventCurrentMessage = "";
}

void MQTT_CallbackHandler (char* topic, byte* payload, unsigned int length)
{
  MQTT_EventBusy = true;
  uint32 m = micros();
  String Topic = cleanasc(String(topic));
  String Message = "";
  for (int i = 0; i < length; i++)
  {
    if (payload[i] != 3) // Ctrl-C ist nicht erlaubt, Event-Record-Trennzeichen
    {
      Message += (char)payload[i];
    }
  }
  if (MQTT_EventTrace)
  {
    Serial.print("Event " + String(m) + " [" + Topic + "] ");
    Serial.println(left(Message, 50));
    int L = MQTT_EventQueue.length();
    if (L > 0)
    {
      Serial.println("Total length of event-queue = " + str(L));
    }
  }
  MQTT_EventAvail = true;
  MQTT_EventQueue += String(m);
  MQTT_EventQueue += chr(9);
  MQTT_EventQueue += Topic;
  MQTT_EventQueue += chr(13);
  MQTT_EventQueue += Message;
  MQTT_EventQueue += chr(3);
  MQTT_EventBusy = false;
}

void MQTT_Disconnect ()
{
  MQTT_Client.disconnect();
}

void MQTT_Init (String serverURL, uint16 serverPort, String clientID, String userName, String userPassword)
{
  MQTT_ServerURL      = serverURL;
  MQTT_ServerPort     = serverPort;
  MQTT_ServerClientID = clientID;
  MQTT_ServerUserName = userName;
  MQTT_ServerUserPW   = userPassword;

  MQTT_Disconnect();
  MQTT_Client.setServer  (MQTT_ServerURL.c_str(), MQTT_ServerPort);
  MQTT_Client.setCallback(MQTT_CallbackHandler);
  MQTT_ClearEvents();
}

void MQTT_Init (IPAddress ipAdr, uint16 serverPort, String clientID, String userName, String userPassword)
{
  MQTT_ServerURL      = strIP(ipAdr, false);
  MQTT_ServerPort     = serverPort;
  MQTT_ServerClientID = clientID;
  MQTT_ServerUserName = userName;
  MQTT_ServerUserPW   = userPassword;

  MQTT_Disconnect();
  MQTT_Client.setServer  (ipAdr, MQTT_ServerPort);
  MQTT_Client.setCallback(MQTT_CallbackHandler);
  MQTT_ClearEvents();
}

int MQTT_GetNextEvent () // 0 = Kein Event vorhanden, 1 = Event erfolgreich gelesen, <0 = Fehlercodes
{
  if (MQTT_EventAvail)
  {
    while (MQTT_EventBusy)    // Neuer Event wird gerade empfangen
    {
      delay(1);               // 1 Millisekunde warten
    }
    int m = position(MQTT_EventQueue, chr(9), 1);
    if (m > 0)
    {
      MQTT_EventCurrentTime = val(mid(MQTT_EventQueue, 1, m - 1));
      int t = position(MQTT_EventQueue, chr(13), m + 1);
      if (t > 0)
      {
        MQTT_EventCurrentTopic = mid(MQTT_EventQueue, m + 1, t - m - 1);
        int p = position(MQTT_EventQueue, chr(3), t + 1);
        if (p > 0)
        {
          MQTT_EventCurrentMessage = mid(MQTT_EventQueue, t + 1, p - t - 1);
          MQTT_EventQueue = part(MQTT_EventQueue, p + 1);
          MQTT_EventAvail = (MQTT_EventQueue != "");
          return (1);
        }
        else
        {
          return (-3); // Error: Event-Message nicht gefunden
        }
      }
      else
      {
        return (-2); // Error: Event-Topic nicht gefunden
      }
    }
    else
    {
      return (-1); // Error: Event-Time nicht gefunden
    }
  }
  else
  {
    return (0);
  }
}

bool MQTT_Connect (String topic)
{
  if (MQTT_ConnectTrace)
  {
    Serial.print("Looking for MQTT server '" + MQTT_ServerURL + "'... ");
  }
  bool success = false;
  MQTT_Client.disconnect();
  if ((MQTT_ServerUserName == "") && (MQTT_ServerUserPW == ""))
  {
    success = MQTT_Client.connect(MQTT_ServerClientID.c_str());
  }
  else
  {
    success = MQTT_Client.connect(MQTT_ServerClientID.c_str(), MQTT_ServerUserName.c_str(),  MQTT_ServerUserPW.c_str());
  }
  if (success)
  {
    if (MQTT_ConnectTrace)
    {
      Serial.println("Connected.");
    }
    MQTT_ServerTopic = topic;
    MQTT_Client.subscribe(MQTT_ServerTopic.c_str());
   }
  else
  {
    if (MQTT_ConnectTrace)
    {
      Serial.println("");
      Serial.print("MQTT server not connected. RC = ");
      Serial.println(MQTT_Client.state());
   }
  }
  return (success);
}

bool MQTT_HandleClient ()
{
  if (MQTT_Client.connected())
  {
    MQTT_Client.loop();
    return (true);
  }
  else
  {
    bool success = MQTT_Connect(MQTT_ServerTopic);
    MQTT_Client.loop();
    return (success);
  }
}

bool MQTT_AllnetParseEventCurrent ()
{
  bool result = true;
  double v = realval(MQTT_EventCurrentMessage);
  //Serial.println("MQTT_AllnetParseEventCurrent: " + MQTT_EventCurrentTopic + " = '" + MQTT_EventCurrentMessage + "', v = " + strreal(v, 0));
  if (MQTT_EventCurrentTopic      == "esp/weather/condition/hamburg")
  {
    MQTT_AllnetTemperaturHamburg = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/berlin")
  {
    MQTT_AllnetTemperaturBerlin = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/frankfurt")
  {
    MQTT_AllnetTemperaturFrankfurt = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/stuttgart")
  {
    MQTT_AllnetTemperaturStuttgart = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/hannover")
  {
    MQTT_AllnetTemperaturHannover = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/muenchen")
  {
    MQTT_AllnetTemperaturMuenchen = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/koeln")
  {
    MQTT_AllnetTemperaturKoeln = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/duesseldorf")
  {
    MQTT_AllnetTemperaturDuesseldorf = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/nuernberg")
  {
    MQTT_AllnetTemperaturNuernberg = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/dresden")
  {
    MQTT_AllnetTemperaturDresden = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/naumburg")
  {
    MQTT_AllnetTemperaturNaumburg = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/heidelberg")
  {
    MQTT_AllnetTemperaturHeidelberg = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/cottbus")
  {
    MQTT_AllnetTemperaturCottbus = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/schwerin")
  {
    MQTT_AllnetTemperaturSchwerin = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/mainz")
  {
    MQTT_AllnetTemperaturMainz = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/wiesbaden")
  {
    MQTT_AllnetTemperaturWiesbaden = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/bremen")
  {
    MQTT_AllnetTemperaturBremen = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/dortmund")
  {
    MQTT_AllnetTemperaturDortmund = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/kiel")
  {
    MQTT_AllnetTemperaturKiel = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/leipzig")
  {
    MQTT_AllnetTemperaturLeipzig = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/regensburg")
  {
    MQTT_AllnetTemperaturRegensburg = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/rostock")
  {
    MQTT_AllnetTemperaturRostock = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/saarbruecken")
  {
    MQTT_AllnetTemperaturSaarbruecken = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/trier")
  {
    MQTT_AllnetTemperaturTrier = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/ulm")
  {
    MQTT_AllnetTemperaturUlm = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/wuerzburg")
  {
    MQTT_AllnetTemperaturWuerzburg = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/oldenburg")
  {
    MQTT_AllnetTemperaturOldenburg = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/potsdam")
  {
    MQTT_AllnetTemperaturPotsdam = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/weather/condition/magdeburg")
  {
    MQTT_AllnetTemperaturMagdeburg = int(v * 100);
  }
  else if (MQTT_EventCurrentTopic == "esp/finance/exchange/dax")
  {
    MQTT_AllnetFinanceDaxVal = v;
  }
  else if (MQTT_EventCurrentTopic == "esp/finance/exchange/mdax")
  {
    MQTT_AllnetFinanceMDaxVal = v;
  }
  else if (MQTT_EventCurrentTopic == "esp/finance/exchange/estx50")
  {
    MQTT_AllnetFinanceEStx50Val = v;
  }
  else if (MQTT_EventCurrentTopic == "esp/finance/exchange/gold")
  {
    MQTT_AllnetFinanceGoldVal = v;
  }
  else if (MQTT_EventCurrentTopic == "esp/finance/exchange/oel")
  {
    MQTT_AllnetFinanceOelVal = v;
  }
  else if (MQTT_EventCurrentTopic == "esp/finance/currency/EURUSD")
  {
    MQTT_AllnetFinanceUSDVal = v;
  }
  else if (MQTT_EventCurrentTopic == "esp/finance/currency/EURCHF")
  {
    MQTT_AllnetFinanceCHFVal = v;
  }
  else if (MQTT_EventCurrentTopic == "esp/finance/currency/EURGBP")
  {
    MQTT_AllnetFinanceGBPVal = v;
  }
  else if (MQTT_EventCurrentTopic == "esp/finance/currency/EURJPY")
  {
    MQTT_AllnetFinanceJPYVal = v;
  }
  else if (MQTT_EventCurrentTopic == "esp/finance/currency/EURCNY")
  {
    MQTT_AllnetFinanceCNYVal = v;
  }
  else if (MQTT_EventCurrentTopic == "esp/finance/crypto/BTCEUR")
  {
    MQTT_AllnetFinanceBTCVal = v;
  }
  else if (MQTT_EventCurrentTopic == "esp/finance/crypto/ETHEUR")
  {
    MQTT_AllnetFinanceETHVal = v;
  }
  else if (MQTT_EventCurrentTopic == "esp/finance/crypto/BCHEUR")
  {
    MQTT_AllnetFinanceBCHVal = v;
  }
  else if (MQTT_EventCurrentTopic == "esp/finance/crypto/LTCEUR")
  {
    MQTT_AllnetFinanceLTCVal = v;
  }
  else
  {
    result = false;
  }
  return (result);
}
