// Mail Library
// 1.00 - 2017-07-27
// (c) Copyright 2017
//
// Zur Verwendung mit allen Allnet Libraries


// *** Globale Variablen ***

WiFiClient mail_Client;

bool   mail_ErrorTrace        = true;                             // True = Jeden Fehler auf dem Terminal ausgeben
bool   mail_MessageTrace      = false;                            // True = Jede Send-Mitteilung auf dem Terminal ausgeben
bool   mail_FeedbackTrace     = false;                            // True = Jedes SMTP-Feedback auf dem Terminal ausgeben


// *** Mail Funktionen ***

String mail_ErrorString (int errorCode)
{
  switch (errorCode)
  {
    case 0: // Kein Fehler
      return ("");
      break;
    case 1: // Fehlermeldungen von mail_Status
      return ("Timeout after 10 seconds");
      break;
    case 2:
      return ("Error & Timeout after 10 seconds");
      break;
    case 3:
      return ("Disconnected");
      break;
    case -1: // Fehlermeldungen von mail_Send
      return ("Connection to SMTP server failed");
      break;
    case -2:
      return ("Connection lost before EHLO");
      break;
    case -3:
      return ("Connection lost after EHLO");
      break;
    case -4:
      return ("Connection lost after auth login");
      break;
    case -5:
      return ("Connection lost after user name");
      break;
    case -6:
      return ("Connection lost after user password");
      break;
    case -7:
      return ("Connection lost after sender");
      break;
    case -8:
      return ("Connection lost after recipient");
      break;
    case -9:
      return ("Connection lost after 'DATA'");
      break;
    case -10:
      return ("Connection lost after sending header & body");
      break;
    case -11:
      return ("Connection lost after 'QUIT'");
      break;
    default:
      return ("Unknown error");
      break;
  }
}

void mail_ErrorPrint (int errorCode)
{
  if (mail_ErrorTrace)
  {
    Serial.println(mail_ErrorString(errorCode) + ".");
  }
}

void mail_MessagePrint (String message)
{
  if (mail_MessageTrace)
  {
    Serial.println(message + ".");
  }
}

byte mail_Status ()
{
  int loopCount = 0;
  
  while (not(mail_Client.available()))
  {
    delay(1);
    loopCount++; // 10 Sekunden warten, bis ein TimeOut eintritt
    if (loopCount > 10000)
    {
      mail_Client.stop();
      mail_ErrorPrint(1);
      return (1);
    }
  }
  
  while (mail_Client.available())
  {  
    byte c = mail_Client.read();
    if (mail_FeedbackTrace)
    {
      Serial.write(c);
    }
  }
  
  if (mail_Client.peek() >= '4') // Fehler ist aufgetreten
  {
    loopCount = 0;
    mail_Client.println("QUIT");
    
    while (not(mail_Client.available()))
    {
      delay(1);
      loopCount++; // 10 Sekunden warten, bis ein TimeOut eintritt
      if (loopCount > 10000)
      {
        mail_Client.stop();
        mail_ErrorPrint(2);
        return (2);
      }
    }
    
    while (mail_Client.available())
    {  
      byte c = mail_Client.read();
      if (mail_FeedbackTrace)
      {
        Serial.write(c);
      }
    }
    
    mail_Client.stop();
    mail_ErrorPrint(3);
    return (3); 
  }
  
  return (0);
}

int mail_Send (String SMTPserver, int SMTPport, String userName, String userPassword,
               String senderName, String senderMail, String recipientName, String recipientMail,
               String subject, String body)
{ 
  if (mail_Client.connect(SMTPserver.c_str(), SMTPport) == 1)
  {
    mail_MessagePrint("Connected to '" + SMTPserver + "'");
  }
  else
  {
    mail_ErrorPrint(-1);
    return (-1);
  }
  
  if (mail_Status() != 0)
  {
    mail_ErrorPrint(-2);
    return (-2);
   }

  mail_MessagePrint("Sending 'EHLO'");
  mail_Client.println("EHLO mail.iot-brick.de"); // Beliebiger "fully-qualified hostname"
  if (mail_Status() != 0)
  {
    mail_ErrorPrint(-3);
    return (-3);
   }

  mail_MessagePrint("Sending 'auth login'");
  mail_Client.println("auth login");
  if (mail_Status() != 0)
  {
    mail_ErrorPrint(-4);
    return (-4);
  }

  mail_MessagePrint("Sending user name");
  mail_Client.println(converttobase64(userName));
  if (mail_Status() != 0)
  {
    mail_ErrorPrint(-5);
    return (-5);
  }

  mail_MessagePrint("Sending user password");
  mail_Client.println(converttobase64(userPassword));
  if (mail_Status() != 0)
  {
    mail_ErrorPrint(-6);
    return (-6);
  }

  mail_MessagePrint("Sending sender name & mail");
  mail_Client.println("MAIL From: <" + senderMail + ">");
  if (mail_Status() != 0)
  {
    mail_ErrorPrint(-7);
    return (-7);
  }

  mail_MessagePrint("Sending recipient name & mail");
  mail_Client.println("RCPT To: <" + recipientMail + ">");
  if (mail_Status() != 0)
  {
    mail_ErrorPrint(-8);
    return (-8);
  }

  mail_MessagePrint("Sending 'DATA'");
  mail_Client.println("DATA");
  if (mail_Status() != 0)
  {
    mail_ErrorPrint(-9);
    return (-9);
  }

  mail_MessagePrint("Sending email header & email body");
  mail_Client.println("To: " + cleanasc(recipientName) + " <" + recipientMail + ">");
  mail_Client.println("From: " + cleanasc(senderName) + " <" + senderMail + ">");
  mail_Client.println("Subject: " + cleanasc(subject));
  mail_Client.println(body);
  mail_Client.println(".");         // Ende der Daten (End data with <CR><LF>.<CR><LF>)
  if (mail_Status() != 0)
  {
    mail_ErrorPrint(-10);
    return (-10);
  }

  mail_MessagePrint("Sending 'QUIT'");
  mail_Client.println("QUIT");
  if (mail_Status() != 0)
  {
    mail_ErrorPrint(-11);
    return (-11);
  }
  
  mail_Client.stop(); // Mail Client Disconnected.
  return (0);
}
