// Matrix Library
// 1.00 - 2017-10-18
// 1.01 - 2017-11-02
// 1.02 - 2017-11-15
// (c) Copyright 2017
//
// Zur Verwendung mit allen Allnet Libraries


#if defined(ESP32)                                              // ESP32
  #define matrix_PIN 18                                         // ESP32:   Led Pin an GPIO 18
#else
  #define matrix_PIN 13                                         // ESP8266: Led Pin an GPIO 13
#endif
#define DHT11_PIN 14                                            // Datenleitung DHT11-Sensors an GPIO14

#include <Adafruit_NeoPixel.h>
#include <all_DHT11.h>
#include <all_ALL3500.h>                                        // Beinhaltet #include <ESP8266HTTPClient.h>


// *** Globale Typen ***

struct matrix_ColorType
{
    byte  FontRed         = 0;                                  // RGBW-Farbe der Schrift für die Module
    byte  FontGreen       = 0;
    byte  FontBlue        = 0;
    byte  FontWhite       = 0;
    byte  SeparatorRed    = 0;                                  // RGBW-Farbe der Trennzeichen für die Module
    byte  SeparatorGreen  = 0;
    byte  SeparatorBlue   = 0;
    byte  SeparatorWhite  = 0;
    byte  Brightness      = 0;
};

struct matrix_Parameter
{
//    8 Bytes Parameter-Header:
    int64 parameterID       = 0;                                       // Kennung des Parameter-Blocks "ALmatrix" im Klartext
    int64 parameterHash     = 0;                                       // Kennung des Parameter-Blocks "ALmatrix" als verschlüsselter Hash
    
//    Start des Parameter-Blocks:
    byte   IP_address0      = 0;                                       // 4 Bytes IP-Adresse
    byte   IP_address1      = 0;
    byte   IP_address2      = 0;
    byte   IP_address3      = 0;
    byte   IP_gateway0      = 0;                                       // 4 Bytes Gateway
    byte   IP_gateway1      = 0;
    byte   IP_gateway2      = 0;
    byte   IP_gateway3      = 0;
    byte   IP_subnet0       = 0;                                       // 4 Bytes Subnetzmaske
    byte   IP_subnet1       = 0;
    byte   IP_subnet2       = 0;
    byte   IP_subnet3       = 0;
    byte   staticIP         = 0;                                       // 0 = Dynamische IP-Adresse, 1 = Statische IP-Adresse
};


// *** Globale Variablen ***

Adafruit_NeoPixel matrix_Pixel = Adafruit_NeoPixel(512, matrix_PIN, NEO_RGBW + NEO_KHZ800);

matrix_ColorType  matrix_Color;                                        // Farbeinstellung
matrix_Parameter  matrix_ParameterData;                                // Parameter Block für die Matrix

byte              matrix_StartIndex = 0;                               // Der Index der anzuzeigenden Matrix (0...15)
byte              matrix_VirtualScreen[32];                            // 16 Positionen á 2 Bytes für 16 virtuelle 16x16 Matrices
String            matrix_VirtualHeader[32];                            // 32 Titel für 16 virtuelle 16x16 Matrices
bool              matrix_FlashingColon = true;                         // Blinkender Doppelpunkt
bool              matrix_ShowModules = false;                          // ALL-3500 Module in der Konfigurationsseite anzeigen
byte              matrix_StaticIP = 0;                                 // 0 = Dynamische IP-Adresse, 1 = Statische IP-Adresse
String            matrix_ID = "ESP_Matrix " + left(newuniqueid(), 12); // Eindeutigen Namen erzeugen
String            matrix_ALL3500URL = "";                              // z.B.: http://user:pw@192.168.1.64/xml/json.php?mode=all


// *** Matrix Funktionen ***

void matrix_Plot (int x, int y, byte red, byte green, byte blue, byte white)
{
  if ((x >= 0) && (x <= 31) && (y >= 0) && (y <= 15))
  {
    bool RightPanel = false;
    int p = 0;
    y = 0xF ^ y;                                                // Oben ist Zeile 0
    if (x > 0xF)                                                // Rechte 16x16 Matrix
    {
      x -= 0x10;
      RightPanel = true;
    }
    if (y & 1 == 1)
    {
      p = (y + 1) * 0x10 - x - 1;
    }
    else
    {
      p = y * 0x10 + x;
    }
    if (RightPanel)                                             // Rechte 16x16 Matrix
    {
       p += 0x100;
    }
    matrix_Pixel.setPixelColor(p, green, red, blue, white);     // Punkt in der angegebenen Farbe setzen
  }
}

void matrix_Plot (int x, int y, byte red, byte green, byte blue)
{
  matrix_Plot(x, y, red, green, blue, 0);
}

void matrix_Line (int x1, int y1, int x2, int y2, byte red, byte green, byte blue, byte white)
{
  int Steps = maximum(abs(x2 - x1), abs(y2 - y1));
  if (Steps > 0)                                                // Mehr als nur einen einzelnen Punkt erzeugen
  {
    double IncX = double(x2 - x1) / double(Steps);
    double IncY = double(y2 - y1) / double(Steps);
    double x = double(x1);
    double y = double(y1);
    for (int i = 1; i <= Steps; i++)
    {
      matrix_Plot(int(x + 0.5), int(y + 0.5), red, green, blue, white);
      x = x + IncX;
      y = y + IncY;
    }
  }
  matrix_Plot(x2, y2, red, green, blue, white);
}

void matrix_Line (int x1, int y1, int x2, int y2, byte red, byte green, byte blue)
{
  matrix_Line(x1, y1, x2, y2, red, green, blue, 0);
}

void matrix_Rect (int x1, int y1, int x2, int y2, byte red, byte green, byte blue, byte white)
{
  matrix_Line(x1, y1, x2, y1, red, green, blue, white);
  matrix_Line(x2, y1, x2, y2, red, green, blue, white);
  matrix_Line(x2, y2, x1, y2, red, green, blue, white);
  matrix_Line(x1, y2, x1, y1, red, green, blue, white);
}

void matrix_Rect (int x1, int y1, int x2, int y2, byte red, byte green, byte blue)
{
  matrix_Rect(x1, y1, x2, y2, red, green, blue, 0);
}

void matrix_FillRect (int x1, int y1, int x2, int y2, byte red, byte green, byte blue, byte white)
{
  if (y1 > y2)                                                  // y1 muss kleiner oder gleich y2 sein
  {
    y1 = y1 ^ y2;                                               // y1 und y2 vertauschen ohne Hilfsvariable
    y2 = y1 ^ y2;
    y1 = y1 ^ y2;
  }
  for (int i = y1; i <= y2; i++)
  {
    matrix_Line(x1, i, x2, i, red, green, blue, white);
  }
}

void matrix_FillRect (int x1, int y1, int x2, int y2, byte red, byte green, byte blue)
{
  matrix_FillRect(x1, y1, x2, y2, red, green, blue, 0);
}

void matrix_DrawChar (byte c, int x, int y, byte red, byte green, byte blue, byte white, bool transparency)
{
  if (not(transparency))
  {
    matrix_FillRect(x, y, x + 3, y + 4, 0, 0, 0, 0);
  }
  if ((c >= 32) && (c <= 95))                                   // Druckbare Zeichen " " bis "_"
  {
    if ((c >= 32) && (c <= 47))                                 // Druckbare Zeichen " " bis "/"
    {
      switch (c)
      {
        case 32:                                                // " "
          break;
        case 33:                                                // "!"
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 1, y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          break;
        case 34:                                                // """
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          break;
        case 35:                                                // "#"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 36:                                                // "$"
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 1, y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
         break;
        case 37:                                                // "%"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 38:                                                // "&"
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 39:                                                // "'"
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 1, y + 1, red, green, blue, white);
          break;
        case 40:                                                // "("
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          break;
        case 41:                                                // ")"
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          break;
        case 42:                                                // "*"
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          break;
        case 43:                                                // "+"
          matrix_Plot(x + 1, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          break;
        case 44:                                                // ","
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          break;
        case 45:                                                // "-"
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          break;
        case 46:                                                // "."
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          break;
        case 47:                                                // "/"
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          break;
        default:
          break;
      }
    }
    else if ((c >= 48) && (c <= 63))                            // Druckbare Zeichen "0" bis "?"
    {
      switch (c)
      {
        case 48:                                                // "0"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 49:                                                // "1"
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 50:                                                // "2"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 51:                                                // "3"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 52:                                                // "4"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
         break;
        case 53:                                                // "5"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 54:                                                // "6"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 55:                                                // "7"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 56:                                                // "8"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 57:                                                // "9"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 58:                                                // ":"
          matrix_Plot(x + 1, y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          break;
        case 59:                                                // ";"
          matrix_Plot(x + 1, y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          break;
        case 60:                                                // "<"
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x + 1, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 61:                                                // "="
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          break;
        case 62:                                                // ">"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          break;
        case 63:                                                // "?"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          break;
        default:
          break;
      }
    }
    else if ((c >= 64) && (c <= 79))                            // Druckbare Zeichen "@" bis "O"
    {
      switch (c)
      {
        case 64:                                                // "@"
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 65:                                                // "A"
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
         break;
        case 66:                                                // "B"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
         break;
        case 67:                                                // "C"
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 68:                                                // "D"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
         break;
        case 69:                                                // "E"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 70:                                                // "F"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          break;
        case 71:                                                // "G"
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          break;
        case 72:                                                // "H"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 73:                                                // "I"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x + 1, y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 74:                                                // "J"
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          break;
        case 75:                                                // "K"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 76:                                                // "L"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 77:                                                // "M"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 78:                                                // "N"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 79:                                                // "O"
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          break;
        default:
          break;
      }
    }
    else if ((c >= 80) && (c <= 95))                            // Druckbare Zeichen "P" bis "_"
    {
      switch (c)
      {
        case 80:                                                // "P"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          break;
        case 81:                                                // "Q"
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
         break;
        case 82:                                                // "R"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
         break;
        case 83:                                                // "S"
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          break;
        case 84:                                                // "T"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x + 1, y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
         break;
        case 85:                                                // "U"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 86:                                                // "V"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          break;
        case 87:                                                // "W"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 88:                                                // "X"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 89:                                                // "Y"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 1, y + 3, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          break;
        case 90:                                                // "Z"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 91:                                                // "["
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x    , y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 92:                                                // "\"
          matrix_Plot(x    , y + 1, red, green, blue, white);
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          break;
        case 93:                                                // "]"
          matrix_Plot(x    , y    , red, green, blue, white);
          matrix_Plot(x + 1, y    , red, green, blue, white);
          matrix_Plot(x + 2, y    , red, green, blue, white);
          matrix_Plot(x + 2, y + 1, red, green, blue, white);
          matrix_Plot(x + 2, y + 2, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        case 94:                                                // "^"
          matrix_Plot(x + 1, y + 2, red, green, blue, white);
          matrix_Plot(x    , y + 3, red, green, blue, white);
          matrix_Plot(x + 2, y + 3, red, green, blue, white);
          break;
        case 95:                                                // "_"
          matrix_Plot(x    , y + 4, red, green, blue, white);
          matrix_Plot(x + 1, y + 4, red, green, blue, white);
          matrix_Plot(x + 2, y + 4, red, green, blue, white);
          break;
        default:
          break;
      }
    }
  }
  else if (c == 176)                                            // "°"
  {
    matrix_Plot(x + 1, y    , red, green, blue, white);
    matrix_Plot(x    , y + 1, red, green, blue, white);
    matrix_Plot(x + 2, y + 1, red, green, blue, white);
    matrix_Plot(x + 1, y + 2, red, green, blue, white);
  }
  else                                                          // Missing Character
  {
    matrix_FillRect(x, y, x + 2, y + 4, red, green, blue, white);
  }
}

void matrix_DrawChar (byte c, int x, int y, byte red, byte green, byte blue, bool transparency)
{
  matrix_DrawChar(c, x, y, red, green, blue, 0, transparency);
}

void matrix_DrawString (String s, int x, int y, byte red, byte green, byte blue, byte white, bool transparency)
{
  int L = len(s);
  int p = x;
  for (int i = 1; i <= L; i++)
  {
    matrix_DrawChar(indasc(s, i), p, y, red, green, blue, white, transparency);
    p += 4;
  }
}

void matrix_DrawString (String s, int x, int y, byte red, byte green, byte blue, bool transparency)
{
  matrix_DrawString(s, x, y, red, green, blue, 0, transparency);
}

void matrix_Clear ()
{
  matrix_Pixel.clear();                                         // Pixel löschen
}

void matrix_Show ()
{
  delay(1);
  #if defined(ESP32)                                            // ESP 32
    portDISABLE_INTERRUPTS();                                   // Interrupts ausschalten
  #endif
  matrix_Pixel.show();                                          // Anzeige aktualisieren
  #if defined(ESP32)                                            // ESP 32
    portENABLE_INTERRUPTS();                                    // Interrupts einschalten
  #endif
}

void matrix_Init ()
{
  matrix_Pixel.begin();
  matrix_Pixel.setBrightness(255);                              // Helligkeit (0..255)
  matrix_Clear();                                               // Pixel löschen & Anzeige aktualisieren, RGBW-Modus
  matrix_Show();
  for (int i = 0; i < 32; i++)                                  // Modul-Einstellungen löschen
  {
    matrix_VirtualScreen[i] = 0;
    matrix_VirtualHeader[i] = "";
  }
  matrix_Color.FontRed        = 0;                              // Grün als Standard-Farbe auswählen
  matrix_Color.FontGreen      = 250;
  matrix_Color.FontBlue       = 0;
  matrix_Color.FontWhite      = 0;
  matrix_Color.SeparatorRed   = 250;                            // Violett als Trennzeichen-Farbe auswählen
  matrix_Color.SeparatorGreen = 0;
  matrix_Color.SeparatorBlue  = 250;
  matrix_Color.SeparatorWhite = 0;
  matrix_Color.Brightness     = 200;
  #if defined(ESP32)                                            // ESP32 - Matrix Taster an GPIO 23
    pinMode(23, INPUT);                                         // GPIO 23 auf Eingabe schalten (MOSI)
  #endif
}

void matrix_WlanAutoConnect (bool useSavedSettings)
{
  matrix_Clear();                                               // Pixel löschen
  matrix_DrawString("LOOK", 1,  1, matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
  matrix_DrawString("WLAN", 1,  9, matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
  matrix_Show();                                                // Anzeige aktualisieren
  IoT_WLANautoConnect(useSavedSettings);                        // Verbindung, ggf. gespeicherte, über WLAN herstellen
  matrix_Clear();                                               // Pixel löschen und Blau/Rot/Grün anzeigen
  matrix_Show();                                                // Anzeige aktualisieren
}

bool matrix_Keypress ()                                         // Eingebauten Taster abfragen
{
  #if defined(ESP32)                                            // ESP32
    return ((digitalRead(23) == LOW));                          // ESP32:   Taster an GPIO 23
  #else
    return ((digitalRead(12) == LOW));                          // ESP8266: Taster an GPIO 12
  #endif
}


// *** Matrix Module ***

void matrix_NextScreen ()
{
  matrix_StartIndex += 1;
  if (matrix_StartIndex > 15)
  {
    matrix_StartIndex = 0;
  }
  else
  {
    while ((matrix_VirtualScreen[matrix_StartIndex * 2] == 0) &&
           (matrix_VirtualScreen[matrix_StartIndex * 2 + 1] == 0) && (matrix_StartIndex <= 15))
    {
      matrix_StartIndex += 1;
    }
  }
  if (matrix_StartIndex > 15)
  {
    matrix_StartIndex = 0;
  }
}

void matrix_DisplayData (int value, String unit, int x, int y)
{
  matrix_DrawString(left(str(value) + unit, 4), x + 1, y + 1,
                    matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
  matrix_Pixel.show();                                          // Anzeige aktualisieren
}

void matrix_DisplayText (int x, int y, String header)
{
  matrix_DrawString(left(header, 4), x + 1, y + 1,
                    matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
  matrix_Pixel.show();                                          // Anzeige aktualisieren
}

void matrix_DisplayTime (int x, int y)
{
  int s = second();
  int h = hour();
  int m = minute();
  matrix_DrawString(strform(h, 48, 2, false), x + 1, y + 1,
                    matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
  matrix_DrawString(strform(m, 48, 2, false), x + 9, y + 1,
                    matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
  if (not(matrix_FlashingColon) || (s & 0x1 == 1))
  {
    matrix_Plot(8, y + 2, matrix_Color.SeparatorRed, matrix_Color.SeparatorGreen, matrix_Color.SeparatorBlue, matrix_Color.SeparatorWhite);
    matrix_Plot(8, y + 4, matrix_Color.SeparatorRed, matrix_Color.SeparatorGreen, matrix_Color.SeparatorBlue, matrix_Color.SeparatorWhite);
  }
  matrix_Pixel.show();                                          // Anzeige aktualisieren
}

void matrix_DisplaySecond (int x, int y)
{
  int s = second();
  matrix_DrawString(strform(s, 48, 2, false), x + 5, y + 1,
                    matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
  matrix_Pixel.show();                                          // Anzeige aktualisieren
}

void matrix_DisplayDate (int x, int y)
{
  int d = day();
  int m = month();
  matrix_DrawString(strform(d, 48, 2, false), x + 1, y + 1,
                    matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
  matrix_DrawString(strform(m, 48, 2, false), x + 9, y + 1,
                    matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
  matrix_Plot(x + 8, y + 5, matrix_Color.SeparatorRed, matrix_Color.SeparatorGreen, matrix_Color.SeparatorBlue, matrix_Color.SeparatorWhite);
  matrix_Pixel.show();                                          // Anzeige aktualisieren
}

void matrix_DisplayYear (int x, int y)
{
  int j = year();
  matrix_DrawString(strform(j, 48, 4, false), x + 1, y + 1,
                    matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
  matrix_Pixel.show();                                          // Anzeige aktualisieren
}

void matrix_DisplayTemperature (int x, int y)
{
  float t = DHT11_Temperature();                                // Temperatur auslesen (Celsius)
  IoT_Idle();
  if (not(isnan(t)))
  {
    matrix_DrawString(left(str(int(t + 0.5)) + chr(176) + "C", 4), x + 1, y + 1,
                      matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
    matrix_Pixel.show();                                        // Anzeige aktualisieren
  }
}

void matrix_DisplayHumidity (int x, int y)
{
  float h = DHT11_Humidity();                                   // Feuchtigkeit auslesen (Prozent)
  IoT_Idle();
  if (not(isnan(h)))
  {
    matrix_DrawString(str(int(h + 0.5)) + "%", x + 1, y + 1,
                      matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
    matrix_Pixel.show();                                        // Anzeige aktualisieren
  }
}

void matrix_DisplayBitCoin (int x, int y, String currency)
{
  String page = "http://blockchain.info/tobtc?currency=" + currency + "&value=1";
  String payload = IoT_GetContentHTTP(page, false);             // false = Keine Statusanzeige für HTTP-GET
  long v = 0;
  double r = realval(payload);
  if (r != 0.0)
  {
    v = long(1.0 / r);
  }
  matrix_DrawString(strform(v, 32, 1, false), x + 1, y + 1,
                    matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
  matrix_Pixel.show();                                          // Anzeige aktualisieren
}

void matrix_DrawCandle (int x, int y)
{
  matrix_FillRect(x + 1, y - 1, x + 3, y + 7,
                  matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite);
  matrix_FillRect(x + 2, y - 2, x + 2, y - 3, 0, 0, 0, 250);  // Weißer Docht
  if (second() % 2 == 0)
  {
    matrix_Plot(x + 2, y - 8   , 250, 250, 0, 0);               // Gelb
    matrix_Plot(x + 1, y - 7   , 250, 250, 0, 0);               // Gelb
    matrix_Plot(x + 2, y - 7   , 250, 250, 0, 0);               // Gelb
    matrix_Plot(x + 1, y - 6   , 250, 250, 0, 0);               // Gelb
    matrix_Plot(x + 1, y - 5   , 250, 250, 0, 0);               // Gelb
    matrix_Plot(x + 3, y - 7   , 250,   0, 0, 0);               // Rot
    matrix_Plot(x + 2, y - 6   , 250, 128, 0, 0);               // Orange
    matrix_Plot(x + 3, y - 6   , 250,   0, 0, 0);               // Rot
    matrix_Plot(x + 2, y - 5   , 250,   0, 0, 0);               // Rot
    matrix_Plot(x + 3, y - 5   , 250,   0, 0, 0);               // Rot
    matrix_Plot(x + 2, y - 4   , 250,   0, 0, 0);               // Rot
  }
  else
  {
    matrix_Plot(x + 2, y - 8   , 250,   0, 0, 0);               // Rot
    matrix_Plot(x + 1, y - 7   , 250,   0, 0, 0);               // Rot
    matrix_Plot(x + 2, y - 7   , 250,   0, 0, 0);               // Rot
    matrix_Plot(x + 1, y - 6   , 250,   0, 0, 0);               // Rot
    matrix_Plot(x + 1, y - 5   , 250,   0, 0, 0);               // Rot
    matrix_Plot(x + 3, y - 7   , 250, 255, 0, 0);               // Gelb
    matrix_Plot(x + 2, y - 6   , 250, 128, 0, 0);               // Orange
    matrix_Plot(x + 3, y - 6   , 250, 255, 0, 0);               // Gelb
    matrix_Plot(x + 2, y - 5   , 250, 255, 0, 0);               // Gelb
    matrix_Plot(x + 3, y - 5   , 250, 255, 0, 0);               // Gelb
    matrix_Plot(x + 2, y - 4   , 250, 255, 0, 0);               // Gelb
  }
}

void matrix_DisplayAdvent (int x, int y, String forceAdvent)
{
  String f = forceAdvent;
  byte m = month();
  byte d = day();
  matrix_FillRect(x, y - 8, x + 15, y + 7, 0, 0, 0, 0);
  if (((m == 12) && (d >= 3) && (d <= 9)) || (f == "1"))        // 1. Advent
  {
    matrix_DrawCandle(x + 6, y);
  }
  else if (((m == 12) && (d >= 10) && (d <= 16)) || (f == "2")) // 2. Advent
  {
    matrix_DrawCandle(x + 2, y);
    matrix_DrawCandle(x + 9, y);
  }
  else if (((m == 12) && (d >= 17) && (d <= 23)) || (f == "3")) // 3. Advent
  {
    matrix_DrawCandle(x + 1, y);
    matrix_DrawCandle(x + 6, y);
    matrix_DrawCandle(x + 11, y);
  }
  else if (((m == 12) && (d >= 24) && (d <= 24)) || (f == "4")) // 4. Advent
  {
    matrix_DrawCandle(x, y);
    matrix_DrawCandle(x + 4, y);
    matrix_DrawCandle(x + 8, y);
    matrix_DrawCandle(x + 12, y);
  }
  else
  {
    matrix_DrawString("KEIN", x + 1, y - 7,
                      matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
    matrix_DrawString("ADVT", x + 1, y + 1,
                      matrix_Color.FontRed, matrix_Color.FontGreen, matrix_Color.FontBlue, matrix_Color.FontWhite, true);
  }
  matrix_Pixel.show();                                          // Anzeige aktualisieren
}

void matrix_DisplayALL3500 (byte index, int x, int y)
{
  if ((index >= 0) && (index < all3500_SensorCount))            // Sensordaten vorhanden?
  {
    if (all3500_SensorChipID[index] == 0x85)                    // Gassensor
    {
      int value = 100 - int(all3500_SensorValue[index] + 0.5);  // Wert des Sensors in Toxiditär, umkehren in Qualität
      if (value <= 33)
      {
        matrix_FillRect(x, y, x + 15, y + 7, 250, 0, 0, 0);     // Roter Hintergrund
      }
      else if (value <= 66)
      {
        matrix_FillRect(x, y, x + 15, y + 7, 250, 125, 0, 0);   // Oranger Hintergrund
      }
      else
      {
        matrix_FillRect(x, y, x + 15, y + 7, 0, 125, 250, 0);   // Hellblauer Hintergrund
      }
      matrix_DisplayData (value, "%", x, y);
    }
    else
    {
      int id = all3500_SensorChipID[index];                     // Sensor-Typ (Chip-ID)
      String unit = "";
      int value = int(all3500_SensorValue[index] + 0.5);        // Wert des Sensors in % oder °C
      if ((id == 0x02) || (id == 0x03))                         // Temperatur-Sensoren
      {
        unit = chr(176) + "C";
      }
      else if (id == 0x20)                                      // Feuchtigkeits-Sensoren
      {
        unit = "%";
      }
      else                                                      // Bei unbekanntem Typ hinterlegte Einheit verwenden
      {
        unit = left(all3500_SensorUnit[index], 2);
      }
      matrix_FillRect(x, y, x + 15, y + 7, 0, 0, 0, 0);
      matrix_DisplayData (value, unit, x, y);
    }
  }
}

void matrix_DisplayModule (byte module, int x, int y, String header)
{
  matrix_FillRect(x, y, x + 15, y + 7, 0, 0, 0, 0);
  switch (module)
  {
    case 1:                                                     // Header anzeigen
      matrix_DisplayText(x, y, header);
      break;
    case 2:                                                     // Uhrzeit hh:mm
      matrix_DisplayTime(x, y);
      break;
    case 3:                                                     // Uhrzeit ss
      matrix_DisplaySecond(x, y);
      break;
    case 4:                                                     // Datum dd.mm
      matrix_DisplayDate(x, y);
      break;
    case 5:                                                     // Datum jjjj
      matrix_DisplayYear(x, y);
      break;
    case 6:                                                     // Temperatur
      matrix_DisplayTemperature(x, y);
     break;
    case 7:                                                     // Luftfeuchtigkeit
      matrix_DisplayHumidity(x, y);
      break;
    case 8:                                                     // BitCoin Kurs
      matrix_DisplayBitCoin(x, y, header);
      break;
    case 9:                                                     // Adventskerzen
      matrix_DisplayAdvent(x, y, header);
      break;
    default:
      if (module >= 16)
      {
        matrix_DisplayALL3500(module - 16, x, y);
      }
      break;
  }
}

bool matrix_DisplayMatrix ()
{
  byte index   = matrix_StartIndex * 2;
  byte m_oben  = matrix_VirtualScreen[index];
  byte m_unten = matrix_VirtualScreen[index + 1];
  if ((m_oben == 0) && (m_unten == 0))                          // Virtuelle Matrix ist leer
  {
    return (false);
  }
  else
  {
    matrix_DisplayModule(m_oben,  0, 0, matrix_VirtualHeader[index]);
    matrix_DisplayModule(m_unten, 0, 8, matrix_VirtualHeader[index + 1]);
    return (true);
  }
}


// *** Matrix Parameter-RAM ***

bool matrix_ParameterValidate ()
{
  if ((matrix_ParameterData.parameterID == crc64("ALmatrix", 0)) &&
      (matrix_ParameterData.parameterHash == xorshift128plus("ALmatrix", 0)))
  {
    return (true);                                                          // Parameter sind gültig
  }
  else
  {
    return (false);                                                         // Ungültige Parameter
  }
}

bool matrix_ParameterApply ()
{
  if (matrix_ParameterValidate())
  {
    if (matrix_ParameterData.staticIP == 1)
    {
      WiFi.localIP()[0] = matrix_ParameterData.IP_address0;                 // 4 Bytes IP-Adresse
      WiFi.localIP()[1] = matrix_ParameterData.IP_address1;
      WiFi.localIP()[2] = matrix_ParameterData.IP_address2;
      WiFi.localIP()[3] = matrix_ParameterData.IP_address3;
      WiFi.gatewayIP()[0] = matrix_ParameterData.IP_gateway0;               // 4 Bytes Gateway
      WiFi.gatewayIP()[1] = matrix_ParameterData.IP_gateway1;
      WiFi.gatewayIP()[2] = matrix_ParameterData.IP_gateway2;
      WiFi.gatewayIP()[3] = matrix_ParameterData.IP_gateway3;
      WiFi.subnetMask()[0] = matrix_ParameterData.IP_subnet0;               // 4 Bytes Subnetzmaske
      WiFi.subnetMask()[1] = matrix_ParameterData.IP_subnet1;
      WiFi.subnetMask()[2] = matrix_ParameterData.IP_subnet2;
      WiFi.subnetMask()[3] = matrix_ParameterData.IP_subnet3;
      IPAddress ip1 = IPAddress(matrix_ParameterData.IP_address0,
                                matrix_ParameterData.IP_address1,
                                matrix_ParameterData.IP_address2,
                                matrix_ParameterData.IP_address3);
      IPAddress ip2 = IPAddress(matrix_ParameterData.IP_gateway0,
                                matrix_ParameterData.IP_gateway1,
                                matrix_ParameterData.IP_gateway2,
                                matrix_ParameterData.IP_gateway3);
      IPAddress ip3 = IPAddress(matrix_ParameterData.IP_subnet0,
                                matrix_ParameterData.IP_subnet1,
                                matrix_ParameterData.IP_subnet2,
                                matrix_ParameterData.IP_subnet3);
      WiFi.config(ip1, ip2, ip3);                                           // Parameter: IP, Gateway, Subnet, DNS
    }
    matrix_StaticIP = matrix_ParameterData.staticIP;                        // 0 = Dynamische IP-Adresse, 1 = Statische IP-Adresse
    return (true);                                                          // Parameter-Block ist gültig und wurde verwendet
  }
  else
  {
    return (false);                                                         // Parameter-Block ist ungültig und wurde nicht verwendet
  }
}

void matrix_ParameterCreate ()
{
//   32 Bytes Parameter-Header:
  matrix_ParameterData.parameterID      = crc64("ALmatrix", 0);             // Kennung des Parameter-Blocks "ALmatrix" im Klartext
  matrix_ParameterData.parameterHash    = xorshift128plus("ALmatrix", 0);   // Kennung des Parameter-Blocks "ALmatrix" als verschlüsselter Hash
    
//   Start des Parameter-Blocks:
  matrix_ParameterData.IP_address0      = WiFi.localIP()[0];                // 4 Bytes IP-Adresse
  matrix_ParameterData.IP_address1      = WiFi.localIP()[1];
  matrix_ParameterData.IP_address2      = WiFi.localIP()[2];
  matrix_ParameterData.IP_address3      = WiFi.localIP()[3];
  matrix_ParameterData.IP_gateway0      = WiFi.gatewayIP()[0];              // 4 Bytes Gateway
  matrix_ParameterData.IP_gateway1      = WiFi.gatewayIP()[1];
  matrix_ParameterData.IP_gateway2      = WiFi.gatewayIP()[2];
  matrix_ParameterData.IP_gateway3      = WiFi.gatewayIP()[3];
  matrix_ParameterData.IP_subnet0       = WiFi.subnetMask()[0];             // 4 Bytes Subnetzmaske
  matrix_ParameterData.IP_subnet1       = WiFi.subnetMask()[1];
  matrix_ParameterData.IP_subnet2       = WiFi.subnetMask()[2];
  matrix_ParameterData.IP_subnet3       = WiFi.subnetMask()[3];
  matrix_ParameterData.staticIP         = matrix_StaticIP;                  // 0 = Dynamische IP-Adresse, 1 = Statische IP-Adresse
}

void matrix_ParameterNew ()
{
//   32 Bytes Parameter-Header:
  matrix_ParameterData.parameterID      = crc64("ALmatrix", 0);             // Kennung des Parameter-Blocks "ALmatrix" im Klartext
  matrix_ParameterData.parameterHash    = xorshift128plus("ALmatrix", 0);   // Kennung des Parameter-Blocks "ALmatrix" als verschlüsselter Hash
    
//   Start des Parameter-Blocks:
  matrix_ParameterData.IP_address0      = 0;                                // 4 Bytes IP-Adresse
  matrix_ParameterData.IP_address1      = 0;
  matrix_ParameterData.IP_address2      = 0;
  matrix_ParameterData.IP_address3      = 0;
  matrix_ParameterData.IP_gateway0      = 0;                                // 4 Bytes Gateway
  matrix_ParameterData.IP_gateway1      = 0;
  matrix_ParameterData.IP_gateway2      = 0;
  matrix_ParameterData.IP_gateway3      = 0;
  matrix_ParameterData.IP_subnet0       = 0;                                // 4 Bytes Subnetzmaske
  matrix_ParameterData.IP_subnet1       = 0;
  matrix_ParameterData.IP_subnet2       = 0;
  matrix_ParameterData.IP_subnet3       = 0;
  matrix_ParameterData.staticIP         = 0;                                // 0 = Dynamische IP-Adresse, 1 = Statische IP-Adresse

  for (int i = 0; i < 32; i++)                                              // Modul-Einstellungen löschen
  {
    matrix_VirtualScreen[i] = 0;
    matrix_VirtualHeader[i] = "";
  }

  matrix_VirtualScreen[0] = 1;
  matrix_VirtualHeader[0] = "ZEIT";
  matrix_VirtualScreen[1] = 2;
  matrix_VirtualHeader[1] = "";
  
  matrix_VirtualScreen[2] = 1;
  matrix_VirtualHeader[2] = "DAT.";
  matrix_VirtualScreen[3] = 4;
  matrix_VirtualHeader[3] = "";
  
  matrix_VirtualScreen[4] = 1;
  matrix_VirtualHeader[4] = "SEK.";
  matrix_VirtualScreen[5] = 3;
  matrix_VirtualHeader[5] = "";
 
  matrix_VirtualScreen[6] = 1;
  matrix_VirtualHeader[6] = "JAHR";
  matrix_VirtualScreen[7] = 5;
  matrix_VirtualHeader[7] = "";
 
  matrix_VirtualScreen[8] = 1;
  matrix_VirtualHeader[8] = "TEMP";
  matrix_VirtualScreen[9] = 6;
  matrix_VirtualHeader[9] = "";

  matrix_VirtualScreen[10] = 1;
  matrix_VirtualHeader[10] = "FCHT";
  matrix_VirtualScreen[11] = 7;
  matrix_VirtualHeader[11] = "";
}

bool matrix_ParameterLoad ()
{
  EEPROM.get(0, matrix_ParameterData);                                      // Parameter aus dem EEPROM laden
  if (matrix_ParameterValidate())                                           // Die geladenen Daten validieren
  {
    EEPROM.get(0x200, matrix_Color);                                        // Farbeinstellungen aus dem EEPROM laden
    matrix_ID = IoT_EEPROMgetString(0xC00);                                 // Geräte-Namen aus dem EEPROM laden
    matrix_ALL3500URL = IoT_EEPROMgetString(0xD00);                         // ALL-3500-URL aus dem EEPROM laden
    if (matrix_ID == "")
    {
      matrix_ID = "ESP_Matrix " + left(newuniqueid(), 12);
    }
    EEPROM.get(1024, matrix_VirtualScreen);                                 // Parameter aus dem EEPROM laden
    for (int i = 0; i < 32; i++)                                            // Strings sequentiell sichern
    {
      matrix_VirtualHeader[i] = IoT_EEPROMgetString(0x800 + i * 16);
    }
    return (true);                                                          // Parameter sind gültig und wurden erfolgreich geladen
  }
  else
  {
    return (false);                                                         // Ungültige Parameter
  }
}

void matrix_ParameterSave ()
{
  if (matrix_ParameterValidate())
  {
    EEPROM.put(0, matrix_ParameterData);                                    // Parameter in das EEPROM sichern
    EEPROM.put(0x200, matrix_Color);                                        // Farbeinstellungen in das EEPROM sichern
    matrix_ID = left(matrix_ID, 63);                                        // maximale Länge = 63 Zeichen
    if (matrix_ID == "")
    {
      matrix_ID = "ESP_Matrix " + left(newuniqueid(), 12);
    }
    EEPROM.put(1024, matrix_VirtualScreen);                                 // Parameter aus dem EEPROM laden
    for (int i = 0; i < 32; i++)                                            // Strings sequentiell laden
    {
      IoT_EEPROMputString(0x800 + i * 16, matrix_VirtualHeader[i]);
    }
    IoT_EEPROMputString(0xC00, matrix_ID);                                  // Geräte-Namen in das EEPROM speichern
    IoT_EEPROMputString(0xD00, matrix_ALL3500URL);                          // ALL-3500-URL in das EEPROM speichern
  }
}

