// PRAM Library
// 1.00 - 2017-12-12
// 1.01 - 2018-02-27
// (c) Copyright 2017
//
// Zur Verwendung mit allen Allnet Libraries


// *** PRAM Datentypen ***

struct pram_Type
{
//    8 Bytes Parameter-Header:
    int64 parameterID       = 0;                                      // Kennung des Parameter-Blocks "ALL-IoT" im Klartext
    int64 parameterHash     = 0;                                      // Kennung des Parameter-Blocks "ALL-IoT" als verschlüsselter Hash
    
//    Start des Parameter-Blocks:
    byte   IP_address0      = 0;                                      // 4 Bytes IP-Adresse
    byte   IP_address1      = 0;
    byte   IP_address2      = 0;
    byte   IP_address3      = 0;
    byte   IP_gateway0      = 0;                                      // 4 Bytes Gateway
    byte   IP_gateway1      = 0;
    byte   IP_gateway2      = 0;
    byte   IP_gateway3      = 0;
    byte   IP_subnet0       = 0;                                      // 4 Bytes Subnetzmaske
    byte   IP_subnet1       = 0;
    byte   IP_subnet2       = 0;
    byte   IP_subnet3       = 0;
    byte   staticIP         = 0;                                      // 0 = Dynamische IP-Adresse, 1 = Statische IP-Adresse
};


// *** PRAM Globale Variablen ***

pram_Type          pram_Data;                                         // Parameter Block mit allen Konfigurations-Daten
String             pram_CreatorID          = "default";               // ID des Programms für diese Daten (maximal 8 Zeichen für Klartext-ID)

// *** PRAM Funktionen ***

void pram_Init (String CreatorID)
{
  pram_CreatorID = CreatorID;
}

bool pram_Validate ()
{
  if ((pram_Data.parameterID == crc64(pram_CreatorID, 0)) &&
      (pram_Data.parameterHash == xorshift128plus(pram_CreatorID, 0)))
  {
    return (true);                                                    // Parameter sind gültig
  }
  else
  {
    return (false);                                                   // Ungültige Parameter
  }
}

bool pram_Apply ()
{
  if (pram_Validate())
  {
    if (pram_Data.staticIP == 1)
    {
      WiFi.localIP()[0] = pram_Data.IP_address0;                      // 4 Bytes IP-Adresse
      WiFi.localIP()[1] = pram_Data.IP_address1;
      WiFi.localIP()[2] = pram_Data.IP_address2;
      WiFi.localIP()[3] = pram_Data.IP_address3;
      WiFi.gatewayIP()[0] = pram_Data.IP_gateway0;                    // 4 Bytes Gateway
      WiFi.gatewayIP()[1] = pram_Data.IP_gateway1;
      WiFi.gatewayIP()[2] = pram_Data.IP_gateway2;
      WiFi.gatewayIP()[3] = pram_Data.IP_gateway3;
      WiFi.subnetMask()[0] = pram_Data.IP_subnet0;                    // 4 Bytes Subnetzmaske
      WiFi.subnetMask()[1] = pram_Data.IP_subnet1;
      WiFi.subnetMask()[2] = pram_Data.IP_subnet2;
      WiFi.subnetMask()[3] = pram_Data.IP_subnet3;
      IPAddress ip1 = IPAddress(pram_Data.IP_address0,
                                pram_Data.IP_address1,
                                pram_Data.IP_address2,
                                pram_Data.IP_address3);
      IPAddress ip2 = IPAddress(pram_Data.IP_gateway0,
                                pram_Data.IP_gateway1,
                                pram_Data.IP_gateway2,
                                pram_Data.IP_gateway3);
      IPAddress ip3 = IPAddress(pram_Data.IP_subnet0,
                                pram_Data.IP_subnet1,
                                pram_Data.IP_subnet2,
                                pram_Data.IP_subnet3);
      WiFi.config(ip1, ip2, ip3);                                     // Parameter: IP, Gateway, Subnet, DNS
    }
    IoT_StaticIP = pram_Data.staticIP;                                // 0 = Dynamische IP-Adresse, 1 = Statische IP-Adresse
    return (true);                                                    // Parameter-Block ist gültig und wurde verwendet
  }
  else
  {
    return (false);                                                   // Parameter-Block ist ungültig und wurde nicht verwendet
  }
}

void pram_Create ()
{
  String ID = pram_CreatorID;
//   32 Bytes Parameter-Header:
  pram_Data.parameterID      = crc64(ID, 0);                          // Kennung des Parameter-Blocks im Klartext
  pram_Data.parameterHash    = xorshift128plus(ID, 0);                // Kennung des Parameter-Blocks als verschlüsselter Hash
    
//   Start des Parameter-Blocks:
  pram_Data.IP_address0      = WiFi.localIP()[0];                     // 4 Bytes IP-Adresse
  pram_Data.IP_address1      = WiFi.localIP()[1];
  pram_Data.IP_address2      = WiFi.localIP()[2];
  pram_Data.IP_address3      = WiFi.localIP()[3];
  pram_Data.IP_gateway0      = WiFi.gatewayIP()[0];                   // 4 Bytes Gateway
  pram_Data.IP_gateway1      = WiFi.gatewayIP()[1];
  pram_Data.IP_gateway2      = WiFi.gatewayIP()[2];
  pram_Data.IP_gateway3      = WiFi.gatewayIP()[3];
  pram_Data.IP_subnet0       = WiFi.subnetMask()[0];                  // 4 Bytes Subnetzmaske
  pram_Data.IP_subnet1       = WiFi.subnetMask()[1];
  pram_Data.IP_subnet2       = WiFi.subnetMask()[2];
  pram_Data.IP_subnet3       = WiFi.subnetMask()[3];
  pram_Data.staticIP         = IoT_StaticIP;                          // 0 = Dynamische IP-Adresse, 1 = Statische IP-Adresse
}

void pram_New ()
{
  String ID = pram_CreatorID;
//   32 Bytes Parameter-Header:
  pram_Data.parameterID      = crc64(ID, 0);                          // Kennung des Parameter-Blocks im Klartext
  pram_Data.parameterHash    = xorshift128plus(ID, 0);                // Kennung des Parameter-Blocks "als verschlüsselter Hash
    
//   Start des Parameter-Blocks:
  pram_Data.IP_address0      = 0;                                     // 4 Bytes IP-Adresse
  pram_Data.IP_address1      = 0;
  pram_Data.IP_address2      = 0;
  pram_Data.IP_address3      = 0;
  pram_Data.IP_gateway0      = 0;                                     // 4 Bytes Gateway
  pram_Data.IP_gateway1      = 0;
  pram_Data.IP_gateway2      = 0;
  pram_Data.IP_gateway3      = 0;
  pram_Data.IP_subnet0       = 0;                                     // 4 Bytes Subnetzmaske
  pram_Data.IP_subnet1       = 0;
  pram_Data.IP_subnet2       = 0;
  pram_Data.IP_subnet3       = 0;
  pram_Data.staticIP         = 0;                                     // 0 = Dynamische IP-Adresse, 1 = Statische IP-Adresse
}

bool pram_Load ()
{
  EEPROM.get(0, pram_Data);                                           // Parameter aus dem EEPROM laden (1023 = aktueller Status)
  if (pram_Validate())                                                // Die geladenen Daten validieren
  {
    String DeviceID = IoT_EEPROMgetString(3072);                      // $C00 = Geräte-Namen aus dem EEPROM laden
    int L = len(DeviceID);
    if ((L > 0) || (L <= 63))                                         // Länge muss zwischen 1 und 63 Zeichen sein
    {
      IoT_DeviceID = DeviceID;
    }
    if (IoT_DeviceID == "")
    {
      IoT_DeviceID = pram_CreatorID + "-" + left(newuniqueid(), 8);
    }
    return (true);                                                    // Parameter sind gültig und wurden erfolgreich geladen
  }
  else
  {
    return (false);                                                   // Ungültige Parameter
  }
}

void pram_Save ()
{
  if (pram_Validate())
  {
    EEPROM.put(0, pram_Data);                                         // Parameter in das EEPROM sichern
    IoT_DeviceID = left(IoT_DeviceID, 63);                            // maximale Länge = 63 Zeichen
    if (IoT_DeviceID == "")
    {
      IoT_DeviceID = pram_CreatorID + "-" + left(newuniqueid(), 8);
    }
    IoT_EEPROMputString(3072, IoT_DeviceID);                          // $C00 = Geräte-Namen in das EEPROM schreiben
  }
}

void pram_TestResetKonfig ()
{
  IoT_DisplayClear(24);                                               // OLED Display löschen (24 Punkt Schrift)
  IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                            // Zentrierte Darstellung des Textes
  IoT_DisplayDrawText(64, 5, "Reset");
  IoT_DisplayDrawText(64, 34, "Konfig.?");
  IoT_DisplayUpdate();                                                // OLED-Anzeige aktualisieren
  IoT_WaitKeypress(2000, false);                                      // 2 s warten, in der Zeit kann mit Taster die Konfig zurückgesetzt werden
  if (IoT_Keypress())                                                 // Wenn Taster gedrückt wird, Konfiguration im EEPROM löschen
  {
    IoT_EEPROMclear();                                                // Alle Parameter aus dem EEPROM löschen
    IoT_EEPROMupdate();                                               // EEPROM aktualisieren
    Serial.print("Netzwerk Parameter aus EEPROM geloescht. ");
    Serial.println("Geraet wird neu gestartet...");
    delay(100);                                                       // 100 ms. warten
    IoT_DisplayClear(24);                                             // OLED Display löschen (24 Punkt Schrift)
    IoT_DisplayUpdate();                                              // OLED-Anzeige aktualisieren
    IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
    t_Restart();                                                      // Neustart durchführen
  }
  IoT_DisplayClear(12);                                               // OLED Display löschen (12 Punkt Schrift)
  IoT_DisplayUpdate();                                                // OLED-Anzeige aktualisieren
}

void pram_LoadAndApply ()
{
  if (pram_Load())                                                    // Parameter-Block aus EEPROM laden incl. IoT_DeviceID
  {
    Serial.println("Netzwerk Parameter aus EEPROM geladen.");
    pram_Apply();                                                     // Parameter-Block in die aktuelle Konfiguration übernehmen
  }
  else
  {
    Serial.println("Netzwerk Parameter im EEPROM nicht vorhanden.");
  }
  if (IoT_DeviceID == "")
  {
    IoT_DeviceID = pram_CreatorID + "-" + left(newuniqueid(), 8);
  }
}
