// SE95 Library (Temperatur-Sensor)
// 1.00 - 2018-01-15
// (c) Copyright 2018
//
// Zur Verwendung mit dem Allnet IoT-Brick


// *** Globale Variablen ***

bool                 SE95_Inited                       = true;     // Variable für die Initialisierung der SE95 Benutzung
int                  SE95_Address                      = 0x48;     // i2c-Adresse des SE95-Sensors


// *** Temperatur-Sensor Funktionen ***

double SE95_Temperature (bool messages)
{
  if (SE95_Inited)
  {
    #if defined(ESP8266)
      ESP.wdtFeed();                                // Watchdog Timer zurücksetzen
    #endif
    Wire.beginTransmission(SE95_Address);
    Wire.write(0);
    Wire.endTransmission(); 
    #if defined(ESP8266)
      ESP.wdtFeed();                                // Watchdog Timer zurücksetzen
    #endif
    Wire.requestFrom(SE95_Address, 2);
    int high = Wire.read();
    int low  = Wire.read();
    #if defined(ESP8266)
      ESP.wdtFeed();                                // Watchdog Timer zurücksetzen
    #endif
    if ((low == -1) && (high == -1)) // i2c-Bus nicht initialisiert oder Sensor nicht gefunden
    {
      if (messages)
      {
        Serial.println("SE95_Temperature: Sensor not found or i2c-Bus not initialized");
      }
      return (-128.0); // Sensor nicht vorhanden oder i2c-Bus nicht initialisiert
    }
    else if ((low == 0) && (high == 0)) // Fehlerhafte Messung
    {
      if (messages)
      {
        Serial.println("SE95_Temperature: Invalid temperature value");
      }
      return (-129.0); // Sensor-Daten konnten nicht korrekt gelesen werden oder Sensor nicht vorhanden
    }
    else
    {
      int temp = ((high & 0x7F) << 5) + (low >> 3);
      if (high > 0x7F)
      {
        temp -= 0x1000; // Negative Temperatur
      }
      temp = (temp * 3) + (temp / 8);  // * 3.125
      return ((double)temp / 100.0);
    }
  }
  else
  {
    if (messages)
    {
      Serial.println("SE95_Temperature: Library not initialized");
    }
    return (-127.0); // Library not initialized
  }
}
