// Sensor Library (für ALLNET Sensoren mit Transistor-gepuffertem i2c-Bus)
// 1.00 - 2018-02-10
// 1.01 - 2018-03-01
// (c) Copyright 2018
//
// Zur Verwendung mit dem Allnet IoT-Brick

#include <all_i2c.h>


// *** Globale Konstanten ***

byte                 sensor_DigitalButton   = 0;                  // Sensortyp: Digitaler Knopf   (0/1)
byte                 sensor_Temperature     = 1;                  // Sensortyp: Temperatur        (-100 bis 100 °C)
byte                 sensor_Humidity        = 2;                  // Sensortyp: Luftfeuchtigkeit  (0 bis 100 %)
byte                 sensor_Toxicity        = 3;                  // Sensortyp: Luftverschmutzung (0 bis 1000 ‰)
byte                 sensor_Pressure        = 4;                  // Sensortyp: Luftdruck         (300 bis 1100 mBar)
byte                 sensor_Volt12          = 5;                  // Sensortyp: Spannung          (0 bis 12 V)
byte                 sensor_Amp1            = 6;                  // Sensortyp: Stromstärke       (0 bis 1000 mA)
byte                 sensor_Volt80          = 7;                  // Sensortyp: Spannung          (0 bis 80 V)
byte                 sensor_Amp5            = 8;                  // Sensortyp: Stromstärke       (0 bis 5 A)
byte                 sensor_Watt400         = 9;                  // Sensortyp: Leistung          (0 bis 400 W)
byte                 sensor_Watt1           = 10;                 // Sensortyp: Leistung          (0 bis 1000 mW)


// *** Globale Variablen ***

String               sensor_ConnectedType              = "";      // Name des angeschlossenen Sensors
String               sensor_LastValue                  = "";      // Letzter Wert des angeschlossenen Sensors

double               ALL4529_LastTemperature           = -129.0;  // Die zuletzt gemessene Temperatur
double               ALL4529_LastPressure              = -129.0;  // Der zuletzt gemessene Luftdruck
uint32_t             ALL4529_LastMeasureTime           = -1000;   // Der millis()-Zeitpunkt der letzten Messung


// *** ALLNET ALL3500 Kommunikation ***

String sensor_GetInitSequence (byte SensorID, String SensorName, byte ActuatorSensorType)
{
  String digitalButton      = "0";
  String display_min        = "0";                                // Kleinster anzeigbarer Wert
  String display_max        = "100";                              // Größter anzeigbarer Wert
  String limit_min          = "10";                               // Grenze für unteren Bereich (blauer Bereich)
  String limit_max          = "90";                               // Grenze für oberen Bereich (roter Bereich)
  String today_min          = "1000";                             // Wert muss so groß sein, dass er sofort überschrieben wird
  String today_max          = "-1000";                            // Wert muss so klein sein, dass er sofort überschrieben wird
  String view               = "2";                                // Anzeige 0 = Boolscher Wert, 2 = Analoger Wert (Graph/Instrument)
  String fe_view            = "2";
  String unit               = "";                                 // Einheit (z.B. "°C" oder "%")
  String deviceType         = "4";                                // 4 = Sensor, 10 = Actuator
  String type               = "1";                                // 1 = Sensor, 2 = Actuator, 3 = Reserved, 4 = Analog Actuator
  String colors             = "1E7EAC:990000:990000";             // Kachelfarbe bei Darstellung im ALL 3500

  if (ActuatorSensorType == sensor_DigitalButton)
  {
    digitalButton = "1";
    fe_view       = "0";
    view          = "0";
    display_min   = "0";
    display_max   = "1";
    limit_min     = "0";
    limit_max     = "1";
  }
  else if (ActuatorSensorType == sensor_Temperature)
  {
    display_min   = "-20";
    display_max   = "140";
    limit_min     = "0";
    limit_max     = "100";
    unit          = "°C";
  }
  else if (ActuatorSensorType == sensor_Humidity)
  {
    display_min   = "0";
    display_max   = "100";
    limit_min     = "10";
    limit_max     = "90";
    unit          = "%";
  }
  else if (ActuatorSensorType == sensor_Toxicity)
  {
    display_min   = "0";
    display_max   = "1000";
    limit_min     = "250";
    limit_max     = "500";
    unit          = "‰";
    colors        = "009000:1E7EAC:900000";
  }
  else if (ActuatorSensorType == sensor_Pressure)
  {
    display_min   = "600";
    display_max   = "1100";
    limit_min     = "700";
    limit_max     = "1015";
    unit          = " mBar";
  }
  else if (ActuatorSensorType == sensor_Volt80)
  {
    display_min   = "0";
    display_max   = "80";
    limit_min     = "0";
    limit_max     = "80";
    unit          = "V";
  }
  else if (ActuatorSensorType == sensor_Volt12)
  {
    display_min   = "0";
    display_max   = "12";
    limit_min     = "0";
    limit_max     = "12";
    unit          = "V";
  }
  else if (ActuatorSensorType == sensor_Amp5)
  {
    display_min   = "0";
    display_max   = "6";
    limit_min     = "0";
    limit_max     = "5";
    unit          = "A";
  }
  else if (ActuatorSensorType == sensor_Amp1)
  {
    display_min   = "0";
    display_max   = "1000";
    limit_min     = "0";
    limit_max     = "1000";
    unit          = "mA";
  }
  else if (ActuatorSensorType == sensor_Watt400)
  {
    display_min   = "0";
    display_max   = "400";
    limit_min     = "0";
    limit_max     = "400";
    unit          = "W";
  }
  else if (ActuatorSensorType == sensor_Watt1)
  {
    display_min   = "0";
    display_max   = "1000";
    limit_min     = "0";
    limit_max     = "1000";
    unit          = "mW";
  }

  String s = "\
  {\"id\":\"" + str(SensorID) + "\",\"name\":\"" + SensorName + "\",\
   \"description\":\"\",\"fe_view\": \"" + fe_view + "\",\"sort\":\"2:1\",\
   \"fe_cvs_show_typ\":\"1\",\"Actuator_analogValue\":null,\"digitalToText\":\"0::\",\
   \"tileColors\":\"" + colors + "\",\"tileFormats\":\"55:\",\
   \"lang_port_identifier\":\"_999074_\",\
   \"device_type\":\"" + deviceType + "\",\"value\":\"1\",\"error\":0,\"config\":{\"icon\":null,\
   \"display\":\{\"min\":\"" + display_min + "\",\"max\":\"" + display_max + "\",\"minmax\":\"0\",\
   \"digital\":\"" + digitalButton + "\"},\
   \"limit\":{\"min\":\"" + limit_min + "\",\"max\":\"" + limit_max + "\"}},\
   \"info\":{\"activ\":\"1\",\"enabled\":\"1\",\"unit\":\"" + unit + "\",\
   \"type\":\"" + type + "\",\"view\":\"" + view + "\",\
   \"chipid\":\"2\",\"chipnumber\":\"1\",\"chipaddress\":\"0\",\
   \"helperchipnumber\":\"0\",\"helperchipaddress\":\"0\",\"bitaddress\":\"0\"},\
   \"minmax\":{\"today\":{\"min\":\"" + today_min + "\",\"max\":\"" + today_max + "\"},\
   \"absolute\":{\"min\":\"" + today_min + "\",\"max\":\"-" + today_max + "\"}},\
   \"connection\":{\"port\":\"0\",\"bus\":\"0\",\"group\":\"0\"}}";
   
  return (removespaces(s));
}

String sensor_GetSensorSequence (byte SensorID, String SensorName, String value)
{
  String s = "\
  {\"id\":\"" + str(SensorID) + "\",\"name\":\"" + SensorName + "\",\
   \"description\": \"\",\"fe_view\": \"\",\"sort\": \"\",\
   \"fe_cvs_show_typ\": \"\",\"Actuator_analogValue\": \"\",\
   \"digitalToText\": \"\",\"tileColors\": \"\",\"tileFormats\": \"\",\
   \"lang_port_identifier\": \"\",\
   \"value\": \"" + value + "\",\"config\": {\"icon\": \"\",\
   \"display\":{\"min\": \"0\",\"max\": \"1\"},\
   \"limit\":{\"min\": \"0\",\"max\": \"1\"}},\
   \"info\":{\"unit\": \"\",\"type\": \"2\"}}";
  return (removespaces(s));
}


// *** ALLNET Sensor Funktionen ***

String sensor_Identify (bool messages)
{
  byte result = 0;
  sensor_ConnectedType = "";

  if (sensor_ConnectedType == "")
  {
    i2c_SendStart();
    result = i2c_SendByte(145 + 2 * 3, 2);                        // Adresse senden für ALL4454 und ACK erwarten
    if (result == 0)
    {
      byte tmp1 = i2c_ReceiveByte(1);                             // Byte lesen und ACK senden
      byte tmp2 = i2c_ReceiveByte(1);                             // Byte lesen und ACK senden
      byte stat = i2c_ReceiveByte(0);                             // Byte lesen und NAK senden
      if (stat != 0xFF)
      {
        if (messages)
        {
          Serial.println("Sensor found: ALL 4454");
        }
        sensor_ConnectedType = "ALL 4454";
      }
    }
    i2c_SendStop();
  }

  if (sensor_ConnectedType == "")
  {
    i2c_SendStart();
    result = i2c_SendByte(238, 2);                                // Adresse senden für ALL4529 und ACK erwarten
    if (result == 0)                                              // 3=1110 1110 = Write to BMP085 ($EE)
    {
      if (messages)
      {
        Serial.println("Sensor found: ALL 4529");
      }
      sensor_ConnectedType = "ALL 4529";
    }
    i2c_SendStop();
  }

  if (sensor_ConnectedType == "")
  {
    i2c_SendStart();
    result = i2c_SendByte(144 + 2 * 3, 2);                        // Adresse senden für ALL3006 oder ALL4454 und ACK erwarten
    if (result == 0)
    {
      if (messages)
      {
        Serial.println("Sensor found: ALL 3006");
      }
      sensor_ConnectedType = "ALL 3006";
    }
  i2c_SendStop();
  }
  
  if (sensor_ConnectedType == "")
  {
    i2c_SendStart();
    result = i2c_SendByte(144 + 2 * 7, 2);                        // Adresse senden für ALL3008 und ACK erwarten
    if (result == 0)
    {
      if (messages)
      {
        Serial.println("Sensor found: ALL 3008");
      }
      sensor_ConnectedType = "ALL 3008";
    }
    i2c_SendStop();
  }
  
  if (sensor_ConnectedType == "")
  {
    i2c_SendStartSHT();                                           // Sonderfall für den SHT71 Chip
    result = i2c_SendByte(3, 2);                                  // Adresse senden für ALL3018 und ACK erwarten
    if (result == 0)
    {
      if (messages)
      {
        Serial.println("Sensor found: ALL 3018");
      }
      sensor_ConnectedType = "ALL 3018";                          // Daten für diesen Sensor müssen ausgelesen werden!
      i2c_Delay(285);                                             // 285 ms. warten, bevor Werte abgerufen werden
      byte high = i2c_ReceiveByte(1);                             // Byte lesen und ACK senden
      byte low  = i2c_ReceiveByte(1);                             // Byte lesen und ACK senden
      byte crc  = i2c_ReceiveByte(0);                             // Byte lesen und NAK senden
     }
    i2c_SendStop();
  }

  return (sensor_ConnectedType);
}

double ALL3006_Temperature (bool messages)                        // Temperatur in °C
{
  long temperature = 0;
  if (i2c_Inited)
  {
    if (sensor_ConnectedType == "ALL 3006")
    {
      i2c_SendStart();
      byte result = i2c_SendByte(145 + 2 * 3, 2);                 // Adresse senden für ALL3006 und ACK erwarten
      if (result == 0)
      {
        byte high = i2c_ReceiveByte(1);                           // Byte lesen und ACK senden
        byte low  = i2c_ReceiveByte(0);                           // Byte lesen und NAK senden
        if ((high == 0xFF) && (low == 0xFF))
        {
          if (messages)
          {
            Serial.println("ALL3006_Temperature: Invalid temperature value");
          }
          temperature = -129;                                     // Sensor-Daten konnten nicht korrekt gelesen werden oder Sensor nicht vorhanden
        }
        else
        {
          temperature = ((high & 0x7F) << 5) + (low >> 3);
          if (high > 0x7F)                                        // Negative Temperatur
          {
            temperature -= 0x1000;
          }
          temperature = (temperature * 6) + (temperature / 4);    // temperature * 6.25
          sensor_LastValue = strrealform((double)temperature / 100.0, 32, 1, 2, true, false) + "°C";
          if (messages)
          {
            Serial.println("ALL3006_Temperature: " + strrealform((double)temperature / 100.0, 32, 1, 2, true, false) + chr(176) + "C");
          }
        }
      }
      else
      {
        if (messages)
        {
          Serial.println("ALL3006_Temperature: Sensor has been removed");
        }
        temperature = -128;                                       // Sensor nicht vorhanden, wurde im Nachhinein entfernt
      }
    }
    else
    {
      if (messages)
      {
        Serial.println("ALL3006_Temperature: Sensor not found");
      }
      temperature = -128;                                         // Sensor nicht vorhanden
    }
    i2c_SendStop();
  }
  else
  {
    if (messages)
    {
      Serial.println("ALL3006_Temperature: Library not initialized");
    }
    temperature = -127;                                           // Library not initialized
  }
  if (temperature < 0)
  {
    return ((double)temperature);
  }
  else
  {
    return ((double)temperature / 100.0);
  }
}

double ALL3008_Temperature (bool messages)                        // Temperatur in °C
{
  long temperature = 0;
  if (i2c_Inited)
  {
    if (sensor_ConnectedType == "ALL 3008")
    {
      i2c_SendStart();
      byte result = i2c_SendByte(145 + 2 * 7, 2);                 // Adresse senden für ALL3008 und ACK erwarten
      if (result == 0)
      {
        byte high = i2c_ReceiveByte(1);                           // Byte lesen und ACK senden
        byte low  = i2c_ReceiveByte(0);                           // Byte lesen und NAK senden
        if ((high == 0xFF) && (low == 0xFF))
        {
          if (messages)
          {
            Serial.println("ALL3008_Temperature: Invalid temperature value");
          }
          temperature = -129;                                     // Sensor-Daten konnten nicht korrekt gelesen werden oder Sensor nicht vorhanden
        }
        else
        {
          temperature = ((high & 0x7F) << 5) + (low >> 3);
          if (high > 0x7F)                                        // Negative Temperatur
          {
            temperature -= 0x1000;
          }
          temperature = (temperature * 3) + (temperature / 8);    // temperature * 3.125
          sensor_LastValue = strrealform((double)temperature / 100.0, 32, 1, 2, true, false) + "°C";
          if (messages)
          {
            Serial.println("ALL3008_Temperature: " + strrealform((double)temperature / 100.0, 32, 1, 2, true, false) + chr(176) + "C");
          }
        }
      }
      else
      {
        if (messages)
        {
          Serial.println("ALL3008_Temperature: Sensor has been removed");
        }
        temperature = -128;                                       // Sensor nicht vorhanden, wurde im Nachhinein entfernt
      }
    }
    else
    {
      if (messages)
      {
        Serial.println("ALL3008_Temperature: Sensor not found");
      }
      temperature = -128;                                         // Sensor nicht vorhanden
    }
    i2c_SendStop();
  }
  else
  {
    if (messages)
    {
      Serial.println("ALL3008_Temperature: Library not initialized");
    }
    temperature = -127;                                           // Library not initialized
  }
  if (temperature < 0)
  {
    return ((double)temperature);
  }
  else
  {
    return ((double)temperature / 100.0);
  }
}

double ALL3018_Temperature (bool messages)                        // Temperatur in °C
{
  long temperature = 0;
  if (i2c_Inited)
  {
    if ((sensor_ConnectedType == "ALL 3018") || (sensor_ConnectedType == "ALL 4529"))
    {
      i2c_SendStartSHT();                                         // Sonderfall für den SHT71 Chip
      byte result = i2c_SendByte(3, 2);                           // Adresse senden für ALL3018 und ACK erwarten
      if (result == 0)
      {
        i2c_Delay(285);                                           // 285 ms. warten, bevor Werte abgerufen werden
        byte high = i2c_ReceiveByte(1);                           // Byte lesen und ACK senden
        byte low  = i2c_ReceiveByte(1);                           // Byte lesen und ACK senden
        byte crc  = i2c_ReceiveByte(0);                           // Byte lesen und NAK senden
        if ((high == 0xFF) && (low == 0xFF))
        {
          if (messages)
          {
            Serial.println("ALL3018_Temperature: Invalid temperature value");
          }
          temperature = -129;                                     // Sensor-Daten konnten nicht korrekt gelesen werden oder Sensor nicht vorhanden
        }
        else
        {
          double SOt = ((long)high << 8) + (long)low;             // Messwert 14-bit
          double d1 = -39.65;                                     // 3.5V: -39.7, 3V: -39.6
          double d2 = 0.01;
          double Tlin = d1 + d2 * SOt;
          temperature = double(Tlin * 100.0);
          sensor_LastValue = strrealform((double)temperature / 100.0, 32, 1, 2, true, false) + "°C";
           if (messages)
          {
            Serial.println("ALL3018_Temperature: " + strrealform((double)temperature / 100.0, 32, 1, 2, true, false) + chr(176) + "C");
          }
        }
      }
      else
      {
        if (messages)
        {
          Serial.println("ALL3018_Temperature: Sensor has been removed");
        }
        temperature = -128;                                       // Sensor nicht vorhanden, wurde im Nachhinein entfernt
      }
    }
    else
    {
      if (messages)
      {
        Serial.println("ALL3018_Temperature: Sensor not found");
      }
      temperature = -128;                                        // Sensor nicht vorhanden
    }
    i2c_SendStop();
  }
  else
  {
    if (messages)
    {
      Serial.println("ALL3018_Temperature: Library not initialized");
    }
    temperature = -127;                                          // Library not initialized
  }
  if (temperature < 0)
  {
    return ((double)temperature);
  }
  else
  {
    return ((double)temperature / 100.0);
  }
}

double ALL3018_Humidity (bool messages)                          // Feuchtigkeit in % (Prozent)
{
  long humidity = 0;
  if (i2c_Inited)
  {
    if ((sensor_ConnectedType == "ALL 3018") || (sensor_ConnectedType == "ALL 4529"))
    {
      i2c_SendStartSHT();                                        // Sonderfall für den SHT71 Chip
      byte result = i2c_SendByte(5, 2);                          // Adresse senden für ALL3018 und ACK erwarten
      if (result == 0)
      {
        i2c_Delay(285);                                          // 285 ms. warten, bevor Werte abgerufen werden
        byte high = i2c_ReceiveByte(1);                          // Byte lesen und ACK senden
        byte low  = i2c_ReceiveByte(1);                          // Byte lesen und ACK senden
        byte crc  = i2c_ReceiveByte(0);                          // Byte lesen und NAK senden
        if ((high == 0xFF) && (low == 0xFF))
        {
          if (messages)
          {
            Serial.println("ALL3018_Humidity: Invalid humidity value");
          }
          humidity = -129;                                       // Sensor-Daten konnten nicht korrekt gelesen werden oder Sensor nicht vorhanden
        }
        else
        {
          humidity = ((long)high << 8) + (long)low;              // Messwert
          if (humidity <= 1712)
          {
            humidity = 143 * humidity;
            if (humidity < 8192)                                 // Check for underflow
            {
              humidity = 8192;
            }              
            humidity -= 8192;
          }
          else
          {
            humidity = 111 * humidity + 46288;
            if (humidity > 409600) 
            {
              humidity = 409600;                                 // check for overflow (optional)
            }
          }
          humidity = humidity * 20 / 819;                        // result = result /40.96
          sensor_LastValue = strrealform((double)humidity / 100.0, 32, 1, 2, true, false) + "%";
          if (messages)
          {
            Serial.println("ALL3018_Humidity: " + strrealform((double)humidity / 100.0, 32, 1, 2, true, false) + "%");
          }
        }
      }
      else
      {
        if (messages)
        {
          Serial.println("ALL3018_Humidity: Sensor has been removed");
        }
        humidity = -128;                                         // Sensor nicht vorhanden, wurde im Nachhinein entfernt
      }
    }
    else
    {
      if (messages)
      {
        Serial.println("ALL3018_Humidity: Sensor not found");
      }
      humidity = -128;                                           // Sensor nicht vorhanden
    }
    i2c_SendStop();
  }
  else
  {
    if (messages)
    {
      Serial.println("ALL3018_Humidity: Library not initialized");
    }
    humidity = -127;                                             // Library not initialized
  }
  if (humidity < 0)
  {
    return ((double)humidity);
  }
  else
  {
    return ((double)humidity / 100.0);
  }
}

double ALL4454_Toxicity (bool messages)                          // Gas-Toxizität in ‰ (Promille)
{
  double multiplier = 0.32767;
  long toxicity = 0;
  if (i2c_Inited)
  {
    if (sensor_ConnectedType == "ALL 4454")
    {
      i2c_SendStart();
      byte result = i2c_SendByte(144 + 2 * 3, 2);                // Adresse senden für ALL4454 und ACK erwarten
      if (result == 0)
      {
        result = i2c_SendByte(156, 2);                           // 16(15) bit 8 sps: Byte senden, ACK ermitteln: 1001 1100 DR1:1,DR0:1
        if (result != 0)
        {
          if (messages)
          {
            Serial.println("ALL4454_Toxicity: Sensor not found");
          }
          toxicity = -128;                                       // Sensor nicht vorhanden
        }
        else
        {
          i2c_SendStop();
          for (int i = 0; i < 16; i++)                           // Maximal 16 Versuche unternehmen, um einen gültigen Wert auszulesen
          {
            i2c_Delay(10);                                       // 285 ms. warten, bevor Werte abgerufen werden
            i2c_SendStart();
            result = i2c_SendByte(145 + 2 * 3, 2);               // Byte senden, ACK ermitteln: 145=10010xx1 = Read from ID #address (ADS1100_ADx)
            if (result != 0)
            {
              if (messages)
              {
                Serial.println("ALL4454_Toxicity: Sensor not found");
              }
              toxicity = -128;                                   // Sensor nicht vorhanden,
              i = 16;                                            // Schleife vorzeitig verlassen
            }
            else
            {
              byte high = i2c_ReceiveByte(1);                    // Byte lesen und ACK senden
              byte low  = i2c_ReceiveByte(1);                    // Byte lesen und ACK senden
              byte stat = i2c_ReceiveByte(0);                    // Byte lesen und NAK senden
              if (((high == 0xFF) && (low == 0xFF)) || ((high == 0x07) && (low == 0xFF)))
              {
                if (messages)
                {
                  Serial.println("ALL4454_Toxicity: Invalid value");
                }
                toxicity = -129;                                 // Sensor-Daten konnten nicht korrekt gelesen werden oder Sensor nicht vorhanden
              }
              else
              {
                if (stat < 0x80)
                {
                  toxicity = ((long)high << 8) + (long)low;      // 16 Bit Wert, davon 15 Bit positiver Wert sowie 0x7FFF = Fehler
                  sensor_LastValue = strrealform((double)toxicity / multiplier, 32, 1, 2, true, false) + "‰";
                  if (messages)
                  {
                    Serial.println("ALL4454_Toxicity: " + strrealform((double)toxicity / multiplier, 32, 1, 2, true, false));
                  }
                  i = 16;                                        // Schleife vorzeitig verlassen,
                }                                                // korrekte Werte wurden empfangen
              }
            }
            i2c_SendStop();
          }                                                      // Ende der for-Schleife
        }
      }
      else
      {
        if (messages)
        {
          Serial.println("ALL4454_Toxicity: Sensor has been removed");
        }
        toxicity = -128;                                         // Sensor nicht vorhanden, wurde im Nachhinein entfernt
      }
      i2c_SendStop();
    }
    else
    {
      if (messages)
      {
        Serial.println("ALL4454_Toxicity: Sensor not found");
      }
      toxicity = -128;                                           // Sensor nicht vorhanden
    }
  }
  else
  {
    if (messages)
    {
      Serial.println("ALL4454_Toxicity: Library not initialized");
    }
    toxicity = -127;                                             // Library not initialized
  }
  if (toxicity < 0)
  {
    return ((double)toxicity);
  }
  else
  {
    return ((double)toxicity / multiplier);                      // 15 Bit Wertebereich in Promill umrechnen
  }
}

long ALL4529_GetCalibration (byte command, bool messages)
{
  i2c_SendStart();                                                // Kalibration starten
  byte result = i2c_SendByte(238, 2);                             // Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
  if (result != 0)
  {
    if (messages)
    {
      Serial.println("ALL4529_Measure: Sensor has been removed");
    }
     i2c_SendStop();
    return (-128);                                                // Sensor nicht vorhanden
  }
  result = i2c_SendByte(command, 2);                              // Byte senden, ACK ermitteln Register Address (command)
  if (result != 0)
  {
    if (messages)
    {
      Serial.println("ALL4529_Measure: Sensor not found");
    }
    i2c_SendStop();
    return (-128);                                                // Sensor nicht vorhanden
  }
  i2c_SendStart();                                                // i2c Restart senden
  result = i2c_SendByte(239, 2);                                  // Byte senden, ACK ermitteln: 3=1110 1111 = Read from BMP085 ($EF)
  if (result != 0)
  {
    if (messages)
    {
      Serial.println("ALL4529_Measure: Sensor not found");
    }
    i2c_SendStop();
    return (-128);                                                // Sensor nicht vorhanden
  }
  byte high = i2c_ReceiveByte(1);                                 // Byte lesen und ACK senden
  byte low  = i2c_ReceiveByte(0);                                 // Byte lesen und NAK senden
  i2c_SendStop();
  if ((high == 0xFF) && (low == 0xFF))
  {
    if (messages)
    {
      Serial.println("ALL4529_Measure: Invalid value");
    }
    i2c_SendStop();
    return (-129);                                                // Sensor-Daten konnten nicht korrekt gelesen werden oder Sensor nicht vorhanden
  }
  return (((long)high << 8) + (long)low);
}

long ALL4529_SendCommand (byte value, bool messages)
{
  i2c_SendStart();
  byte result = i2c_SendByte(238, 2);                             // Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
  if (result != 0)
  {
    if (messages)
    {
      Serial.println("ALL4529_Measure: Sensor not found");
    }
    i2c_SendStop();
    return (-128);                                                // Sensor nicht vorhanden
  }
  result = i2c_SendByte(244, 2);                                  // Byte senden, ACK ermitteln Register Address $F4 (Control Register)
  if (result != 0)
  {
    if (messages)
    {
      Serial.println("ALL4529_Measure: Sensor not found");
    }
    i2c_SendStop();
    return (-128);                                                // Sensor nicht vorhanden
  }
  result = i2c_SendByte(value, 2);                                // Byte senden, ACK ermitteln Register Data $F4 (Pressure osrs=3)
  if (result != 0)
  {
    if (messages)
    {
      Serial.println("ALL4529_Measure: Sensor not found");
    }
    i2c_SendStop();
    return (-128);                                                // Sensor nicht vorhanden
  }
  i2c_SendStop();
}

long ALL4529_GetValue (bool messages)
{
  i2c_SendStart();
  byte result = i2c_SendByte(238, 2);                             // Byte senden, ACK ermitteln: 3=1110 1110 = Write to BMP085 ($EE)
  if (result != 0)
  {
    if (messages)
    {
      Serial.println("ALL4529_Measure: Sensor not found");
    }
    i2c_SendStop();
    return (-128);                                                // Sensor nicht vorhanden
  }
  result = i2c_SendByte(246, 2);                                  // Byte senden, ACK ermitteln Register Address $F4 (Control Register)
  if (result != 0)
  {
    if (messages)
    {
      Serial.println("ALL4529_Measure: Sensor not found");
    }
    i2c_SendStop();
    return (-128);                                                // Sensor nicht vorhanden
  }
  i2c_SendStart();                                                // i2c Restart senden
  result = i2c_SendByte(239, 2);                                  // Byte senden, ACK ermitteln Register Data $F4 (Pressure osrs=3)
  if (result != 0)
  {
    if (messages)
    {
      Serial.println("ALL4529_Measure: Sensor not found");
    }
    i2c_SendStop();
    return (-128);                                                // Sensor nicht vorhanden
  }
  return (0);
}

double ALL4529_Measure (bool messages, bool measurement)          // Luftdruck in mbar (measurement = true) oder Temperatur in °C
{
  if (i2c_Inited)
  {
    if (sensor_ConnectedType == "ALL 4529")                       // BMP085
    {
      int32_t c_AC1 = ALL4529_GetCalibration(0xAA, messages);     // AC1-Kalibration: Byte senden, ACK ermitteln Register Address $AA
      if (c_AC1 < 0) { return (c_AC1); }                          // Fehlercode übergeben
      if (c_AC1 > 32767) c_AC1 -= 0x10000;
      int32_t c_AC2 = ALL4529_GetCalibration(0xAC, messages);     // AC2-Kalibration: Byte senden, ACK ermitteln Register Address $AC
      if (c_AC2 < 0) { return (c_AC2); }                          // Fehlercode übergeben
      if (c_AC2 > 32767) c_AC2 -= 0x10000;
      int32_t c_AC3 = ALL4529_GetCalibration(0xAE, messages);     // AC3-Kalibration: Byte senden, ACK ermitteln Register Address $AE
      if (c_AC3 < 0) { return (c_AC3); }                          // Fehlercode übergeben
      if (c_AC3 > 32767) c_AC3 -= 0x10000;
      int32_t c_AC4 = ALL4529_GetCalibration(0xB0, messages);     // AC4-Kalibration: Byte senden, ACK ermitteln Register Address $B0
      if (c_AC4 < 0) { return (c_AC4); }                          // Fehlercode übergeben
      int32_t c_AC5 = ALL4529_GetCalibration(0xB2, messages);     // AC5-Kalibration: Byte senden, ACK ermitteln Register Address $B2
      if (c_AC5 < 0) { return (c_AC5); }                          // Fehlercode übergeben
      int32_t c_AC6 = ALL4529_GetCalibration(0xB4, messages);     // AC6-Kalibration: Byte senden, ACK ermitteln Register Address $B4
      if (c_AC6 < 0) { return (c_AC6); }                          // Fehlercode übergeben
      int32_t c_B1  = ALL4529_GetCalibration(0xB6, messages);     // B1-Kalibration:  Byte senden, ACK ermitteln Register Address $B6
      if (c_B1 < 0)  { return (c_B1); }                           // Fehlercode übergeben
      if (c_B1 > 32767) c_B1 -= 0x10000;
      int32_t c_B2  = ALL4529_GetCalibration(0xB8, messages);     // B2-Kalibration:  Byte senden, ACK ermitteln Register Address $B8
      if (c_B2 < 0)  { return (c_B2); }                           // Fehlercode übergeben
      if (c_B2 > 32767) c_B2 -= 0x10000;
      int32_t c_MB  = ALL4529_GetCalibration(0xBA, messages);     // MB-Kalibration:  Byte senden, ACK ermitteln Register Address $BA
      if (c_MB < 0)  { return (c_MB); }                           // Fehlercode übergeben
      if (c_MB > 32767) c_MB -= 0x10000;
      int32_t c_MC  = ALL4529_GetCalibration(0xBC, messages);     // MC-Kalibration:  Byte senden, ACK ermitteln Register Address $BC
      if (c_MC < 0)  { return (c_MC); }                           // Fehlercode übergeben
      if (c_MC > 32767) c_MC -= 0x10000;
      int32_t c_MD  = ALL4529_GetCalibration(0xBE, messages);     // MC-Kalibration:  Byte senden, ACK ermitteln Register Address $BE
      if (c_MD < 0)  { return (c_MD); }                           // Fehlercode übergeben
      if (c_MD > 32767) c_MD -= 0x10000;
   
      long result = ALL4529_SendCommand(244, true);               // -----   Luftdruck-Messung starten mit höchstmöglicher Genauigkeit   -----
      if (result < 0) { return (result); }                        // Fehlercode übergeben
      i2c_Delay(26);                                              // 26 ms warten (max. 25.5 ms laut Bosch DB Rev. 1.2 von 2009)
      result = ALL4529_GetValue(true);                            // -----   Luftdruck-Messwert abholen   -----
      if (result < 0) { return (result); }                        // Fehlercode übergeben
      byte high = i2c_ReceiveByte(1);                             // Byte lesen und ACK senden (MSB)
      byte low  = i2c_ReceiveByte(1);                             // Byte lesen und ACK senden (LSB)
      byte xlow = i2c_ReceiveByte(0);                             // Byte lesen und NAK senden (XLSB)
      i2c_SendStop();
      if ((high == 0xFF) && (low == 0xFF))
      {
        if (messages)
        {
          Serial.println("ALL4529_Measure: Invalid value");
        }
        i2c_SendStop();
        return (-129);                                            // Sensor-Daten konnten nicht korrekt gelesen werden oder Sensor nicht vorhanden
      }
      int32_t UP = ((((int32_t)high << 8) + (int32_t)low) << 3) + ((int32_t)xlow >> 5);

      result = ALL4529_SendCommand(46, true);                     // -----   Temperatur-Messung starten   -----
      if (result < 0) { return (result); }                        // Fehlercode übergeben
      i2c_Delay(5);                                               // 5 ms warten (max. 4.5 ms laut Bosch DB Rev. 1.2 von 2009)
      
      result = ALL4529_GetValue(true);                            // -----   Temperatur-Messwert abholen   -----
      high = i2c_ReceiveByte(1);                                  // Byte lesen und ACK senden (MSB)
      low  = i2c_ReceiveByte(0);                                  // Byte lesen und NAK senden (LSB)
      i2c_SendStop();
      if ((high == 0xFF) && (low == 0xFF))
      {
        if (messages)
        {
          Serial.println("ALL4529_Measure: Invalid value");
        }
        i2c_SendStop();
        return (-129);                                            // Sensor-Daten konnten nicht korrekt gelesen werden oder Sensor nicht vorhanden
      }
      int32_t UT = ((long)high << 8) + (long)low;

      int32_t X1 = (UT - (int32_t)c_AC6) * (int32_t)c_AC5 >> 15;  // -----   Temperatur ausrechnen   -----
      int32_t X2 = ((int32_t)c_MC << 11) / (X1 + (int32_t)c_MD);
      int32_t B5 = X1 + X2;
      long temperature = (B5 + 8) >> 4;

      int32_t B6 = B5 - 4000;                                     // -----   Luftdruck ausrechnen   -----
      X1 = ((int32_t)c_B2 * ((B6 * B6) >> 12)) >> 11;
      X2 = ((int32_t)c_AC2 * B6) >> 11;
      int32_t X3 = X1 + X2;
      int32_t B3 = ((((int32_t)c_AC1 * 4 + X3) * 8) + 2) / 4;
      X1 = ((int32_t)c_AC3 * B6) >> 13;
      X2 = ((int32_t)c_B1 * ((B6 * B6) >> 12)) >> 16;
      X3 = ((X1 + X2) + 2) >> 2;
      uint32_t B4 = ((uint32_t)c_AC4 * (uint32_t)(X3 + 32768)) >> 15;
      uint32_t B7 = ((uint32_t)UP - B3) * (uint32_t)((uint32_t)50000 / 8);
      long pressure = 0;
      if (B7 < 0x80000000)
      {
        pressure = (B7 * 2) / B4;
      }
      else
      {
        pressure = (B7 / B4) * 2;
      }
      X1 = (pressure >> 8) * (pressure >> 8);
      X1 = (X1 * 3038) >> 16;
      X2 = (-7357 * pressure) >> 16;
      pressure += ((X1 + X2 + (int32_t)3791) >> 4);

      ALL4529_LastTemperature = (double)temperature /  10.0;
      ALL4529_LastPressure    = (double)pressure    / 100.0;
      ALL4529_LastMeasureTime = millis();

      if (messages)
      {
        if (measurement)
        {
          Serial.println("ALL4529_Pressure: " + strrealform(ALL4529_LastPressure, 32, 1, 2, true, false) + " mBar");
        }
        else
        {
         Serial.println("ALL4529_Temperature: " + strrealform(ALL4529_LastTemperature, 32, 1, 1, true, false) + chr(176) + "C");
        }
      }
      if (measurement)
      {
        return (ALL4529_LastPressure);                            // Luftdruck in mbar (hPa)
      }
      else
      {
        return (ALL4529_LastTemperature);                         // Temperatur in °C
      }
    }
    else
    {
      if (messages)
      {
        Serial.println("ALL4529_Measure: Sensor not found");
      }
      return (-128);                                              // Sensor nicht vorhanden
    }
  }
  else
  {
    if (messages)
    {
      Serial.println("ALL4529_Measure: Library not initialized");
    }
    return (-127);                                                // Library not initialized
  }
}

double ALL4529_Pressure (bool messages)                           // Luftdruck in mbar (hPa)
{
  if (difference(millis(), ALL4529_LastMeasureTime) > 5000)       // Messungen nur alle 5 Sekunden durchführen
  {
    return (ALL4529_Measure(messages, true));
  }
  else
  {
    Serial.println("ALL4529_Pressure: " + strrealform(ALL4529_LastPressure, 32, 1, 2, true, false) + " mBar (last value)");
    return (ALL4529_LastPressure);
  }
}

double ALL4529_Temperature (bool messages)                        // Temperatur in °C
{
  if (difference(millis(), ALL4529_LastMeasureTime) > 5000)       // Messungen nur alle 5 Sekunden durchführen
  {
    return (ALL4529_Measure(messages, false));
  }
  else
  {
    Serial.println("ALL4529_Temperature: " + strrealform(ALL4529_LastTemperature, 32, 1, 1, true, false) + chr(176) + "C (last value)");
    return (ALL4529_LastTemperature);
  }
}
