// String Library
// 1.00 - 2017-05-17
// 1.01 - 2017-06-23
// 1.02 - 2017-07-18
// 1.03 - 2017-07-27
// 1.04 - 2017-11-15
// 1.05 - 2018-02-10
// (c) Copyright 2017-2018
//
// Zur Verwendung mit allen Allnet Libraries


// *** Globale Konstanten ***

const char b64_Alphabet [] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
		             "abcdefghijklmnopqrstuvwxyz"
		             "0123456789+/";


// *** Globale Variablen ***

int64  rnd_0 = 0;        // Die zuletzt erzeugte ganzzahlige Zufallszahl
double rndreal_0 = 0.0;  // Die zuletzt erzeugte Fließkomma Zufallszahl


// *** Math-Funktionen ***

double minimum (double x, double y)
{
  if (x < y)
  {
    return x;
  }
  else
  {
    return y;
  }
}

double maximum (double x, double y)
{
  if (x > y)
  {
    return x;
  }
  else
  {
    return y;
  }
}

double difference (double x, double y)
{
  return (maximum(x, y) - minimum(x, y));
}

bool odd (long n)
{
  return ((n & 0x01) == 1);
}

bool even (long n)
{
  return ((n & 0x01) == 0);
}

byte sgn (double n)
{
  if (n > 0)
  {
    return 1;
  }
  else if (n < 0)
  {
    return -1;
  }
  else
  {
    return 0;
  }
}

byte boolval (bool b)
{
    if (b)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

int64 exponent2 (byte n)
{
  if ((n < 0) || (n > 63))
  {
    return 0;
  }
  else
  {
    int64 i = (int64)1 << n;
    return (i);                // exp2(63) = -9223372036854775808 wegen Vorzeichen
  }
}

int64 exponent10 (byte n)
{
  if ((n < 0) || (n > 18))
  {
    return 0;
  }
  else
  {
    long result = 1;
    for (byte i = 1; i <= n; i++)
    {
      result *= 10;
    }
    return result;
  }
}

double frac (double n)
{
  return n - trunc(n);
}


// *** Mathematische Funktionen für hexadezimale und BCD-Verarbeitung von Zahlen ***

byte lobyte (uint64 n)
{
    return (n & 0xFF);
}

byte hibyte (uint64 n)
{
    return ((n & 0xFF00) >> 8);
}

uint16 loword (uint64 n)
{
    return (n & 0xFFFF);
}

uint16 hiword (uint64 n)
{
    return (n >> 16);
}

byte bcd2bin (byte val)
{
  return val - 6 * (val >> 4);
}

byte bin2bcd (byte val)
{
  return val + 6 * (val / 10);
}


// *** Zufallszahlen- & Kryptographie-Funktionen ***

int64 crc64 (int64 Old64BitCRC, byte AddThis)
{
  int64 hash = Old64BitCRC;
  int64 crc = hash;
  for (int i = 1; i <= 8; i++)
  {
    if (hash < 0) // 0001 0000 0010 0001 0001 0000 0001 0001 0001 0000 0010 0001 0001 0000 0000 0001
    {             // = 116.222.784.430.938.931 = 0x1021101110211001 (Primzahl)
      crc = (0x1021101110211001 ^ (crc << 1));
    }
    else
    {
      crc <<= 1;
    }
    hash = (int64(AddThis) ^ crc);
  }
  return (hash);
}

int64 crc64 (String s, int64 hash)
{
  int L = s.length();
  if (L > 0)
  {
    for (int i = 0; i < L; i++)
    {
      hash = crc64(hash, s[i] & 0xFF);
    }
  }
  return (hash);
}

uint64 xorshift128plus (uint64 seed0, uint64 seed1)
{
  uint64 state0 = seed0;
  uint64 state1 = seed1;
  if ((state0 == 0) && (state1 == 0))
  {
    state0 = 1; // Beide Statusvariablen dürfen nicht 0 sein
  }
  uint64 s1 = state0;
  uint64 s0 = state1;
  state0 = s0;
  s1 ^= s1 << 23;
  s1 ^= s1 >> 17;
  s1 ^= s0;
  s1 ^= s0 >> 26;
  state1 = s1;
  return ((state0 & 0x7FFFFFFFFFFFFFFF) + (state1 & 0x7FFFFFFFFFFFFFFF));
}

uint64 xorshift128plus (String s, uint64 hash)
{
  int L = s.length();
  if (L > 0)
  {
    for (int i = 0; i < L; i++)
    {
      hash = xorshift128plus(hash, s[i]);
    }
  }
  return (hash);
}

byte rndbyte () // Statistisch und absolut gesehen eine richtige Zufallszahl zwischen 0 und 255
{
    int count = 0;
    long a = 0;
    long b = 0;
    while (((a == 0) || (b == 0)) && (count < 100))
    {
        a = random(65536) & 65535; // Es werden mit random() bei jedem Programmstart die gleichen Zufallszahlen erzeugt
        b = micros() & 65535;
        count += 1;
    }
    return (hiword(a * b) & 0xFF);
}

int64 rnd (int64 n) // Zufallszahl zwischen 0 und n, inclusive 0 und n
{
  if (n != 0)
  {
    int64 zahl = 0;
    for (byte i = 0; i <= 7; i++)
    {
      if (i != 7)
      {
        zahl <<= 8;
        zahl |= rndbyte();
      }
      else
      {
        zahl <<= 7;
        zahl |= (rndbyte() & 0x7F);
      }
    }
    rnd_0 = zahl % (n + 1);
  }
  return (rnd_0);
}

byte rndbytesecure () // Statistisch und absolut gesehen eine richtige Zufallszahl, wesentlich sicherer als rndbyte
{
    uint64 urand1 = (uint64(rnd(0xFFFFFFFF)) << 32) | uint64(rnd(0xFFFFFFFF));
    uint64 urand2 = (uint64(rnd(0xFFFFFFFF)) << 32) | uint64(rnd(0xFFFFFFFF));
    uint64 rand64 = xorshift128plus(urand1, urand2);
    return byte(rand64 & 0xFF);
}

double rndreal (double n) // Zufallszahl zwischen 0.0 und n, inclusive 0.0 und n
{
  if (n != 0.0)
  {
    rndreal_0 = double(rnd(9999999999)) / 9999999999.0 * n;
  }
  return (rndreal_0);
}


// *** Interne Base64 Konvertierungsfunktionen ***

inline void a3_to_a4 (unsigned char * a4, unsigned char * a3)
{
  a4[0] = (a3[0] & 0xfc) >> 2;
  a4[1] = ((a3[0] & 0x03) << 4) + ((a3[1] & 0xf0) >> 4);
  a4[2] = ((a3[1] & 0x0f) << 2) + ((a3[2] & 0xc0) >> 6);
  a4[3] = (a3[2] & 0x3f);
}

inline void a4_to_a3 (unsigned char * a3, unsigned char * a4)
{
  a3[0] = (a4[0] << 2) + ((a4[1] & 0x30) >> 4);
  a3[1] = ((a4[1] & 0xf) << 4) + ((a4[2] & 0x3c) >> 2);
  a3[2] = ((a4[2] & 0x3) << 6) + a4[3];
}

inline unsigned char b64_lookup (char c)
{
  int i;
  for (i = 0; i < 64; i++)
  {
    if (b64_Alphabet[i] == c)
    {
      return (i);
    }
  }
  return (-1);
}


// *** Base64 Low-Level Funktionen ***

int b64_Encode (char *output, char *input, int inputLen)
{
  int i = 0, j = 0;
  int encLen = 0;
  unsigned char a3[3];
  unsigned char a4[4];

  while(inputLen--)
  {
    a3[i++] = *(input++);
    if(i == 3)
    {
      a3_to_a4(a4, a3);
      for(i = 0; i < 4; i++)
      {
        output[encLen++] = b64_Alphabet[a4[i]];
      }
      i = 0;
    }
  }
  if (i)
  {
    for (j = i; j < 3; j++)
    {
      a3[j] = '\0';
    }
    a3_to_a4(a4, a3);
    for (j = 0; j < i + 1; j++)
    {
      output[encLen++] = b64_Alphabet[a4[j]];
    }
    while((i++ < 3))
    {
      output[encLen++] = '=';
    }
  }
  output[encLen] = '\0';
  return (encLen);
}

int b64_Decode (char * output, char * input, int inputLen)
{
  int i = 0, j = 0;
  int decLen = 0;
  unsigned char a3[3];
  unsigned char a4[4];

  while (inputLen--)
  {
    if (*input == '=')
    {
      break;
    }
    a4[i++] = *(input++);
    if (i == 4)
    {
      for (i = 0; i <4; i++)
      {
        a4[i] = b64_lookup(a4[i]);
      }
      a4_to_a3(a3,a4);
      for (i = 0; i < 3; i++)
      {
        output[decLen++] = a3[i];
      }
      i = 0;
    }
  }

  if (i)
  {
    for (j = i; j < 4; j++)
    {
      a4[j] = '\0';
    }
    for (j = 0; j <4; j++)
    {
      a4[j] = b64_lookup(a4[j]);
    }
    a4_to_a3(a3,a4);
    for (j = 0; j < i - 1; j++)
    {
      output[decLen++] = a3[j];
    }
  }
  output[decLen] = '\0';
  return (decLen);
}

int b64_EncodedLength (int plainLen)
{
  int n = plainLen;
  return ((n + 2 - ((n + 2) % 3)) / 3 * 4);
}

int b64_DecodedLength (char * input, int base64Len)
{
  int i = 0;
  int numEq = 0;
  for (i = base64Len - 1; input[i] == '='; i--)
  {
    numEq++;
  }
  return (((6 * base64Len) / 8) - numEq);
}


// *** String-Funktionen ***

String chr (int c)
{
  String s = "";
  s += char(c);
  return s;
}

int asc (String s)
{
  if (s == "")
  {
    return 0;
  }
  else
  {
    return s.charAt(0);
  }
}

int len (String s)
{
  return s.length();
}

bool number (byte n)
{
  return ((n >= 48) && (n <= 57)); // "0" bis "9"
}

bool letter (byte n)
{
  return (((n >= 65) && (n <= 90)) || ((n >= 97) && (n <= 122))); // "A" bis "Z" und "a" bis "z"
}

String spc (int length)
{
  String s = "";
  if (length > 0)
  {
    if (length > 32767)
    {
      length = 32767;
    }
    for (int i = 1; i <= length; i++)
    {
      s += " ";
    }
  }
  return s;
}

String ucase (String s)
{
  s.toUpperCase();
  return s;
}

String lcase (String s)
{
  s.toLowerCase();
  return s;
}

String boolstr (bool b, String caseTrue, String caseFalse)
{
  if (b)
  {
    return (caseTrue);
  }
  else
  {
    return (caseFalse);
  }
}

String left (String s, int length)
{
  if (length < 1)
  {
    return "";
  }
  else if (length >= s.length())
  {
    return s;
  }
  else
  {
    return s.substring(0, length);
  }
}

String part (String s, int start)
{
  if (start <= 1)
  {
    return s;
  }
  else if (start > s.length())
  {
    return "";
  }
  else
  {
    return (s.substring(start - 1));
  }
}

String right (String s, int length)
{
  int L = s.length();
  if (L <= length)
  {
    return s;
  }
  else if (length < 1)
  {
    return "";
  }
  else
  {
    return s.substring(L - length, L);
  }
}

String mid (String s, int start, int length)
{
  int L = s.length();
  if (start <= 1)
  {
    return left(s, length);
  }
  else if ((start > L) || (length < 1))
  {
    return "";
  }
  else
  {
    if (length >= (L - start + 1))
    {
      return s.substring(start - 1);
    }
    else
    {
      return s.substring(start - 1, start + length - 1);
    }
  }
}

String replace (String s, String oldstr, String newstr)
{
  s.replace(oldstr, newstr);
  return s;
}

int position (String s, String t, int start)
{
  if (start <= 1)
  {
    return s.indexOf(t) + 1;
  }
  else
  {
    return s.indexOf(t, start - 1) + 1;
  }
}

String indchar (String s, int i)
{
  if ((i >= 1) && (i <= s.length()))
  {
    return chr(s[i - 1]);
  }
  else
  {
    return ("");
  }
}

int indasc (String s, int i)
{
  if ((i >= 1) && (i <= s.length()))
  {
    return (s[i - 1]);
  }
  else
  {
    return (0);
  }
}

String deleteasc (String s, int p)
{
    return (left(s, p - 1) + part(s, p + 1));
}

String repeatstr (String s, byte n)
{
    String r = "";
    if (n > 0)
    {
        for (int i = 1; i <= n; i++)
        {
            r = r + s;
        }
    }
    return (r);
}

String repeatasc (int a, int n)
{
    String r = "";
    String zchn = chr(a);
    if (n > 0)
    {
        for (int i = 1; i <= n; i++)
        {
            r = r + zchn;
        }
    }
    return (r);
}

String strform (int64 n, int asc, byte MinVorkomma, bool TausenderPunkt)
{
  String vorzeichen = "";
  if (n < 0)
  {
    vorzeichen = "-";
    //n = abs(n);
  }
  String vorkomma = "";
  if (n == 0)
  {
    vorkomma = "0";
  }
  else
  {
    while (n !=  0)
    {
      vorkomma = chr(abs(n % 10) + 48) + vorkomma;
      n = n / 10;
    }
  }
  if (TausenderPunkt)
  {
      String s = "";
      byte l = vorkomma.length();
      for (int i = 0; i < l; i++)
      {
          if ((((l - i) % 3) == 0) && (s != ""))
          {
              s = s + chr(46); // "."
          }
          s = s + indchar(vorkomma, i + 1);
      }
      vorkomma = s;
  }
  vorkomma = vorzeichen + vorkomma;
  byte VL = vorkomma.length();
  if (MinVorkomma > VL)
  {
      vorkomma = repeatasc(asc, MinVorkomma - VL) + vorkomma;
  }
  return (vorkomma);
}

String str (int64 n)
{
  return strform(n, 32, 1, true);
}

String strrealform (double n, byte asc, byte MinVorkomma, byte MaxNachkomma, bool TausenderPunkt, bool CutZero)
{
  String vorzeichen = "";
  if (n < 0) // Negativ
  {
    vorzeichen = "-";
    n = abs(n);
  }
  if (MaxNachkomma > 9) // float ist auf 7 Stellen genau, Double auf 9 Stellen
  {
    MaxNachkomma = 9;
  }
  n = n + (1.0 / exponent10(MaxNachkomma) / 2.0); // Runden
  String result = "";
  long mantissa = n;
  String vorkomma = String(mantissa);
  if (TausenderPunkt)
  {
    String s = "";
    byte l = vorkomma.length();
    for (int i = 0; i < l; i++)
    {
      if ((((l - i) % 3) == 0) && (s != ""))
      {
        s = s + chr(46); // "."
      }
      s = s + indchar(vorkomma, i + 1);
    }
    vorkomma = s;
  }
  vorkomma = vorzeichen + vorkomma;
  byte VL = vorkomma.length();
  if (MinVorkomma > VL)
  {
    vorkomma = repeatasc(asc, MinVorkomma - VL) + vorkomma;
  }
  double nachkomma = frac(n);
  if ((nachkomma != 0) || not(CutZero))
  {
    vorkomma = vorkomma + ",";
    for (byte i = 1; i <= MaxNachkomma; ++i)
    {
      nachkomma = nachkomma * 10;
      vorkomma = vorkomma + String((long)nachkomma);
      nachkomma = nachkomma - trunc(nachkomma);
    }
    if (CutZero)
    {
      while (right(vorkomma, 1) == "0")
      {
        vorkomma = left(vorkomma, vorkomma.length() - 1);
      }
    }
    if (right(vorkomma, 1) == ",")
    {
      vorkomma = left(vorkomma, vorkomma.length() - 1);
    }
  }
  return vorkomma;
}

String strrealform (double n, byte asc, byte MinVorkomma, byte MaxNachkomma, bool TausenderPunkt, bool CutZero, bool ShowPlus)
{
  String s = strrealform(n, asc, MinVorkomma, MaxNachkomma, TausenderPunkt, CutZero);
  if ((n > 0) && ShowPlus)
  {
     s = "+" + s;
  }
  return s;
}

String strreal (double n, int Nachkomma)
{
  bool CutZero = (Nachkomma == 0);
  if ((Nachkomma < 1) || (Nachkomma > 9))
  {
    Nachkomma = 9;
  }
  return strrealform(n, 32, 1, Nachkomma, true, CutZero);
}

#ifdef WiFi_h
  String strIP (IPAddress ip, bool leadingZero)
  {
    if (leadingZero)
    {
      return (strform(ip[0], 48, 3, false) + "." + strform(ip[1], 48, 3, false) + "." +
              strform(ip[2], 48, 3, false) + "." + strform(ip[3], 48, 3, false));
    }
    else
    {
      return (str(ip[0]) + "." + str(ip[1]) + "." + str(ip[2]) + "." + str(ip[3]));
    }
  }
#endif

int64 val (String s, bool ignoredot)
{
  bool vorzeichen = false;
  if (s == "")
  {
    return 0;
  }
  else
  {
    byte p = 0;
    byte L = s.length();
    int64 result = 0;
    if (s[0] == 45) // "-"
    {
      vorzeichen = true;
      p = 1;
    }
    while (p < L)
    {
      byte ziffer = s[p];
      ++p;
      if ((ziffer >= 48) && (ziffer <= 57)) // "0" bis "9"
      {
        result = (result * 10) + ziffer - 48;
      }
      else
      {
        if (ziffer == 46) // Tausenderpunkt ggf. ignorieren
        {
          if (not(ignoredot))
          {
            p = L; // Abbruch bei illegalen Zeichen
          }
        }
        else
        {
          p = L; // Abbruch bei illegalen Zeichen
        }
      }
    }
    if (vorzeichen)
    {
      return -result;
    }
    else
    {
      return result;
    }
  }
}

int64 val (String s)
{
  return (val(s, true));
}

double realval (String s)  // Funktioniert besser als "s.toFloat()"
{
  bool vorzeichen = false;
  double r = 0.0;
  int p = 1;
  String zahl = "";
  if (indasc(s, 1) == 45) // "-"
  {
    zahl = part(s, 2);
    vorzeichen = true;
  }
  else
  {
    zahl = s;
  }
  int a = indasc(zahl, p);
  while ((a != 44) && (a != 46) && ((a >= 48) && (a <= 57))) // "," oder "." oder Zahl 0-9
  {
    p += 1;
    a = indasc(zahl, p);
  }
  if ((a != 44) && (a != 46)) // Kein Punkt oder Komma = Ganzzahl
  {
    r = double(val(zahl, false));
  }
  else // Zahl mit Nachkommateil
  {
    int t = len(zahl);
    double fract = 0.0;
    a = indasc(zahl, p);
    if ((a == 44) || (a == 46)) // "," oder "."
    {
      String mantissa = part(zahl, p + 1);
      t = p - 1;
      int L = len(mantissa);
      int i = 1;
      while (i <= L)
      {
        int c = indasc(mantissa, i);
        if (not(number(c)))
        {
          L = i - 1;
        }
        else
        {
          i += 1;
        }
      }
      while (L > 0)
      {
        int c = indasc(mantissa, L);
        if (number(c))
        {
          fract /= 10;
          fract += double(c - 48);
          p += 1;
          L -= 1;
        }
        else
        {
          L = 0;
        }
      }
      fract /= 10;
      p += 1;
    }
    r = double(val(left(zahl, t), false)) + fract;
  }
  a = indasc(zahl, p);
  if ((a == 69) || (a == 101)) // Exponent
  {
    int expo = val(part(zahl, p + 1), false);
    if (expo != 0)
    {
      for (int i = 1; i <= abs(expo); i++)
      {
        if (expo > 0)
        {
          r *= 10;
        }
        else
        {
          r /= 10;
        }
      }
    }
  }
  if (vorzeichen)
  {
    return(-r);
  }
  else
  {
    return(r);
  }
}

#ifdef WiFi_h
  IPAddress IPval (String s)
  {
    int v1, v2, v3, v4;
    int p = 1;
    p = position(s, ".", 1);
    if (p > 1)
    {
      v1 = val(left(s, p - 1));
      s = part(s, p + 1);
    }
    else
    {
      return(IPAddress(0, 0, 0, 0));                                  // Fehlerhafte IP-Adresse
    }
    p = position(s, ".", 1);
    if (p > 1)
    {
      v2 = val(left(s, p - 1));
      s = part(s, p + 1);
    }
    else
    {
      return(IPAddress(0, 0, 0, 0));                                  // Fehlerhafte IP-Adresse
    }
    p = position(s, ".", 1);
    if (p > 1)
    {
      v3 = val(left(s, p - 1));
      s = part(s, p + 1);
    }
    else
    {
      return(IPAddress(0, 0, 0, 0));                                  // Fehlerhafte IP-Adresse
    }
    v4 = val(s);
    if ((v1 >= 0) && (v1 <= 255) && (v2 >= 0) && (v2 <= 255) &&
        (v3 >= 0) && (v3 <= 255) && (v4 >= 0) && (v4 <= 255)) 
    {
      return(IPAddress(v1, v2, v3, v4));                              // Gültige IP-Adresse
    }
    else
    {
      return(IPAddress(0, 0, 0, 0));                                  // Fehlerhafte IP-Adresse
    }
  }
#endif

String fillcenter (String s, String fill, int width)
{
  int L = s.length();
  if (L < width)
  {
    s = repeatstr(fill, (width - L) / 2) + s;
    s += repeatstr(fill, width - L - (width - L) / 2);
  }
  return (s);
}


// *** Hexadezimal-Funktionen ***

bool hexnumber (byte n)
{   // "0" bis "9" und "A" bis "F" und "a" bis "f"
    return (((n >= 48) && (n <= 57)) || (((n >= 65) && (n <= 70)) || ((n >= 97) && (n <= 102))));
}

String hex (uint64 v)
{
  String hexStr = "";
  if (v == 0)
  {
    hexStr = "0";
  }
  else
  {
    while (v > 0)
    {
      byte digit = v & 0xF;
      v = v / 16;
      if (digit < 10)
      {
        hexStr = chr(digit + 48) + hexStr;
      }
      else
      {
        hexStr = chr(digit + 55) + hexStr;
      }
    }
  }
  return hexStr;
}

String hexbyte (byte v)
{
  return right("0" + hex(v), 2);
}

String hexword (uint16 v)
{
  return right("000" + hex(v), 4);
}

String hexlong (ulong v)
{
  return right("0000000" + hex(v), 8);
}

String hexint64 (uint64 v)
{
  return right("00000000000000" + hex(v), 16);
}

uint64 dec (String s)
{
  uint64 v = 0;
  byte l = s.length();
  if (l > 0)
  {
    for (int i = 0; i < l; i++)
    {
        byte b = s[i];
        if ((b >= 48) && (b <= 57))
        {
          v = (v * 16) + b - 48;
        }
        else if ((b >= 65) && (b <= 70)) // Großbuchstaben
        {
          v = (v * 16) + b - 55;
        }
        else if ((b >= 97) && (b <=102)) // Kleinbuchstaben
        {
          v = (v * 16) + b - 87;
        }
        else
        {
          i = l;
        }
    }
  }
  return v;
}

String newuniqueid ()
{
    String ID = "";
    for (byte i = 0; i <= 15; i++)
    {
        ID += hexbyte(rndbyte());
    }
    return (ID);
}


// *** Konvertierungs-Funktionen ***

String cleanasc (String s)
{
  String t = "";
  s.replace("ä", "ae");
  s.replace("ö", "oe");
  s.replace("ü", "ue");
  s.replace("ß", "ss");
  s.replace("Ä", "Ae");
  s.replace("Ö", "Oe");
  s.replace("Ü", "Ue");
  s.replace("€", "EUR");
  s.replace("•", ".");
  s.replace("¢", "Cent");
  s.replace("£", "Pfund");
  s.replace("¥", "Yen");
  s.replace("¦", "|");
  s.replace("§", "Paragraph");
  s.replace("©", "(c)");
  s.replace("«", "<<");
  s.replace("®", "(R)");
  s.replace("°", "Grad");
  s.replace("´", "'");
  s.replace("µ", "u");
  s.replace("·", ".");
  s.replace("»", ">>");
  s.replace("¼", "1/4");
  s.replace("½", "1/2");
  s.replace("¾", "3/4");
  s.replace("Æ", "AE");
  s.replace("×", "x");
  s.replace("æ", "ae");
  s.replace("–", "-");
  s.replace("—", "-");
  int L = s.length();
  if (L > 0)
  {
    for (int i = 0; i < L; i++)
    {
      int c = s.charAt(i);                                          // Character Code an der Position
      switch (c)
      {
        case 228: // "ä"
          t += "ae";
          break;
        case 246: // "ö"
          t += "oe";
          break;
        case 252: // "ü"
          t += "ue";
          break;
        case 223: // "ß"
          t += "ss";
          break;
        case 196: // "Ä"
          t += "Aae";
          break;
        case 214: // "Ö"
          t += "Oe";
          break;
        case 220: // "Ü"
          t += "Ue";
          break;
        case 128: // "€"
          t += "EUR";
          break;
        case 149: // "•"
          t += ".";
          break;
        case 160: // "␣"
          t += " ";
          break;
        case 162: // "¢"
          t += "Cent";
          break;
        case 163: // "£"
          t += "Pfund";
          break;
        case 165: // "¥"
          t += "Yen";
          break;
        case 166: // "¦"
          t += "|";
          break;
        case 167: // "§"
          t += "Paragraph";
          break;
        case 169: // "©"
          t += "(c)";
          break;
        case 171: // "«"
          t += "<<";
          break;
        case 174: // "®"
          t += "(R)";
          break;
        case 176: // "°"
          t += "Grad";
          break;
        case 180: // "´"
          t += "'";
          break;
        case 181: // "µ"
          t += "u";
          break;
        case 183: // "·"
          t += ".";
          break;
        case 187: // "»"
          t += ">>";
          break;
        case 188: // "¼"
          t += "1/4";
          break;
        case 189: // "½"
          t += "1/2";
          break;
        case 190: // "¾"
          t += "3/4";
          break;
        case 198: // "Æ"
          t += "AE";
          break;
        case 215: // "×"
          t += "x";
          break;
        case 230: // "æ"
          t += "ae";
          break;
        case 8211: // "–"
          t += "-";
          break;
        case 8212: // "—"
          t += "-";
          break;
        default:
          if (c < 32)
          {
            t += "Ctrl-" + chr(c + 64);
          }
          else if (c < 128)
          {
            t += chr(c);
          }
          else
          {
            t += "{" + str(c) + "}";
          }
          break;
      }
    }
  }
  return (t);
}

String cleanhtml (String s)
{
  s.replace(" ", "&nbsp;");
  s.replace("'", "&apos;");
  s.replace("ä", "&auml;");
  s.replace("ö", "&ouml;");
  s.replace("ü", "&uuml;");
  s.replace("ß", "&szlig;");
  s.replace("Ä", "&Auml;");
  s.replace("Ö", "&Ouml;");
  s.replace("Ü", "&Uuml;");
  s.replace("€", "&euro;");
  s.replace("•", "&bull;");
  s.replace(" ", "&nbsp;");
  s.replace("¢", "&cent;");
  s.replace("£", "&pound;");
  s.replace("¥", "&yen;");
  s.replace("¦", "&sect;");
  s.replace("§", "&xxx;");
  s.replace("©", "&copy;");
  s.replace("«", "&laquo;");
  s.replace("®", "&reg;");
  s.replace("°", "&deg;");
  s.replace("´", "&acute;");
  s.replace("µ", "&micro;");
  s.replace("·", "&middot;");
  s.replace("»", "&raquo;");
  s.replace("¼", "&frac14;");
  s.replace("½", "&frac12;");
  s.replace("¾", "&frac34;");
  s.replace("Æ", "&AElig;");
  s.replace("×", "&times;");
  s.replace("æ", "&aelig;");
  s.replace("–", "&ndash;");
  s.replace("—", "&mdash;");
  return (s);
}

String convertescape (String s)
{
  String result = s;
  int p = 1;
  while (p > 0)
  {
    p = position(result, chr(92), p);
    if (p > 0)
    {
      if (indasc(result, p + 1) == 117)   // "u"
      {
        String h = mid(result, p + 2, 4); // 4 Zeichen Hex-Code
        int asc = dec(h);                 // Entsprechender Caracter-Code
        result = left(result, p - 1) + chr(asc) + part(result, p + 6);
      }
      else
      {
        p += 1;                           // Einzelnen Backslash ignorieren
      }
    }
  }
  return (result);
}

String removespaces (String s)
{
  bool quotationmark = false;
  int index = 0;
  int L = s.length();
  String result = "";
  for (int i = 0; i < L; i++)
  {
    byte b = s[i];
    if (b == 34)
    {
      quotationmark = not(quotationmark);
    }
    if ((quotationmark) || (b > 32))
    {
      result += " ";
      result[index] = b;
      index++;
    }
  }
  return (result);
}

String quotate (String s)
{
  return chr(34) + s + chr(34);
}

String converttobase64 (String s)
{
  int str_len = s.length() + 1;
  char input[str_len];
  s.toCharArray(input, str_len);
  int inputLen = sizeof(input) - 1;
  int encodedLen = b64_EncodedLength(inputLen);
  char encoded[encodedLen];
  b64_Encode(encoded, input, inputLen);
  return (encoded);
}


// *** Datum & Uhrzeit Funktionen ***


byte monthdays (byte month, int year) // Tage im angegebenen Monat
{
  byte result = 0;
  if ((month >= 1) && (month <= 12) && (year >= 1))
  {
    result = 31 - boolval(month == 9 || month == 4 || month == 6 || month == 11) - 3 * boolval(month == 2);
    result = result + boolval(((year % 4) == 0) && (((year % 100) != 0) || ((year % 400) == 0)) && (month == 2));
  }
  return result;
}

long timetoseconds (int days, byte hour, byte minute, byte second)
{
  return (long)86400 * days + (long)3600 * hour + (long)60 * minute + (long)second;
}

long datetodays2000 (int year, byte month, byte day) // Tage seit dem 01.01.2000
{
  year = year - 2000;
  long days = day;
  for (byte i = 1; i < month; ++i)
  {
    days = days + monthdays(i, year);
  }
  if (month > 2 && year % 4 == 0)
  {
    ++days;
  }
  return days + 365 * year + (year + 3) / 4 - 1;
}

byte dayofweek (int year, byte month, byte day) // Wochentag: 0 = Sonntag, 1 = Montag usw.
{
  int K = int(0.6 + (1.0 / double(month)));
  int L = year - K;
  int Z = int(13 * (12 * K + (int)month + 1) / 5) + int(double(L) * 1.25) - int(L / 100) + int(L / 400) + day - 1;
  return (Z % 7);
}

String dayname (int day)
{
  switch (day)
  {
    case 0:
      return ("Sonntag");
      break;
    case 1:
      return ("Montag");
      break;
    case 2:
      return ("Dienstag");
      break;
    case 3:
      return ("Mittwoch");
      break;
    case 4:
      return ("Donnerstag");
      break;
    case 5:
      return ("Freitag");
      break;
    case 6:
      return ("Samstag");
      break;
    case 7:
      return ("Sonntag");
      break;
    default:
      return ("");
      break;
  }
}

String monthname (int month)
{
  switch (month)
  {
    case 1:
      return ("Januar");
      break;
    case 2:
      return ("Februar");
      break;
    case 3:
      return ("März");
      break;
    case 4:
      return ("April");
      break;
    case 5:
      return ("Mai");
      break;
    case 6:
      return ("Juni");
      break;
    case 7:
      return ("Juli");
      break;
    case 8:
      return ("August");
      break;
    case 9:
      return ("September");
      break;
    case 10:
      return ("Oktober");
      break;
    case 11:
      return ("November");
      break;
    case 12:
      return ("Dezember");
      break;
    default:
      return ("");
      break;
  }
}

byte monthnumber (String month)
{
  byte result = 0;
  month = ucase(month) + "   ";
  byte c0 = month[0];
  byte c1 = month[1];
  byte c2 = month[2];
  if (c0 == 74) // "J"
  {
    if (c1 == 65) // "A"
    {
      result = 1; // Jan
    }
    else if (c2 == 78) // "N"
    {
      result = 6; // Jun
    }
    else if (c2 == 76) // "L"
    {
      result = 7; // Jul
    }
  }
  else if (c0 == 70) // "F"
  {
      result = 2; // Feb
  }
  else if (c0 == 77) // "M"
  {
    if ((c2 == 73) || (c2 == 89)) // "I" oder "Y"
    {
      result = 5; // Mai, May
    }
    else
    {
      result = 3; // Mär, Mar
    }
  }
  else if (c0 == 65) // "A"
  {
    if (c1 == 80) // "P"
    {
      result = 4; // Apr
    }
    else
    {
      result = 8; // Aug
    }
  }
  else if (c0 == 83) // "S"
  {
      result = 9; // Sep
  }
  else if (c0 == 79) // "O"
  {
      result = 10; // Okt, Oct
  }
  else if (c0 == 78) // "N"
  {
      result = 11; // Nov
  }
  else if (c0 == 68) // "D"
  {
      result = 12; // Dez, Dec
  }
  return result;
}

String datestr (t_DateTime DateTime)
{
  byte day = DateTime.day;
  byte month = DateTime.month;
  int year = DateTime.year;
  if ((day >= 1) && (day <= monthdays(month, year))) // Day is valid
  {
    return right("0" + String(day), 2) + "."  + right("0" + String(month), 2) + "." + String(year);
  }
  else
  {
    return "";
  }
}

String timestr (t_DateTime DateTime)
{
  byte hour = DateTime.hour;
  byte minute = DateTime.minute;
  byte second = DateTime.second;
  if ((hour >= 0) && (hour <= 23) && (minute >= 0) && (minute <= 59) && (second >= 0) && (second <= 59)) // Time is valid
  {
    return right("0" + String(hour), 2) + ":"  + right("0" + String(minute), 2) + ":" + right("0" + String(second), 2);
  }
  else
  {
    return "";
  }
}
