// Terminal Library
// 1.00 - 2017-05-23
// 1.01 - 2017-07-23
// 1.02 - 2017-11-10
// 1.03 - 2018-02-06
// (c) Copyright 2017-2018
//
// Zur Verwendung mit allen Allnet Libraries

#if defined(__SAM3X8E__)                        // Arduino Due
  extern "C" char* sbrk(int incr);
#endif

// *** Konstanten für indizierte 8 Farben ***

const byte t_RGBColor8ValueRed    [ ] = {0x00, 0xFF, 0xFF, 0x00, 0x00, 0xFF, 0x00, 0xFF};
const byte t_RGBColor8ValueGreen  [ ] = {0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF};
const byte t_RGBColor8ValueBlue   [ ] = {0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0xFF, 0xFF};


// *** Konstanten für indizierte 16 Farben ***

const byte t_RGBColor16ValueRed   [ ] = {0x00, 0xFF, 0x00, 0xFF, 0x00, 0x54, 0x00, 0x00,
                                         0x80, 0xFF, 0xA8, 0xFF, 0x00, 0xFF, 0x00, 0xFF};
const byte t_RGBColor16ValueGreen [ ] = {0x00, 0x00, 0x00, 0x00, 0x80, 0x54, 0x00, 0x80,
                                         0x40, 0x80, 0xA8, 0x80, 0xFF, 0xFF, 0xFF, 0xFF};
const byte t_RGBColor16ValueBlue  [ ] = {0x00, 0x00, 0x80, 0xFF, 0x00, 0x54, 0xFF, 0xFF,
                                         0x00, 0x00, 0xA8, 0x80, 0x00, 0x00, 0xFF, 0xFF};

// *** Allgemeine Konstanten ***

const byte t_ArduinoUnknown           =  0;     // Microcontroller nicht identifiziert
const byte t_ArduinoMega168           =  1;     // AT Mega 168 Microcontroller
const byte t_ArduinoMega1280          =  2;     // Arduino Mega 1280
const byte t_ATmega1284P              =  3;     // AT Mega 1284P Microcontroller
const byte t_ArduinoMega2560          =  4;     // Arduino Mega 2560
const byte t_ArduinoNano              =  5;     // Arduino Nano
const byte t_ArduinoMicro             =  6;     // Arduino Micro
const byte t_ArduinoDue               =  7;     // Arduino Due
const byte t_Teensy30                 =  8;     // Teensy 3.0
const byte t_Teensy31                 =  9;     // Teensy 3.1
const byte t_ESP8266                  = 10;     // ESP 8266 (IoT-Brick 8266)
const byte t_ESP32                    = 11;     // ESP 32   (IoT-Brick 32)

const byte t_Input_String             = 0;      // Modus für t_TerminalInput: String-Eingabe
const byte t_Input_Integer            = 1;      // Modus für t_TerminalInput: Integer-Eingabe
const byte t_Input_Real               = 2;      // Modus für t_TerminalInput: Real-Eingabe


// *** Variablen ***

byte       t_ResetSequence            = 0;
String     t_TerminalType             = "";      // Terminaltyp als String ("TTY", "UniTerminal" usw.)
String     t_TerminalRespond          = "";      // Die Sequenz, die beim Initialisieren vom Terminal kommt
byte       t_TerminalWidth            = 0;       // Anzahl der Spalten, normalerweise 80
byte       t_TerminalHeight           = 0;       // Anzahl der Zeilen, normalerweise 25
bool       t_TerminalInputOK          = false;   // true = Eingabe wurde mit [Return] abgeschlossen

// *** Forward-Deklarationen ***

String t_TerminalGetKey ();

// *** Utilities ***

byte t_CPUtype ()
{
  #if defined(__AVR_ATmega1280__)                // Arduino Mega 1280 (8 Bit, 128 kB Flash, 8 kB SRAM)
    return t_ArduinoMega1280;
  #elif defined(__AVR_ATmega2560__)              // Arduino Mega 2560 (8 Bit, 256 kB Flash, 8 kB SRAM)
    return t_ArduinoMega2560;
  #elif defined(__AVR_ATmega328P__)              // Arduino Nano, Mini und Uno (8 Bit, 32 kB Flash, 2 kB SRAM)
    return t_ArduinoNano;
  #elif defined(__AVR_ATmega32U4__)              // Arduino Leonardo, Micro (8 Bit, 32 kB Flash, 2,5 kB SRAM)
    return t_ArduinoMicro;
  #elif defined(__AVR_ATmega168__)               // Alte Aruionos (8 Bit, 16 kB Flash, 1 kB SRAM)
    return t_ArduinoMega168;
  #elif defined(__AVR_ATmega1284P__)
    return t_ATmega1284P;
  #elif defined(__SAM3X8E__)                     // Arduino Due (32 Bit,   512 kB Flash, 96 kB SRAM)
    return t_ArduinoDue;
  #elif defined(__MK20DX128__)                   // Teensy 3.0  (32 Bit,   128 kB Flash, 16 kB SRAM)
    return t_Teensy30;
  #elif defined(__MK20DX256__)                   // Teensy 3.1  (32 Bit,   256 kB Flash, 64 kB SRAM)
    return t_Teensy31;
  #elif defined(ESP8266)                         // ESP 8266    (32 Bit,  4096 kB Flash, 96 kB SRAM)
    return t_ESP8266;
  #elif defined(ESP32)                           // ESP 32      (32 Bit, 16384 kB Flash, 520 kB SRAM)
    return t_ESP32;
  #else
    return t_ArduinoUnknown;
  #endif
}

String t_CPUname ()
{
  #if defined(__AVR_ATmega1280__)                // Arduino Mega 1280 (8 Bit, 128 kB Flash, 8 kB SRAM)
    return "Arduino Mega 1280";
  #elif defined(__AVR_ATmega2560__)              // Arduino Mega 2560 (8 Bit, 256 kB Flash, 8 kB SRAM)
    return "Arduino Mega 2560";
  #elif defined(__AVR_ATmega328P__)              // Arduino Nano, Mini und Uno (8 Bit, 32 kB Flash, 2 kB SRAM)
    return "Arduino Nano";
  #elif defined(__AVR_ATmega32U4__)              // Arduino Leonardo, Micro (8 Bit, 32 kB Flash, 2,5 kB SRAM)
    return "Arduino Micro";
  #elif defined(__AVR_ATmega168__)               // Alte Aruionos (8 Bit, 16 kB Flash, 1 kB SRAM)
    return "Arduino Mega 168";
  #elif defined(__AVR_ATmega1284P__)
    return "Arduino Mega 1284";
  #elif defined(__SAM3X8E__)                     // Arduino Due (32 Bit, 512 kB Flash, 96 kB SRAM)
    return "Arduino Due";
  #elif defined(__MK20DX128__)                   // Teensy 3.0  (32 Bit, 128 kB Flash, 16 kB SRAM)
    return "Teensy 3.0";
  #elif defined(__MK20DX256__)                   // Teensy 3.1  (32 Bit, 256 kB Flash, 64 kB SRAM)
    return "Teensy 3.1";
  #elif defined(ESP8266)                         // ESP 8266    (32 Bit, 4096 kB Flash, 96 kB SRAM)
    return "ESP 8266";
  #elif defined(ESP32)                           // ESP 32      (32 Bit, 16384 kB Flash, 520 kB SRAM)
    return "ESP 32";
  #else
    return "(Unknown)";
  #endif
}

byte t_CPUbits ()
{
  #if defined(__AVR_ATmega1280__)                // Arduino Mega 1280 (8 Bit, 128 kB Flash, 8 kB SRAM)
    return 8;
  #elif defined(__AVR_ATmega2560__)              // Arduino Mega 2560 (8 Bit, 256 kB Flash, 8 kB SRAM)
    return 8;
  #elif defined(__AVR_ATmega328P__)              // Arduino Nano, Mini und Uno (8 Bit, 32 kB Flash, 2 kB SRAM)
    return 8;
  #elif defined(__AVR_ATmega32U4__)              // Arduino Leonardo, Micro (8 Bit, 32 kB Flash, 2,5 kB SRAM)
    return 8;
  #elif defined(__AVR_ATmega168__)               // Alte Aruionos (8 Bit, 16 kB Flash, 1 kB SRAM)
    return 8;
  #elif defined(__AVR_ATmega1284P__)
    return 8;
  #elif defined(__SAM3X8E__)                     // Arduino Due (32 Bit, 512 kB Flash, 96 kB SRAM)
    return 32;
  #elif defined(__MK20DX128__)                   // Teensy 3.0  (32 Bit, 128 kB Flash, 16 kB SRAM)
    return 32;
  #elif defined(__MK20DX256__)                   // Teensy 3.1  (32 Bit, 256 kB Flash, 64 kB SRAM)
    return 32;
  #elif defined(ESP8266)                         // ESP 8266    (32 Bit, 4096 kB Flash, 96 kB SRAM)
    return 32;
  #elif defined(ESP32)                           // ESP 32      (32 Bit, 16384 kB Flash, 520 kB SRAM)
    return 32;
  #else
    return 0;
  #endif
}

int t_CPUflashRAM ()
{
  #if defined(__AVR_ATmega1280__)                // Arduino Mega 1280 (8 Bit, 128 kB Flash, 8 kB SRAM)
    return 128;
  #elif defined(__AVR_ATmega2560__)              // Arduino Mega 2560 (8 Bit, 256 kB Flash, 8 kB SRAM)
    return 256;
  #elif defined(__AVR_ATmega328P__)              // Arduino Nano, Mini und Uno (8 Bit, 32 kB Flash, 2 kB SRAM)
    return 32;
  #elif defined(__AVR_ATmega32U4__)              // Arduino Leonardo, Micro (8 Bit, 32 kB Flash, 2,5 kB SRAM)
    return 32;
  #elif defined(__AVR_ATmega168__)               // Alte Aruionos (8 Bit, 16 kB Flash, 1 kB SRAM)
    return 16;
  #elif defined(__AVR_ATmega1284P__)
    return 16;
  #elif defined(__SAM3X8E__)                     // Arduino Due (32 Bit,   512 kB Flash,  96 kB SRAM)
    return 512;
  #elif defined(__MK20DX128__)                   // Teensy 3.0  (32 Bit,   128 kB Flash,  16 kB SRAM)
    return 128;
  #elif defined(__MK20DX256__)                   // Teensy 3.1  (32 Bit,   256 kB Flash,  64 kB SRAM)
    return 256;
  #elif defined(ESP8266)                         // ESP 8266    (32 Bit,  4096 kB Flash,  96 kB SRAM)
    return 4096;
  #elif defined(ESP32)                           // ESP 32      (32 Bit, 16384 kB Flash, 520 kB SRAM)
    return 16384;
  #else
    return 0;
  #endif
}

int t_CPUstaticRAM ()
{
  #if defined(__AVR_ATmega1280__)                // Arduino Mega 1280 (8 Bit, 128 kB Flash, 8 kB SRAM)
    return 8;
  #elif defined(__AVR_ATmega2560__)              // Arduino Mega 2560 (8 Bit, 256 kB Flash, 8 kB SRAM)
    return 8;
  #elif defined(__AVR_ATmega328P__)              // Arduino Nano, Mini und Uno (8 Bit, 32 kB Flash, 2 kB SRAM)
    return 2;
  #elif defined(__AVR_ATmega32U4__)              // Arduino Leonardo, Micro (8 Bit, 32 kB Flash, 2,5 kB SRAM)
    return 3;
  #elif defined(__AVR_ATmega168__)               // Alte Aruionos (8 Bit, 16 kB Flash, 1 kB SRAM)
    return 1;
  #elif defined(__AVR_ATmega1284P__)
    return 1;
  #elif defined(__SAM3X8E__)                     // Arduino Due (32 Bit,   512 kB Flash,  96 kB SRAM)
    return 96;
  #elif defined(__MK20DX128__)                   // Teensy 3.0  (32 Bit,   128 kB Flash,  16 kB SRAM)
    return 16;
  #elif defined(__MK20DX256__)                   // Teensy 3.1  (32 Bit,   256 kB Flash,  64 kB SRAM)
    return 64;
  #elif defined(ESP8266)                         // ESP 8266    (32 Bit,  4096 kB Flash,  96 kB SRAM)
    return 96;
  #elif defined(ESP32)                           // ESP 32      (32 Bit, 16384 kB Flash, 520 kB SRAM)
    return 520;
  #else
    return 0;
  #endif
}

long t_MemAvail ()
{
  #if defined(ESP32) || defined(ESP8266)         // ESP 8266 & ESP 32
    return (system_get_free_heap_size());
  #elif defined(__MK20DX128__) || defined(__MK20DX256__) // Teensy 3.0 & Teensy 3.1
    extern unsigned long _estack; 
    uint32_t heapTop;
    void* foo = malloc(1);
    heapTop = (uint32_t) foo;
    free(foo);
    return (unsigned long)&_estack - heapTop;
  #elif defined(__SAM3X8E__)                     // Arduino Due
    char top;
    return (&top - reinterpret_cast<char*>(sbrk(0)));
  #else                                          // 8 Bit CPU
    extern int __heap_start, *__brkval; 
    byte v; 
    return (long) &v - (__brkval == 0 ? (long) &__heap_start : (long) __brkval); 
  #endif
}

void t_Restart ()
{
  #if defined(ESP32)                             // ESP 32
    ESP.restart();                               // Sauberer Neustart
  #elif defined(ESP8266)                         // ESP 8266
    ESP.wdtFeed();                               // Watchdog Timer zurücksetzen
    ESP.wdtEnable(1000);                         // Watchdog Timer einschalten, 1 Sekunde
    ESP.restart();                               // Sauberer Neustart
    ESP.reset();                                 // Als wenn man den Reset-Button betätigt
    while (true)                                 // Funktioniert leider erst nach einem manuellen Reset nach dem Flashen
    {
      ESP.wdtFeed();                             // Watchdog Timer zurücksetzen
    }
  #elif defined(__MK20DX128__) || defined(__MK20DX256__) || defined(__SAM3X8E__) // 32 Bit CPUs Teensy/Due
    #define RESTART_ADDR       0xE000ED0C
    #define READ_RESTART()     (*(volatile uint32_t *)RESTART_ADDR)
    #define WRITE_RESTART(val) ((*(volatile uint32_t *)RESTART_ADDR) = (val))
    WRITE_RESTART(0x5FA0004);
  #else                                          // 8 Bit CPU
    asm volatile ("  jmp 0");
  #endif
  delay(1000);                                   // 1 Sekunde warten
}

void t_ShutDown ()
{
  while (true)
  {
    t_TerminalGetKey();                          // Reset-Anforderung prüfen
    delay(100); // 100 ms. warten
    #if defined(ESP8266)                         // ESP 8266
      ESP.wdtFeed();                             // Watchdog Timer zurücksetzen
      ESP.wdtDisable();                          // Watchdog Timer ausschalten
    #endif
  }
}

t_DateTime t_CompileDateTime ()
{
  const char* compile_date = __DATE__;
  const char* compile_time = __TIME__;
  String c_date = "";
  String c_time = "";
  for (byte i = 0; i <= 10; i++)
  {
    c_date += compile_date[i];
  }
  for (byte i = 0; i <= 7; i++)
  {
    c_time += compile_time[i];
  }
  String d = mid(c_date, 5, 2);
  if (indasc(d, 1) == 32)
  {
    d = part(d, 2);
  }
  t_DateTime dt;
  dt.year = val(part(c_date, 8));
  dt.month = monthnumber(c_date);
  dt.day = val(d);
  dt.hour = val(left(c_time, 2));
  dt.minute = val(mid(c_time, 4, 2));
  dt.second = val(part(c_time, 7));
  return dt;
}


// *** Terminal USB Serial Steuerzeichen ***

String t_TerminalCursorHome ()
{
  return (chr(2)); // Cursor Home
}

String t_TerminalClearScreen ()
{
  return (chr(12)); // Clear Screen
}

String t_TerminalClearEndOfLine ()
{
  return (chr(27) + "E"); // Clear End Of Line
}

String t_TerminalClearEndOfScreen ()
{
  return (chr(27) + "F"); // Clear End Of Screen
}

String t_TerminalTab ()
{
  return (chr(9)); // Tabulator
}

String t_TerminalGotoYX (int Y, int X)
{
  if (X < 1)
  {
    X = 1;
  }
  if (Y < 1)
  {
    Y = 1;
  }
  if (X > 96)
  {
    X = 96;
  }
  if (Y > 96)
  {
    Y = 96;
  }
  X = X + 31;
  Y = Y + 31;
  String result = chr(27) + "=" + chr(Y) + chr(X);
  return result;
}

String t_TerminalHtab (int X)
{
  if (X < 1)
  {
    X = 1;
  }
  if (X > 96)
  {
    X = 96;
  }
  String result = chr(1) + repeatasc(21, X - 1);
  return result;
}


// *** Terminal USB Serial ***

int t_TerminalPrint (String s)
{
  int bytesSent = 0;
  if (s != "")
  {
    bytesSent = Serial.print(s);
  }
  return (bytesSent);
}

int t_TerminalPrintLn (String s)
{
  return t_TerminalPrint(s + chr(13));
}

String t_TerminalGetKey ()
{
  String s = "";
  if (Serial.available() > 0)
  {
    char c = Serial.read(); // Read a character
    s += c;
    if (c == 31)
    {
      t_ResetSequence++;
      if (t_ResetSequence == 10)
      {
        t_Restart();
      }
    }
    else
    {
      t_ResetSequence = 0;
    }
  }
  return (s);
}

String t_TerminalRead ()
{
  String s = "";
  while (Serial.available() > 0)
  {
    s += t_TerminalGetKey();
  }
  return (s);
}

String t_TerminalGetChar ()
{
  String s = "";
  while (s == "")
  {
    #if defined(ESP8266)                         // ESP 8266
      ESP.wdtFeed();                             // Watchdog Timer zurücksetzen
    #endif
    s += t_TerminalGetKey();
  }
  return (s);
}

int t_TerminalGetAsc ()
{
  int a = asc(t_TerminalGetChar());
  return (a);
}

String t_TerminalInput (String s, String inp, byte maxChars, byte t_param_input_mode)
{
  t_TerminalInputOK = false;
  int    HPos = len(s) + 1;
  String InputStr = left(inp, maxChars);
  int    CursorPos = len(InputStr);
  int    InputLen = 0;
  int    zchn = 0;
  int    a = 0;
  int    t = 0;

  Serial.print(s);
  Serial.print(InputStr);
  while ((zchn != 13) && (zchn != 27))
  {
    #if defined(ESP8266)                         // ESP 8266
      ESP.wdtFeed();                             // Watchdog Timer zurücksetzen
    #endif
    InputLen = len(InputStr);
    zchn = asc(t_TerminalGetChar());
 
    if ((zchn < 32) || (zchn == 127))
    {
      switch (zchn)
      {
        case 1:  // Ctrl-A
          CursorPos = 0;
          Serial.print(t_TerminalHtab(HPos));
          break;
        case 2:  // Ctrl-B
          CursorPos = 0;
          Serial.print(t_TerminalHtab(HPos));
          break;
        case 3:  // Ctrl-C
          CursorPos = len(InputStr);
          Serial.print(t_TerminalHtab(HPos + CursorPos));
          break;
        case 4:  // Ctrl-D
          if (CursorPos < len(InputStr))
          {
            CursorPos = CursorPos + 1;
            while ((CursorPos < len(InputStr)) && (indasc(InputStr, CursorPos) != 32))
            {
              CursorPos = CursorPos + 1;
            }
            Serial.print(t_TerminalHtab(HPos + CursorPos));
          }
          break;
        case 5:  // Ctrl-E
          CursorPos = len(InputStr);
          Serial.print(t_TerminalHtab(HPos + CursorPos));
          break;
        case 6:  // Ctrl-F
          CursorPos = len(InputStr);
          Serial.print(t_TerminalHtab(HPos + CursorPos));
          break;
        case 7:  // Ctrl-G
          InputStr = deleteasc(InputStr, CursorPos + 1);
          Serial.print(t_TerminalHtab(HPos));
          Serial.print(InputStr);
          Serial.print(" ");
          Serial.print(t_TerminalHtab(HPos + CursorPos));
          break;
        case 8:  // Ctrl-H
          CursorPos = CursorPos - 1;
          if (CursorPos < 0)
          {
            CursorPos = 0;
          }
          Serial.print(t_TerminalHtab(HPos + CursorPos));
          break;
        case 9:  // Ctrl-I
          break;
        case 10: // Ctrl-J
          CursorPos = len(InputStr);
          Serial.print(t_TerminalHtab(HPos + CursorPos));
          break;
        case 11: // Ctrl-K
          CursorPos = 0;
          Serial.print(t_TerminalHtab(HPos + CursorPos));
          break;
        case 12: // Ctrl-L
          break;
        case 13: // Ctrl-M
          t_TerminalInputOK = true;
          break;
        case 14: // Ctrl-N
          break;
        case 15: // Ctrl-O
          break;
        case 16: // Ctrl-P
          break;
        case 17: // Ctrl-Q
          break;
        case 18: // Ctrl-R
          CursorPos = 0;
          Serial.print(t_TerminalHtab(HPos + CursorPos));
          break;
        case 19: // Ctrl-S
          if (CursorPos > 0)
          {
            CursorPos = CursorPos - 1;
            while ((CursorPos > 0) && (indasc(InputStr, CursorPos) != asc(" ")))
            {
              CursorPos = CursorPos - 1;
            }
            Serial.print(t_TerminalHtab(HPos + CursorPos));
          }
          break;
        case 20: // Ctrl-T
          break;
        case 21: // Ctrl-U
            CursorPos = CursorPos + 1;
          if (CursorPos > len(InputStr))
          {
            CursorPos = len(InputStr);
          }
          Serial.print(t_TerminalHtab(HPos + CursorPos));
          break;
        case 22: // Ctrl-V
          break;
        case 23: // Ctrl-W
          break;
        case 24: // Ctrl-X
          InputStr = part(InputStr, CursorPos + 1);
          CursorPos = 0;
          Serial.print(t_TerminalHtab(HPos));
          Serial.print(left(InputStr + spc(InputLen), InputLen));
          Serial.print(t_TerminalHtab(HPos + CursorPos));
          break;
        case 25: // Ctrl-Y
          break;
        case 26: // Ctrl-Z
          InputStr = "";
          CursorPos = 0;
          Serial.print(t_TerminalHtab(HPos));
          Serial.print(spc(InputLen));
          Serial.print(t_TerminalHtab(HPos));
          break;
        case 27: // Ctrl-[
          InputStr = "";
          t_TerminalInputOK = false;
          Serial.print(t_TerminalHtab(HPos));
          Serial.print(spc(InputLen));
          Serial.print(t_TerminalHtab(HPos));
          break;
        case 28: // Ctrl-Backslash
          break;
        case 29: // Ctrl-]
          break;
        case 30: // Ctrl-^
          t = t_TerminalGetAsc();
          if (number(t))
          {
            a = t - 48;
            t = t_TerminalGetAsc();
            if (number(t))
            {
              a = a * 10 + t - 48;
              t = t_TerminalGetAsc();
              if (number(t))
              {
                a = a * 10 + t - 48;
              }
            }
            if ((a >= 32) && (a <= 126))
            {
              InputStr = left(InputStr, CursorPos) + chr(a) + part(InputStr, CursorPos + 1);
              CursorPos = CursorPos + 1;
              Serial.print(t_TerminalHtab(HPos));
              Serial.print(InputStr);
            }
          }
          Serial.print(t_TerminalHtab(HPos + CursorPos));
          break;
        case 31: // Ctrl-_
          break;
        case 127:  // Del
          InputStr = deleteasc(InputStr, CursorPos);
          CursorPos = CursorPos - 1;
          if (CursorPos < 0)
          {
            CursorPos = 0;
          }
          Serial.print(t_TerminalHtab(HPos));
          Serial.print(InputStr + " ");
          Serial.print(t_TerminalHtab(HPos + CursorPos));
          break;
      } // switch
    }
    else // Reguläre Zeichen eingeben
    {
      if ((HPos + InputLen < 80) && (InputLen < maxChars))
      {
        int zeichen = zchn;
        if (zeichen == 194)  // Unicode Steuerzeichen ignorieren
        {
          zeichen = 0;
        }
        switch (t_param_input_mode)
        {
          case 0:  // String
            break;
          case 1:  // Integer
            if (not(number(zeichen)) && (zeichen != 45)) // "-" und Zahlen sind erlaubt
            {
              zeichen = 0;
            }
            if ((zeichen == 45) && (CursorPos != 0)) // "-" nur am Anfang erlaubt
            {
              zeichen = 0;
            }
            break;
          case 2:  // Fließkomma
            if (not(number(zeichen)) && ((zeichen < 44) || (zeichen > 46))) // ",-." und Zahlen sind erlaubt
            {
              zeichen = 0;
            }
            if ((zeichen == 45) && (CursorPos != 0)) // "-" nur am Anfang erlaubt
            {
              zeichen = 0;
            }
            if (zeichen == 46)  // "."
            {
              zeichen = 44;    // ","
            }
            if ((zeichen == 44) && (position(InputStr, ",", 1) > 0)) // Nur ein Komma in der Zahl erlaubt
            {
              zeichen = 0;
            }
        }
        if (zeichen != 0)
        {
          InputStr = left(InputStr, CursorPos) + chr(zeichen) + part(InputStr, CursorPos + 1);
          CursorPos += 1;
          Serial.print(t_TerminalHtab(HPos));
          Serial.print(InputStr);
          Serial.print(t_TerminalHtab(HPos + CursorPos));
        }
      }
    } // Reguläres Zeichen
  } // while Loop
  return (InputStr);
}

String t_TerminalInput (String s, byte maxChars, byte t_param_input_mode)
{
  return (t_TerminalInput(s, "", maxChars, t_param_input_mode));
}

t_DateTime t_TerminalGetDateTime ()
{
  t_DateTime DateTime;
  String initSequence = chr(27) + chr(26); // Esc-Ctrl-Z
  Serial.print(initSequence);
  delay(50); // 50 ms warten
  String s = t_TerminalRead ();
  long count = 0;
  if ((left (s, 1) == chr(9)) && (indchar(s, 12) == chr(9)) && (right(s, 1) == chr(9)))
  {
    DateTime.second = val(mid(s, 19, 2));
    do
    {
      count++;
      Serial.print(initSequence);
      delay(50); // 50 ms warten
      s = t_TerminalRead ();
    }
    while (DateTime.second == val(mid(s, 19, 2)));
    DateTime.day = val(mid(s, 2, 2));
    DateTime.month = val(mid(s, 5, 2));
    DateTime.year = val(mid(s, 8, 4));
    DateTime.hour = val(mid(s, 13, 2));
    DateTime.minute = val(mid(s, 16, 2));
    DateTime.second = val(mid(s, 19, 2));
  }
  return DateTime;
}

void t_TerminalInit ()
{
  String initSequence = "   " + chr(13) + chr(27) + "*" + chr(27) + "Z" + chr(27) + "z";
  Serial.print(initSequence);
  delay(500); // 500 ms warten, bis Init-Sequenz verarbeitet ist und Rückmeldung vorliegt
  String s = t_TerminalRead();
  if ((left (s, 1) == chr(9)) && (right(s, 1) == chr(9)))
  {
    s = mid(s, 2, len(s) - 2);
    int p = position(s, chr(9), 1);
    if (p > 0)
    {
      t_TerminalType = left(s, p - 1);
      s = part(s, p + 2); // Beide Tabulatoren in der Mitte entfernen
      p = position(s,"x", 1);
      if (p > 0)
      {
        t_TerminalWidth = val(left(s, p - 1));
        t_TerminalHeight = val(part(s, p + 1));
        t_TerminalRespond = "";
      }
    }
  }
  else
  {
    t_TerminalRespond = right(t_TerminalRespond, 32) + s;
    if (position(t_TerminalRespond, "ttttt", 1) > 0)
    {
      t_TerminalType = "TTY";
      t_TerminalWidth = 80;
      t_TerminalHeight = 24;
      t_TerminalRespond = "";
    }
    else if (position(t_TerminalRespond, "vvvvv", 1) > 0)
    {
      t_TerminalType = "DEC VT52";
      t_TerminalWidth = 80;
      t_TerminalHeight = 24;
      t_TerminalRespond = "";
    }
  }
}

void t_TerminalWaitInit ()
{
  t_TerminalRespond = "";
  while (t_TerminalWidth == 0)
  {
    t_TerminalInit();
    if (t_TerminalWidth == 0)
    {
      delay(500); // 500 ms. warten
    }
  }
  t_TerminalRespond = "";
}