// i2c Library (Transistor-gepufferter i2c-Bus für ALLNET Sensoren)
// 1.00 - 2018-01-30
// (c) Copyright 2018
//
// Zur Verwendung mit dem Allnet IoT-Brick


// *** Globale Variablen ***

byte                 i2c_DataInPin                     = 12;      // i2c-Data Input GPIO
byte                 i2c_DataOutPin                    = 13;      // i2c-Data Output GPIO
byte                 i2c_ClockPin                      = 14;      // i2c-Clock GPIO
bool                 i2c_Inited                        = false;   // Variable für die Initialisierung der i2c Benutzung

// *** ALLNET i2c Funktionen ***

void i2c_Delay (int milli_seconds)
{
 for (int i = 1; i <= milli_seconds; i++)
 {
    #if defined(ESP8266)
      ESP.wdtFeed();                                              // Watchdog Timer zurücksetzen
    #endif
    delay(1);
  }
}

void i2c_ClockLow ()
{
  //Serial.println("i2c_ClockLow");
  digitalWrite(i2c_ClockPin, HIGH);                               // i2c-Clock GPIO auf 0V schalten (Invertiert)
}

void i2c_ClockHigh ()
{
  //Serial.println("i2c_ClockHigh");
  digitalWrite(i2c_ClockPin, LOW);                                // i2c-Clock GPIO auf 5V schalten (Invertiert)
}

void i2c_DataLow ()
{
  //Serial.println("i2c_DataLow");
  digitalWrite(i2c_DataOutPin, HIGH);                             // i2c-Data Output GPIO auf 0V schalten (Invertiert)
}

void i2c_DataHigh ()
{
  //Serial.println("i2c_DataHigh");
  digitalWrite(i2c_DataOutPin, LOW);                              // i2c-Data Output GPIO auf 5V schalten (Invertiert)
}

byte i2c_DataRead ()
{
  //Serial.println("i2c_DataRead");
  return(digitalRead(i2c_DataInPin));                             // i2c-Data Input auslesen (Nicht invertiert)
}

void i2c_Init ()                                                  // Bus initialisieren = Ruhe am Bus
{
  //Serial.println("i2c_Init");
  pinMode(i2c_ClockPin, OUTPUT);                                  // i2c-Clock GPIO auf Ausgabe schalten
  pinMode(i2c_DataOutPin, OUTPUT);                                // i2c-Data Output auf Ausgabe schalten
  pinMode(i2c_DataInPin, INPUT);                                  // i2c-Data Input auf Ausgabe schalten
  #if defined(ESP8266)
    ESP.wdtFeed();                                                // Watchdog Timer zurücksetzen
  #endif
  i2c_ClockHigh();                                                // Clockleitung High
  i2c_DataHigh();                                                 // Datenleitung High
  i2c_Inited = true;
}

void i2c_SendACK ()
{
  //Serial.println("i2c_SendACK");
  i2c_DataLow();                                                  // Datenleitung low setzen
  i2c_Delay(2);                                                   // einfach nur einen Impuls senden
  i2c_ClockHigh();
  i2c_Delay(2);
  i2c_ClockLow();
  i2c_Delay(2);
  i2c_DataHigh();                                                 // Datenleitung High setzen (freigeben)
  i2c_Delay(2);
}

void i2c_SendNAK ()
{
  //Serial.println("i2c_SendNAK");
  i2c_DataHigh();                                                 // Datenleitung High setzen
  i2c_Delay(2);
  i2c_ClockHigh();                                                // einfach nur einen Impuls senden
  i2c_Delay(2);
  i2c_ClockLow();
}

void i2c_SendBRK ()                                               // Leer "durchklingeln", um evtl. blockerte
{                                                                 // Bausteine wieder freizubekommen
  //Serial.println("i2c_SendBRK");
  i2c_Delay(4);                                                   // Pre-Delay
  i2c_ClockHigh();
  i2c_Delay(4);                                                   // Pre-Delay
  i2c_DataHigh();                                                 // Datenleitung High setzen
  i2c_Delay(4);

  for (int t = 0; t < 18; t++)                                    // Jetzt mehrere Clockimpulse senden
  {
    i2c_ClockLow();
    i2c_Delay(2);
    i2c_ClockHigh();
    i2c_Delay(2);
  }

  i2c_Delay(16);
}

void i2c_SendStop ()
{
  //Serial.println("i2c_SendStop");
  i2c_ClockLow();                                                 // Clockleitung Low setzen (sollte eigentlich Low sein)
  i2c_Delay(4);                                                   // Pre-Delay
  i2c_DataLow();                                                  // Datenleitung Low setzen, falls dies nicht der Fall ist
  i2c_Delay(4);                                                   // Pre-Delay
  i2c_ClockHigh();                                                // Clockleitung High setzen, falls diese nicht High ist
  i2c_Delay(4);                                                   // Pre-Delay
  i2c_DataHigh();                                                 // Datenleitung High setzen
  i2c_Delay(4);                                                   // Post-Delay
}

void i2c_SendStart ()
{
  //Serial.println("i2c_SendStart");
  i2c_ClockHigh();                                                // Clockleitung High setzen, falls diese nicht High ist
  i2c_DataHigh();                                                 // Datenleitung High setzen
  #if defined(ESP8266)
    ESP.wdtFeed();                                                // Watchdog Timer zurücksetzen
  #endif
  i2c_Delay(2);                                                   // Pre-Delay
  i2c_DataLow();                                                  // Datenleitung Low setzen
  i2c_Delay(2);                                                   // Kurz warten
  i2c_ClockLow();                                                 // Clockleitung Low setzen
  i2c_Delay(6);                                                   // Warten, damit die Start Condition ueberall bekannt wird
}

void i2c_SendStartSHT ()                                          // Spezielle Startbedingung für SHT71
{
  //Serial.println("i2c_SendStartSHT");
  i2c_DataHigh();                                                 // Datenleitung High setzen
  IoT_Idle();
  i2c_Delay(2);                                                   // Pre-Delay
  i2c_ClockHigh();                                                // Clockleitung High setzen, falls diese nicht High ist
  i2c_Delay(3);                                                   // Kurz warten
  i2c_DataLow();                                                  // Datenleitung Low setzen
  i2c_Delay(3);                                                   // Kurz warten
  i2c_ClockLow();                                                 // Clockleitung Low setzen
  i2c_Delay(2);                                                   // Kurz warten
  i2c_ClockHigh();                                                // Clockleitung High setzen, falls diese nicht High ist
  i2c_Delay(3);                                                   // Kurz warten
  i2c_DataHigh();                                                 // Datenleitung High setzen
  i2c_Delay(2);                                                   // Kurz warten
  i2c_ClockLow();                                                 // Clockleitung Low setzen
  i2c_Delay(10);                                                  // Warten, damit die Start Condition ueberall bekannt wird
}

byte i2c_WaitForACK ()
{
  //Serial.println("i2c_WaitForACK");
  i2c_DataHigh();                                                 // auf High setzen, um Einlesen zu ermöglichen
  i2c_Delay(4);
  byte result = 0;
  if (i2c_DataRead() != 0)                                        // Bit einlesen
  {
    result = 1;                                                   // NACK
  }
  else
  {
    result = 0;                                                   // ACK
  }
  i2c_ClockHigh();                                                // Clockleitung High setzen
  i2c_Delay(1);                                                   // Kleine Pause
  i2c_ClockLow();                                                 // Clockleitung Low setzen
  i2c_Delay(2);
  return (result);
}

void i2c_SendByteRaw (byte b)
{
  //Serial.println("i2c_SendByteRaw");
  for (int t = 7; t >= 0; t--)                                    // MSB first
  {
    IoT_Idle();
    //Serial.println("i2c_SendByteRaw: Loop " + str(t));
    if ((b & (1 << t)) != 0)                                      // Bit auf den Bus legen: bitand(b, 2 ^ t)
    {
      i2c_DataHigh(); 				                  // pin_high(RE,i);
    }
    else
    {
      i2c_DataLow();                                              // pin_low(RE,i);
    }
    IoT_Idle();
    i2c_Delay(1);                                                 // Kleine Pause
    i2c_ClockHigh();                                              // einen Clock-Impuls senden
    IoT_Idle();
    i2c_Delay(2);
    i2c_ClockLow();
    i2c_Delay(1);                                                 // Kleine Pause
  }
  IoT_Idle();
  i2c_DataLow();                                                  // Datenleitung am Ende der Übetragung immer auf Low setzen
  // i2c_Delay(1);
}


byte i2c_SendByte (byte b, byte ack)
{
  //Serial.println("i2c_SendByte");
  i2c_SendByteRaw(b);                                             // Das übergebene Byte über den Bus senden
  for (int i = 1; i <= 3; i++)                                    // Jetzt mehrere Clockimpulse senden
  {
    IoT_Idle();
    i2c_Delay(2);
  }
  switch (ack)
  {
    case 0:                                                       // 0: NAK senden
      i2c_SendNAK();
      return (78);                                                // "N"
      break;
    case 1:                                                       // 1: ACK senden
      i2c_SendACK();
      return (65);                                                // "A"
      break;
    case 2:                      
      return (i2c_WaitForACK());                                  // 2: ACK/NACK vom Slave abwarten
      break;
    default:
      return (255);                                               // Parameter-Fehler
      break;
  }
}

byte i2c_ReceiveByteRaw ()
{
  byte result = 0;
  for (int t = 7;t >= 0;t--)                                     // MSB first
  {
    i2c_Delay(1);                                                 // Kleine Pause
    i2c_ClockHigh();                                              // einen Clock-Impuls senden
    i2c_Delay(1);                                                 // Kleine Pause
    if (i2c_DataRead() != 0)                                      // Bit einlesen
    {
      result += (1 << t);
    }
    i2c_Delay(1);                                                 // Kleine Pause
    i2c_ClockLow();
    i2c_Delay(1);                                                 // Kleine Pause
  }
  return(result);
}

byte i2c_ReceiveByte (byte ack)
{
  byte result = i2c_ReceiveByteRaw();                             // Das Byte vom Bus holen
  switch (ack)
  {
    case 0:                                                       // 0: NAK senden
      i2c_SendNAK();
      break;
    case 1:                                                       // 1: ACK senden
      i2c_SendACK();
      break;
    default:
      break;
  }
  return (result);
}

